@@ -11,6 +11,7 @@
 
 # bail out on errors, be strict
 set -ue
+FIPSMODE=0
 
 #-------------------------------------------------------------------------------
 #
@@ -167,6 +168,12 @@ do
             exit 3
         fi
         ;;
+    --enable-fipsmode )
+        shift
+        FIPSMODE=1
+        WITH_SSL="OFF"
+        BUILD_COMMENT="${BUILD_COMMENT:-}-pro"
+        ;;
     -t | --tag )
         shift
         TAG="$1"
@@ -276,7 +283,11 @@ fi
 TOKUDB_BACKUP_VERSION="${MYSQL_VERSION}${MYSQL_VERSION_EXTRA}"
 
 RELEASE_TAG=''
-PRODUCT_NAME="Percona-XtraDB-Cluster_$MYSQL_VERSION$MYSQL_VERSION_EXTRA"
+if [[ "x${FIPSMODE}" == "x1" ]]; then
+   PRODUCT_NAME="Percona-XtraDB-Cluster-Pro_$MYSQL_VERSION$MYSQL_VERSION_EXTRA"
+else
+   PRODUCT_NAME="Percona-XtraDB-Cluster_$MYSQL_VERSION$MYSQL_VERSION_EXTRA"
+fi
 PRODUCT_FULL_NAME="${PRODUCT_NAME}.${TAG}_$(uname -s)${DIST_NAME:-}.$MACHINE_SPECS${GLIBC_VER:-}${TARBALL_SUFFIX:-}"
 
 #
@@ -291,10 +302,17 @@ then
 else
     REVISION=""
 fi
-PRODUCT_FULL="Percona-XtraDB-Cluster_${MYSQL_VERSION}${MYSQL_VERSION_EXTRA}"
-PRODUCT_FULL="${PRODUCT_FULL}.${TAG}${BUILD_COMMENT:+_}${BUILD_COMMENT}$(uname -s)${DIST_NAME:-}.$TARGET${GLIBC_VER:-}"
-COMMENT="Percona XtraDB Cluster binary (GPL) $MYSQL_VERSION"
-COMMENT="$COMMENT, Revision $REVISION${BUILD_COMMENT:-}, WSREP version $WSREP_VERSION"
+if [[ "x${FIPSMODE}" == "x1" ]]; then
+    PRODUCT_FULL="Percona-XtraDB-Cluster-Pro_${MYSQL_VERSION}${MYSQL_VERSION_EXTRA}"
+    PRODUCT_FULL="${PRODUCT_FULL}.${TAG}${BUILD_COMMENT:+_}${BUILD_COMMENT}$(uname -s)${DIST_NAME:-}.$TARGET${GLIBC_VER:-}"
+    COMMENT="Percona XtraDB Cluster Pro binary (GPL) $MYSQL_VERSION"
+    COMMENT="$COMMENT, Revision $REVISION${BUILD_COMMENT:-}, WSREP version $WSREP_VERSION"
+else
+    PRODUCT_FULL="Percona-XtraDB-Cluster_${MYSQL_VERSION}${MYSQL_VERSION_EXTRA}"
+    PRODUCT_FULL="${PRODUCT_FULL}.${TAG}${BUILD_COMMENT:+_}${BUILD_COMMENT}$(uname -s)${DIST_NAME:-}.$TARGET${GLIBC_VER:-}"
+    COMMENT="Percona XtraDB Cluster binary (GPL) $MYSQL_VERSION"
+    COMMENT="$COMMENT, Revision $REVISION${BUILD_COMMENT:-}, WSREP version $WSREP_VERSION"
+fi
 
 #-------------------------------------------------------------------------------
 #
@@ -334,6 +352,9 @@ then
 fi
 
 BUILD_PARAMETER=""
+if [[ "x${FIPSMODE}" == "x1" ]]; then
+    BUILD_PARAMETER="-DPROBUILD=1"
+fi
 
 #-------------------------------------------------------------------------------
 #
@@ -430,7 +451,6 @@ BUILD_PARAMETER=""
             -DWITHOUT_TOKUDB=ON \
             -DWITH_INNODB_MEMCACHED=ON \
             -DDOWNLOAD_BOOST=1 \
-            -DWITH_PACKAGE_FLAGS=OFF \
             -DFORCE_INSOURCE_BUILD=1 \
             -DWITH_SYSTEM_LIBS=ON \
             -DWITH_PROTOBUF=bundled \
@@ -456,6 +476,10 @@ BUILD_PARAMETER=""
         (make install) || exit 1
         (cp -v runtime_output_directory/mysqld-debug $TARGETDIR/usr/local/$PRODUCT_FULL_NAME/bin/mysqld) || true
         echo "mysqld in build in debug mode"
+        (
+            echo "Packaging the test files"
+            cp -R $SOURCEDIR/percona-xtradb-cluster-tests $TARGETDIR/usr/local/$PRODUCT_FULL_NAME/
+        ) || exit 1
     else
         cmake $SOURCEDIR/ ${CMAKE_OPTS:-} -DBUILD_CONFIG=mysql_release \
             -DCMAKE_BUILD_TYPE=${CMAKE_BUILD_TYPE:-RelWithDebInfo} \
@@ -470,7 +494,6 @@ BUILD_PARAMETER=""
             -DWITHOUT_TOKUDB=ON \
             -DWITH_INNODB_MEMCACHED=ON \
             -DDOWNLOAD_BOOST=1 \
-            -DWITH_PACKAGE_FLAGS=OFF \
             -DFORCE_INSOURCE_BUILD=1 \
             -DWITH_SYSTEM_LIBS=ON \
             -DWITH_PROTOBUF=bundled \
@@ -487,8 +510,8 @@ BUILD_PARAMETER=""
             -DWITH_BOOST="$TARGETDIR/libboost" \
             -DMYSQL_SERVER_SUFFIX=".$TAG" \
             -DWITH_WSREP=ON \
-            -DWITH_UNIT_TESTS=0 \
             -DWITH_PERCONA_TELEMETRY=ON \
+            -DWITH_UNIT_TESTS=0 \
             ${BUILD_PARAMETER} $WITH_MECAB_OPTION $OPENSSL_INCLUDE $OPENSSL_LIBRARY $CRYPTO_LIBRARY
 
         (make $MAKE_JFLAG $QUIET) || exit 1
@@ -513,13 +536,12 @@ BUILD_PARAMETER=""
     ) || exit 1
     fi
 
-    # Extract the Percona Xtrabackup binaries
-    # Look for the pxb 2.4 tarball
+    # Look for the pxb 8.4 tarball
     (
         cd "$TARGETDIR"
-        pxb_tar=$(ls -1td percona-xtrabackup-2.4.* | grep ".tar" | sort --version-sort | tail -n1)
+        pxb_tar=$(ls -1td percona-xtrabackup-8.4.* | grep ".tar" | sort --version-sort | tail -n1)
         if [[ -z $pxb_tar ]]; then
-            echo "Could not find percona-xtrabackup-2.4 tarball in $TARGETDIR.  Terminating."
+            echo "Could not find percona-xtrabackup-8.4 tarball in $TARGETDIR.  Terminating."
             exit 1
         fi
         # Remove the .tar.gz extension
@@ -528,24 +550,25 @@ BUILD_PARAMETER=""
         if [[ $pxb_basename =~ x86_64 ]]; then
             pxb_basename="${pxb_basename%x86_64*}x86_64"
         fi
-        pxb_dir="pxb-2.4"
+        pxb_dir="pxb-8.4"
 
         mkdir -p pxc_extra
         cd pxc_extra
         if [[ -d ${pxb_basename} ]]; then
-            echo "Using existing pxb 2.4 directory : ${pxb_basename}"
+            echo "Using existing pxb 8.4 directory : ${pxb_basename}"
         else
-            echo "Removing existing percona-xtrabackup-2.4 basedir (if found)"
-            find . -maxdepth 1 -type d -name 'percona-xtrabackup-2.*' -exec rm -rf {} \+
+            echo "Removing existing percona-xtrabackup-8.4 basedir (if found)"
+            find . -maxdepth 1 -type d -name 'percona-xtrabackup-8.*' -exec rm -rf {} \+
 
-            echo "Extracting pxb 2.4 tarball"
+            echo "Extracting pxb 8.4 tarball"
             tar -xzf "../$pxb_tar"
         fi
         echo "Creating symlink $pxb_dir --> $pxb_basename"
-        rm -f pxb-2.4
-        ln -s ./${pxb_basename} pxb-2.4
+        rm -f pxb-8.4
+        ln -s ./${pxb_basename} pxb-8.4
     ) || exit 1
 
+    # Look for the pxb 8.0 tarball
     (
         cd "$TARGETDIR"
         pxb_tar=$(ls -1td percona-xtrabackup-8.0.* | grep ".tar" | sort --version-sort | tail -n1)
@@ -579,9 +602,9 @@ BUILD_PARAMETER=""
 
     # Only copy over the bin and lib portions of the xtrabackup packages
     # Test cases and other files are not copied
-    mkdir -p "$TARGETDIR/usr/local/$PRODUCT_FULL_NAME/bin/pxc_extra/pxb-2.4"
-    (cp -v -r $TARGETDIR/pxc_extra/pxb-2.4/bin/  $TARGETDIR/usr/local/$PRODUCT_FULL_NAME/bin/pxc_extra/pxb-2.4) || true
-    (cp -v -r $TARGETDIR/pxc_extra/pxb-2.4/lib/  $TARGETDIR/usr/local/$PRODUCT_FULL_NAME/bin/pxc_extra/pxb-2.4) || true
+    mkdir -p "$TARGETDIR/usr/local/$PRODUCT_FULL_NAME/bin/pxc_extra/pxb-8.4"
+    (cp -v -r $TARGETDIR/pxc_extra/pxb-8.4/bin/  $TARGETDIR/usr/local/$PRODUCT_FULL_NAME/bin/pxc_extra/pxb-8.4) || true
+    (cp -v -r $TARGETDIR/pxc_extra/pxb-8.4/lib/  $TARGETDIR/usr/local/$PRODUCT_FULL_NAME/bin/pxc_extra/pxb-8.4) || true
 
     mkdir -p "$TARGETDIR/usr/local/$PRODUCT_FULL_NAME/bin/pxc_extra/pxb-8.0"
     (cp -v -r $TARGETDIR/pxc_extra/pxb-8.0/bin/ $TARGETDIR/usr/local/$PRODUCT_FULL_NAME/bin/pxc_extra/pxb-8.0) || true
@@ -591,8 +614,8 @@ BUILD_PARAMETER=""
 
 # Patch needed libraries
 (
-    LIBLIST="libatomic.so libnuma.so libgssapi.so libldap_r-2.4.so.2 liblber-2.4.so.2 libaio.so libprocps.so libreadline.so libtinfo.so libsasl2.so libbrotlidec.so libbrotlicommon.so librtmp.so libfreebl3.so libsmime3.so libnss3.so libnssutil3.so libplds4.so libplc4.so libnspr4.so libtirpc.so libncurses.so.5 libboost_program_options"
-    DIRLIST="bin bin/pxc_extra/pxb-8.0/bin bin/pxc_extra/pxb-2.4/bin lib bin/pxc_extra/pxb-8.0/lib/plugin bin/pxc_extra/pxb-2.4/lib/plugin lib/private lib/plugin lib/mysqlrouter/plugin lib/mysqlrouter/private"
+    LIBLIST="libnuma.so libgssapi.so libldap_r-2.4.so.2 liblber-2.4.so.2 libaio.so libprocps.so libgcrypt.so libtinfo.so libsasl2.so libbrotlidec.so libbrotlicommon.so librtmp.so libfreebl3.so libssl3.so libsmime3.so libnss3.so libnssutil3.so libplds4.so libplc4.so libnspr4.so libtirpc.so libncurses.so.5 libboost_program_options"
+    DIRLIST="bin bin/pxc_extra/pxb-8.0/bin bin/pxc_extra/pxb-8.4/bin lib bin/pxc_extra/pxb-8.0/lib/plugin bin/pxc_extra/pxb-8.4/lib/plugin lib/private lib/plugin lib/mysqlrouter/plugin lib/mysqlrouter/private"
 
     LIBPATH=""
     OVERRIDE=false
@@ -685,11 +708,11 @@ BUILD_PARAMETER=""
         # Set proper runpath
         export override=false
         set_runpath bin '$ORIGIN/../lib/private/'
-        set_runpath bin/pxc_extra/pxb-2.4/bin '$ORIGIN/../../../../lib/private/'
         set_runpath bin/pxc_extra/pxb-8.0/bin '$ORIGIN/../../../../lib/private/'
+        set_runpath bin/pxc_extra/pxb-8.4/bin '$ORIGIN/../../../../lib/private/'
         set_runpath lib '$ORIGIN/private/'
-        set_runpath bin/pxc_extra/pxb-2.4/lib/plugin '$ORIGIN/../../../../../lib/private/'
         set_runpath bin/pxc_extra/pxb-8.0/lib/plugin '$ORIGIN/../../../../../lib/private/'
+        set_runpath bin/pxc_extra/pxb-8.4/lib/plugin '$ORIGIN/../../../../../lib/private/'
         set_runpath lib/plugin '$ORIGIN/../private/'
         set_runpath lib/private '$ORIGIN'
         #  LIBS MYSQLROUTER
@@ -702,6 +725,9 @@ BUILD_PARAMETER=""
         unset override && export override=true && set_runpath bin/mysqlrouter_keyring '$ORIGIN/../lib/mysqlrouter/private/:$ORIGIN/../lib/mysqlrouter/plugin/:$ORIGIN/../lib/private/'
         #  BINS XTRABACKUP
         unset override && export override=true && set_runpath bin/pxc_extra/pxb-8.0/bin/xtrabackup '$ORIGIN/../../../../lib/private/:$ORIGIN/../lib/private/'
+        unset override && export override=true && set_runpath bin/pxc_extra/pxb-8.0/bin/xtrabackup-debug '$ORIGIN/../../../../lib/private/:$ORIGIN/../lib/private/'
+        unset override && export override=true && set_runpath bin/pxc_extra/pxb-8.4/bin/xtrabackup '$ORIGIN/../../../../lib/private/:$ORIGIN/../lib/private/'
+        unset override && export override=true && set_runpath bin/pxc_extra/pxb-8.4/bin/xtrabackup-debug '$ORIGIN/../../../../lib/private/:$ORIGIN/../lib/private/'
 
         # Replace libs
         for DIR in ${DIRLIST}; do
