@@ -21,6 +21,7 @@ Usage: $0 [OPTIONS]
         --rpm_release       RPM version( default = 1)
         --deb_release       DEB version( default = 1)
         --bin_release       BIN version( default = 1)
+        --enable_fipsmode   Build gated PXC
         --debug             Build debug tarball
         --help) usage ;;
 Example $0 --builddir=/tmp/PXC80 --get_sources=1 --build_src_rpm=1 --build_rpm=1
@@ -59,6 +60,7 @@ parse_arguments() {
             --bin_release=*) BIN_RELEASE="$val" ;;
             --no_clone=*) NO_CLONE="$val" ;;
             --debug=*) DEBUG="$val" ;;
+            --enable_fipsmode=*) FIPSMODE="$val" ;;
             --help) usage ;;      
             *)
               if test -n "$pick_args"
@@ -162,10 +164,10 @@ get_sources(){
         popd
         GALERA_REVNO="$(test -r percona-xtradb-cluster-galera/GALERA-REVISION && cat percona-xtradb-cluster-galera/GALERA-REVISION)"
     fi
-    if [ -f VERSION ]; then
-        source VERSION
-    elif [ -f MYSQL_VERSION ]; then
-        source MYSQL_VERSION
+    if [ -f ./VERSION ]; then
+        source ./VERSION
+    elif [ -f ./MYSQL_VERSION ]; then
+        source ./MYSQL_VERSION
     else
         echo "VERSION file does not exist"
        exit 1
@@ -252,9 +254,9 @@ get_sources(){
 
     cd ${WORKDIR} || exit
     #
-    #pushd ${PXCDIR}
-    #    sed -i 's:boostorg\.jfrog\.io/artifactory/main/release/.*/source:jenkins.percona.com/downloads/boost:g' cmake/boost.cmake
-    #popd
+    pushd ${PXCDIR}
+        sed -i 's:boostorg\.jfrog\.io/artifactory/main/release/.*/source:downloads.percona.com/downloads/packaging/boost:g' cmake/boost.cmake
+    popd
     #
     tar --owner=0 --group=0 --exclude=.bzr --exclude=.git -czf ${PXCDIR}.tar.gz ${PXCDIR}
     rm -fr ${PXCDIR}
@@ -305,7 +307,7 @@ install_deps() {
     CURPLACE=$(pwd)
 
     if [ "x$OS" = "xrpm" ]; then
-        if [ "x${RHEL}" = "x7" -o x"$RHEL" = x8 ]; then
+        if [ "x${RHEL}" = "x8" -o "x${RHEL}" = "x7" ]; then
             switch_to_vault_repo
         fi
         RHEL=$(rpm --eval %rhel)
@@ -314,16 +316,14 @@ install_deps() {
         yum install -y perl
         yum install -y https://repo.percona.com/yum/percona-release-latest.noarch.rpm
         percona-release enable tools testing
-        percona-release enable pxb-24 testing
-        if [ "x$RHEL" = "x8" -o "x$RHEL" = "x9" -o "x${RHEL}" = "x10" ]; then
-            yum -y install dnf-plugins-core
-            if [ "x${RHEL}" = "x10" ]; then
-                dnf install https://dl.fedoraproject.org/pub/epel/epel-release-latest-10.noarch.rpm
-            else
-                yum -y install epel-release
-            fi
+	if [ x"$ARCH" = "xaarch64" ]; then
+	    percona-release enable pxb-84-lts testing
+	fi
+        if [ "x$RHEL" = "x8" -o "x$RHEL" = "x9" ]; then
+            yum -y install dnf-plugins-core epel-release
             yum config-manager --set-enabled powertools
 	    yum -y install git
+            yum -y install python2-scons || true
             yum -y install python2-pip python36-devel
             yum -y install autoconf automake binutils bison boost-static cmake gcc gcc-c++ make
             yum -y install gperf glibc glibc-devel jemalloc jemalloc-devel libaio-devel
@@ -342,23 +342,24 @@ install_deps() {
 		dnf config-manager --enable ol${RHEL}_codeready_builder
 		yum -y install gperf rpcgen
 	    fi
+
             if [ "x${RHEL}" = "x9" ]; then
-                yum install -y https://yum.oracle.com/repo/OracleLinux/OL9/distro/builder/${ARCH}/getPackage/procps-ng-devel-3.3.17-8.el9.${ARCH}.rpm
+                yum install -y https://yum.oracle.com/repo/OracleLinux/OL9/distro/builder/${ARCH}/getPackage/procps-ng-devel-3.3.17-8.el9.x86_64.rpm
                 yum -y install dnf-utils
                 dnf config-manager --enable ol9_codeready_builder
                 yum -y install libedit-devel
                 yum -y install libtirpc-devel
                 yum -y install gcc
+                yum -y install gcc-toolset-12-gcc gcc-toolset-12-gcc-c++ gcc-toolset-12-binutils gcc-toolset-12-annobin-annocheck gcc-toolset-12-annobin-plugin-gcc gcc-toolset-12-libatomic-devel
                 yum -y install scons pip python3-devel
                 pip install --user typing pyyaml regex Cheetah3
             else
-            #    wget https://jenkins.percona.com/yum-repo/percona-dev.repo
-            #    mv -vf percona-dev.repo /etc/yum.repos.d
                 wget https://downloads.percona.com/downloads/packaging/python2-scons-3.0.1-9.el8.noarch.rpm
                 yum -y install ./python2-scons-3.0.1-9.el8.noarch.rpm || true
                 yum -y clean all
                 yum -y install libtirpc-devel
-                yum -y install python2-pip python36-devel
+                yum -y install perl-Dig
+                yum -y install python2-scons python2-pip python36-devel
                 yum -y install python2-devel
                 /usr/bin/pip3.6 install --user typing pyyaml regex Cheetah3
                 /usr/bin/pip2.7 install --user typing pyyaml regex Cheetah
@@ -375,9 +376,7 @@ install_deps() {
                 echo "waiting"
                 sleep 1
             done
-            if [ "x${RHEL}" = "x7" -o x"$RHEL" = x8 ]; then
-                switch_to_vault_repo
-            fi
+            switch_to_vault_repo
             yum -y install  gcc-c++ devtoolset-8-gcc-c++ devtoolset-8-binutils
             source /opt/rh/devtoolset-8/enable
             yum -y install scons check-devel boost-devel cmake3
@@ -393,12 +392,12 @@ install_deps() {
 --slave /usr/local/bin/ccmake ccmake /usr/bin/ccmake3 \
 --family cmake
         fi
-        if [ "x${RHEL}" = "x8" ]; then
-            yum -y install centos-release-stream
-            switch_to_vault_repo
-            yum -y install git gcc-toolset-11-gcc gcc-toolset-11-gcc-c++ gcc-toolset-11-annobin-plugin-gcc
-            source /opt/rh/gcc-toolset-11/enable
-        fi
+        #if [ "x${RHEL}" = "x8" ]; then
+        #    yum -y install centos-release-stream
+        #    switch_to_vault_repo
+        #    yum -y install git gcc-toolset-11-gcc gcc-toolset-11-gcc-c++ gcc-toolset-11-annobin-plugin-gcc gcc-toolset-11-libatomic-devel
+        #    source /opt/rh/gcc-toolset-11/enable
+        #fi
         if [ "x${RHEL}" = "x7" ]; then
             yum -y install devtoolset-11
             source /opt/rh/devtoolset-11/enable
@@ -422,29 +421,44 @@ install_deps() {
         fi
         if [ "x$RHEL" = "x8" ]; then
             yum -y install centos-release-stream
-            yum -y install gcc-toolset-11-gcc-c++ gcc-toolset-11-binutils
-            yum -y install gcc-toolset-11-valgrind gcc-toolset-11-valgrind-devel gcc-toolset-11-libatomic-devel
+            switch_to_vault_repo
+            yum -y install git gcc-toolset-11-gcc-c++ gcc-toolset-11-binutils gcc-toolset-11-annobin-plugin-gcc
+            yum -y install gcc-toolset-11-valgrind gcc-toolset-11-valgrind-devel gcc-toolset-11-libatomic-devel gcc-toolset-11-libatomic-devel
             yum -y install gcc-toolset-11-libasan-devel gcc-toolset-11-libubsan-devel
+            source /opt/rh/gcc-toolset-11/enable
             yum -y remove centos-release-stream
         fi
-        if [ "x$RHEL" = "x9" ]; then
-            yum -y install libatomic
-        fi
         yum -y install yum-utils patchelf
         yum -y install cyrus-sasl-devel cyrus-sasl-scram krb5-devel
     else
         apt-get -y update
-        DEBIAN_FRONTEND=noninteractive apt-get -y install curl lsb-release wget apt-transport-https software-properties-common
+        DEBIAN_FRONTEND=noninteractive apt-get -y install curl lsb-release gnupg2 wget apt-transport-https software-properties-common
         apt-get -y install dirmngr || true
         apt-get update
         apt-get -y install dirmngr || true
-        wget https://repo.percona.com/apt/percona-release_latest.$(lsb_release -sc)_all.deb && apt -y install gnupg2 lsb-release ./percona-release_latest.$(lsb_release -sc)_all.deb
-        percona-release enable tools release
-        percona-release enable pxb-80 testing
-        percona-release enable pxb-24 testing
+        wget https://repo.percona.com/apt/percona-release_latest.$(lsb_release -sc)_all.deb && dpkg -i percona-release_latest.$(lsb_release -sc)_all.deb
         export DEBIAN_FRONTEND="noninteractive"
         export DIST="$(lsb_release -sc)"
-            until apt-get update; do
+        percona-release enable tools release
+        
+        # (1) PXB compatible with previous PXC LTS version
+	if [ x"$ARCH" = "xx86_64" ]; then
+            percona-release enable pxb-80 release
+	else
+	    percona-release enable pxb-80 testing
+	fi
+        if [ x"${DIST}" = xnoble ]; then
+            percona-release enable pxb-8x-innovation experimental
+        else
+            percona-release enable pxb-8x-innovation release
+        fi
+        percona-release enable pxb-84-lts testing
+        # (2) PXB compatible with previous PXC version (note: it may be LTS as well)
+        percona-release enable pxc-8x-innovation testing
+        # (3) PXB compatible with this PXC version (LTS or Innovative)
+        percona-release enable pxc-84-lts testing
+        
+        until apt-get update; do
             sleep 1
             echo "waiting"
         done
@@ -463,32 +477,46 @@ install_deps() {
         apt-get -y install libudev-dev
 
         if [ x"${DIST}" = xnoble ]; then
-            apt-get -y install gcc-13 g++-13
-            update-alternatives --install /usr/bin/gcc gcc /usr/bin/gcc-13 100 --slave /usr/bin/g++ g++ /usr/bin/g++-13
-            update-alternatives --install /usr/bin/cc cc /usr/bin/gcc-13 100
+            apt-get -y install gcc-11 g++-11
+            update-alternatives --install /usr/bin/gcc gcc /usr/bin/gcc-11 100 --slave /usr/bin/g++ g++ /usr/bin/g++-11
         fi
 
-        if [ x"${DIST}" = xfocal -o x"${DIST}" = xbullseye -o x"${DIST}" = xjammy -o x"${DIST}" = xbookworm -o x"${DIST}" = xnoble ]; then
+        if [ x"${DIST}" = xfocal -o x"${DIST}" = xbullseye -o x"${DIST}" = jammy -o x"${DIST}" = bookworm -o x"${DIST}" = xnoble ]; then
             apt-get -y install python3-mysqldb
         else
             apt-get -y install python-mysqldb
         fi
-
+        if [ x"${DIST}" = xbionic ]; then
+            apt-get -y install gcc-8 g++-8
+            wget https://downloads.percona.com/downloads/packaging/libfido2-1/libcbor0.6_0.6.0-0ubuntu1_amd64.deb
+            wget https://downloads.percona.com/downloads/packaging/libfido2-1/libfido2-1_1.3.1-1ubuntu2_amd64.deb
+            dpkg -i libcbor0.6_0.6.0-0ubuntu1_amd64.deb
+            dpkg -i libfido2-1_1.3.1-1ubuntu2_amd64.deb
+        fi
         if [ x"${DIST}" = xbuster ]; then
             wget https://downloads.percona.com/downloads/packaging/libfido2-1/libfido2-1_1.5.0-2~bpo10+1_amd64.deb
             wget https://downloads.percona.com/downloads/packaging/libfido2-1/libcbor0_0.5.0+dfsg-2_amd64.deb
             dpkg -i libcbor0_0.5.0+dfsg-2_amd64.deb
             dpkg -i libfido2-1_1.5.0-2~bpo10+1_amd64.deb
+            echo "deb http://deb.debian.org/debian buster-backports main" >> /etc/apt/sources.list
+            apt update
+            apt -y install cmake/buster-backports
+        fi
+        if [ x"${DIST}" = xfocal ]; then
+            apt-get -y install gcc-10 g++-10
+            update-alternatives --install /usr/bin/gcc gcc /usr/bin/gcc-10 100
+            update-alternatives --install /usr/bin/g++ g++ /usr/bin/g++-10 100
+            update-alternatives --config gcc
+            update-alternatives --config g++
         fi
-
         apt-get -y install libmecab2 mecab mecab-ipadic
         apt-get -y install build-essential devscripts
         apt-get -y install cmake autotools-dev autoconf automake build-essential devscripts debconf debhelper fakeroot
         apt-get -y install libtool libnuma-dev scons libboost-dev libboost-program-options-dev check
         apt-get -y install doxygen doxygen-gui graphviz rsync libcurl4-openssl-dev
         apt-get -y install libcurl4-openssl-dev libre2-dev pkg-config libtirpc-dev libev-dev
-        apt-get -y install --download-only percona-xtrabackup-24=2.4.29-1.${DIST}
-        apt-get -y install --download-only percona-xtrabackup-80=8.0.35-33-1.${DIST}
+        apt-get -y install --download-only percona-xtrabackup-80=8.0.35-31-1.${DIST}
+        apt-get -y install --download-only percona-xtrabackup-84=8.4.0-1-1.${DIST}
     fi
     return;
 }
@@ -543,7 +571,7 @@ build_srpm(){
         echo "It is not possible to build src rpm here"
         exit 1
     fi
-    if [ "x${RHEL}" = "x7" -o x"$RHEL" = x8 ]; then
+    if [ x"$RHEL" = x8 ]; then
         switch_to_vault_repo
     fi
     cd $WORKDIR || exit
@@ -582,7 +610,44 @@ build_srpm(){
     sed -i "s:@@WSREP_VERSION@@:${WSREP_VERSION}:g" rpmbuild/SPECS/percona-xtradb-cluster.spec
     sed -i "s:@@REVISION@@:${REVISION}:g" rpmbuild/SPECS/percona-xtradb-cluster.spec
     sed -i "s:@@RPM_RELEASE@@:${RPM_RELEASE}:g" rpmbuild/SPECS/percona-xtradb-cluster.spec
+
     #
+
+    if [[ "x${FIPSMODE}" == "x1" ]]; then
+        sed -i -e "s:percona-xtradb-cluster$:percona-xtradb-cluster-pro:g" \
+        -e "s:percona-xtradb-cluster-full$:percona-xtradb-cluster-pro-full:g" \
+        -e "s:percona-xtradb-cluster-server$:percona-xtradb-cluster-server-pro:g" \
+        -e "s:percona-xtradb-cluster-server =:percona-xtradb-cluster-server-pro =:g" \
+        -e "s:percona-xtradb-cluster-server --:percona-xtradb-cluster-server-pro --:g" \
+        -e "s:percona-xtradb-cluster-client$:percona-xtradb-cluster-client-pro:g" \
+        -e "s:percona-xtradb-cluster-client :percona-xtradb-cluster-client-pro :g" \
+        -e "s:percona-xtradb-cluster-devel$:percona-xtradb-cluster-devel-pro:g" \
+        -e "s:percona-xtradb-cluster-devel =:percona-xtradb-cluster-devel-pro =:g" \
+        -e "s:percona-xtradb-cluster-test$:percona-xtradb-cluster-test-pro:g" \
+        -e "s:percona-xtradb-cluster-test =:percona-xtradb-cluster-test-pro =:g" \
+        -e "s:percona-xtradb-cluster-debuginfo$:percona-xtradb-cluster-pro-debuginfo:g" \
+        -e "s:percona-xtradb-cluster-debuginfo =:percona-xtradb-cluster-pro-debuginfo =:g" \
+        -e "s:percona-xtradb-cluster-garbd$:percona-xtradb-cluster-garbd-pro:g" \
+        -e "s:percona-xtradb-cluster-garbd =:percona-xtradb-cluster-garbd-pro =:g" \
+        -e "s:percona-xtradb-cluster-mysql-router$:percona-xtradb-cluster-mysql-router-pro:g" \
+        -e "s:percona-xtradb-cluster-mysql-router :percona-xtradb-cluster-mysql-router-pro :g" \
+        -e "s:percona-xtradb-cluster-mysql-router-devel$:percona-xtradb-cluster-mysql-router-devel-pro:g" \
+        -e "s:percona-xtradb-cluster-mysql-router-devel =:percona-xtradb-cluster-mysql-router-devel-pro =:g" \
+        -e "s:percona-xtradb-cluster-shared$:percona-xtradb-cluster-shared-pro:g" \
+        -e "s:percona-xtradb-cluster-shared :percona-xtradb-cluster-shared-pro :g" \
+        -e "s:Conflicts\:      percona-xtradb-cluster-server-pro:Conflicts\:      percona-xtradb-cluster-server:g" \
+        -e "s:Conflicts\:      percona-xtradb-cluster-client-pro:Conflicts\:      percona-xtradb-cluster-client:g" \
+        -e "s:Conflicts\:      percona-xtradb-cluster-test-pro:Conflicts\:      percona-xtradb-cluster-test:g" \
+        -e "s:Conflicts\:      percona-xtradb-cluster-devel-pro:Conflicts\:      percona-xtradb-cluster-devel:g" \
+        -e "s:Conflicts\:     percona-xtradb-cluster-mysql-router-pro:Conflicts\:     percona-xtradb-cluster-mysql-router:g" \
+        -e "s:Conflicts\:      percona-xtradb-cluster-mysql-router-devel-pro:Conflicts\:      percona-xtradb-cluster-mysql-router-devel:g" \
+        -e "s:Conflicts\:      percona-xtradb-cluster-test-pro:Conflicts\:      percona-xtradb-cluster-test:g" \
+        -e "s:Conflicts\:      percona-xtradb-cluster-shared-pro:Conflicts\:      percona-xtradb-cluster-shared:g" \
+        -e "s:Conflicts\:      percona-xtradb-cluster-garbd-pro:Conflicts\:      percona-xtradb-cluster-garbd:g" \
+        -e "s:Conflicts\:      percona-xtradb-cluster-pro-full:Conflicts\:      percona-xtradb-cluster-full:g" \
+       rpmbuild/SPECS/percona-xtradb-cluster.spec
+    fi
+
     SRCRPM=$(find . -name *.src.rpm)
     RHEL=$(rpm --eval %rhel)
     #
@@ -689,7 +754,7 @@ build_rpm(){
         echo "It is not possible to build rpm here"
         exit 1
     fi
-    if [ "x${RHEL}" = "x7" -o x"$RHEL" = x8 ]; then
+    if [ x"$RHEL" = x8 ]; then
         switch_to_vault_repo
     fi
     SRC_RPM=$(basename $(find $WORKDIR/srpm -iname 'percona-xtradb-cluster*.src.rpm' | sort | tail -n1))
@@ -750,9 +815,17 @@ build_rpm(){
     source ${CURDIR}/srpm/pxc-80.properties
     #
     if [ ${ARCH} = x86_64 ]; then
-        rpmbuild --define "_topdir ${WORKDIR}/rpmbuild" --define "dist el${RHEL}" --define "rpm_version $MYSQL_RELEASE.$RPM_RELEASE" --define "rel $RPM_RELEASE" --define "galera_revision ${GALERA_REVNO}" --define "with_mecab ${MECAB_INSTALL_DIR}/usr" --rebuild rpmbuild/SRPMS/${SRCRPM}
+        if [[ "x${FIPSMODE}" == "x1" ]]; then
+            rpmbuild --define "_topdir ${WORKDIR}/rpmbuild" --define "dist el${RHEL}" --define "rpm_version $MYSQL_RELEASE.$RPM_RELEASE" --define "enable_fipsmode 1" --define "rel $RPM_RELEASE" --define "galera_revision ${GALERA_REVNO}" --define "with_mecab ${MECAB_INSTALL_DIR}/usr" --rebuild rpmbuild/SRPMS/${SRCRPM}
+        else
+            rpmbuild --define "_topdir ${WORKDIR}/rpmbuild" --define "dist el${RHEL}" --define "rpm_version $MYSQL_RELEASE.$RPM_RELEASE" --define "rel $RPM_RELEASE" --define "galera_revision ${GALERA_REVNO}" --define "with_mecab ${MECAB_INSTALL_DIR}/usr" --rebuild rpmbuild/SRPMS/${SRCRPM}
+        fi
     else
-        rpmbuild --define "_topdir ${WORKDIR}/rpmbuild" --define "dist el${RHEL}" --define "rpm_version $MYSQL_RELEASE.$RPM_RELEASE" --define "rel $RPM_RELEASE" --define "galera_revision ${GALERA_REVNO}" --define "with_tokudb 0" --define "with_rocksdb 0" --define "with_mecab ${MECAB_INSTALL_DIR}/usr" --rebuild rpmbuild/SRPMS/${SRCRPM}
+        if [[ "x${FIPSMODE}" == "x1" ]]; then
+            rpmbuild --define "_topdir ${WORKDIR}/rpmbuild" --define "dist el${RHEL}" --define "rpm_version $MYSQL_RELEASE.$RPM_RELEASE" --define "enable_fipsmode 1" --define "rel $RPM_RELEASE" --define "galera_revision ${GALERA_REVNO}" --define "with_tokudb 0" --define "with_rocksdb 0" --define "with_mecab ${MECAB_INSTALL_DIR}/usr" --rebuild rpmbuild/SRPMS/${SRCRPM}
+        else
+            rpmbuild --define "_topdir ${WORKDIR}/rpmbuild" --define "dist el${RHEL}" --define "rpm_version $MYSQL_RELEASE.$RPM_RELEASE" --define "rel $RPM_RELEASE" --define "galera_revision ${GALERA_REVNO}" --define "with_tokudb 0" --define "with_rocksdb 0" --define "with_mecab ${MECAB_INSTALL_DIR}/usr" --rebuild rpmbuild/SRPMS/${SRCRPM}
+        fi
     fi
     return_code=$?
     if [ $return_code != 0 ]; then
@@ -775,7 +848,7 @@ build_source_deb(){
         echo "It is not possible to build source deb here"
         exit 1
     fi
-     source ${WORKDIR}/pxc-80.properties
+    source ${WORKDIR}/pxc-80.properties
     rm -rf percona-server*
     get_tar "source_tarball"
     rm -f *.dsc *.orig.tar.gz *.debian.tar.gz *.changes
@@ -791,7 +864,11 @@ build_source_deb(){
     rm -fr ${HNAME}-${VERSION}
 
     #
-    NEWTAR=${NAME}_${VERSION}-${RELEASE}.orig.tar.gz
+    if [[ "x${FIPSMODE}" == "x1" ]]; then
+        NEWTAR=${NAME}-pro_${VERSION}-${RELEASE}.orig.tar.gz
+    else
+        NEWTAR=${NAME}_${VERSION}-${RELEASE}.orig.tar.gz
+    fi
     mv ${TARFILE} ${NEWTAR}
 
     DEBIAN_VERSION="$(lsb_release -sc)"
@@ -800,6 +877,108 @@ build_source_deb(){
     tar xzf ${NEWTAR}
     cd ${HNAME}-${VERSION}-${MYSQL_RELEASE} || exit
     cp -ap build-ps/debian/ .
+    if [ x"${FIPSMODE}" == x1 ]; then
+        sed -i "s:FIPSFLAGS=:FIPSFLAGS=-DPROBUILD=1:g" debian/rules
+        sed -i "s:Lpercona-xtradb-cluster-server:Lpercona-xtradb-cluster-server-pro:g" debian/rules
+        sed -i "s:percona-xtradb-cluster-dbg:percona-xtradb-cluster-pro-dbg:g" debian/rules
+        sed -i "s:percona-xtradb-cluster-server-debug:percona-xtradb-cluster-server-pro-debug:g" debian/rules
+
+        sed -i "s:percona-xtradb-cluster:percona-xtradb-cluster-pro:g" debian/changelog
+
+        sed -i "s:Source\: percona-xtradb-cluster:Source\: percona-xtradb-cluster-pro:g" debian/control
+
+        sed -i "s:Package\: percona-xtradb-cluster-common:Package\: percona-xtradb-cluster-pro-common:g" debian/control
+        sed -i "s:, percona-xtradb-cluster-pro-common:, percona-xtradb-cluster-common:g" debian/control
+        sed -i "s:Conflicts\: percona-xtradb-cluster-pro-common:Conflicts\: percona-xtradb-cluster-common:g" debian/control
+        sed -i "s:Depends\: percona-xtradb-cluster-common:Depends\: percona-xtradb-cluster-pro-common:g" debian/control
+
+        sed -i "s:Package\: percona-xtradb-cluster-client:Package\: percona-xtradb-cluster-client-pro:g" debian/control
+        sed -i "s:Conflicts\: percona-xtradb-cluster-client-pro:Conflicts\: percona-xtradb-cluster-client:g" debian/control
+        sed -i "s:  percona-xtradb-cluster-client-pro:  percona-xtradb-cluster-client:g" debian/control
+        sed -i "s:percona-xtradb-cluster-client (:percona-xtradb-cluster-client-pro (:g" debian/control
+
+        sed -i "s:Package\: percona-xtradb-cluster-server:Package\: percona-xtradb-cluster-server-pro:g" debian/control
+        sed -i "s:Conflicts\: percona-xtradb-cluster-server-pro:Conflicts\: percona-xtradb-cluster-server:g" debian/control
+        sed -i "s:  percona-xtradb-cluster-server-pro:  percona-xtradb-cluster-server:g" debian/control
+        sed -i "s:percona-xtradb-cluster-server (:percona-xtradb-cluster-server-pro (:g" debian/control
+
+        sed -i "s:Package\: percona-xtradb-cluster-server-debug:Package\: percona-xtradb-cluster-server-pro-debug:g" debian/control
+        sed -i "s:Replaces\: percona-xtradb-cluster-server-pro-debug:Replaces\: percona-xtradb-cluster-server-debug:g" debian/control
+        sed -i "s:Conflicts\: percona-xtradb-cluster-server-pro-debug:Conflicts\: percona-xtradb-cluster-server-debug:g" debian/control
+        sed -i "s:percona-xtradb-cluster-server-debug ,:percona-xtradb-cluster-server-pro-debug ,:g" debian/control
+
+        sed -i "s:Package\: percona-xtradb-cluster$:Package\: percona-xtradb-cluster-pro:g" debian/control
+        sed -i "s:Replaces\: percona-xtradb-cluster-pro:Replaces\: percona-xtradb-cluster:g" debian/control
+        sed -i "s:Conflicts\: percona-xtradb-cluster-pro:Conflicts\: percona-xtradb-cluster:g" debian/control
+
+        sed -i "s:Package\: percona-xtradb-cluster-full:Package\: percona-xtradb-cluster-pro-full:g" debian/control
+        sed -i "s:Replaces\: percona-xtradb-cluster-pro-full:Replaces\: percona-xtradb-cluster-full:g" debian/control
+        sed -i "s:Conflicts\: percona-xtradb-cluster-pro-full:Conflicts\: percona-xtradb-cluster-full:g" debian/control
+        sed -i "s:percona-xtradb-cluster-test (:percona-xtradb-cluster-test-pro (:g" debian/control
+        sed -i "s:percona-xtradb-cluster-dbg (:percona-xtradb-cluster-pro-dbg (:g" debian/control
+        sed -i "s:percona-xtradb-cluster-server-debug,:percona-xtradb-cluster-server-pro-debug,:g" debian/control
+        sed -i "s:percona-xtradb-cluster-garbd (:percona-xtradb-cluster-garbd-pro (:g" debian/control
+        sed -i "s:percona-xtradb-cluster-garbd-debug (:percona-xtradb-cluster-garbd-pro-debug (:g" debian/control
+
+        sed -i "s:Package\: percona-xtradb-cluster-test:Package\: percona-xtradb-cluster-test-pro:g" debian/control
+        sed -i "s:Replaces\: percona-xtradb-cluster-test-pro:Replaces\: percona-xtradb-cluster-test:g" debian/control
+        sed -i "s:Conflicts\: percona-xtradb-cluster-test-pro:Conflicts\: percona-xtradb-cluster-test:g" debian/control
+
+        sed -i "s:Package\: percona-xtradb-cluster-dbg:Package\: percona-xtradb-cluster-pro-dbg:g" debian/control
+        sed -i "s:Replaces\: percona-xtradb-cluster-pro-dbg:Replaces\: percona-xtradb-cluster-dbg:g" debian/control
+        sed -i "s:Conflicts\: percona-xtradb-cluster-pro-dbg:Conflicts\: percona-xtradb-cluster-dbg:g" debian/control
+
+        sed -i "s:Package\: percona-xtradb-cluster-source:Package\: percona-xtradb-cluster-pro-source:g" debian/control
+        sed -i "s:Replaces\: percona-xtradb-cluster-pro-source:Replaces\: percona-xtradb-cluster-source:g" debian/control
+        sed -i "s:Conflicts\: percona-xtradb-cluster-pro-source:Conflicts\: percona-xtradb-cluster-source:g" debian/control
+
+        sed -i "s:Package\: percona-xtradb-cluster-garbd:Package\: percona-xtradb-cluster-garbd-pro:g" debian/control
+        sed -i "s:, percona-xtradb-cluster-garbd-pro$:, percona-xtradb-cluster-garbd:g" debian/control
+
+        sed -i "s:Package\: percona-xtradb-cluster-garbd-debug:Package\: percona-xtradb-cluster-garbd-pro-debug:g" debian/control
+        sed -i "s:, percona-xtradb-cluster-garbd-pro-debug$:, percona-xtradb-cluster-garbd-debug:g" debian/control
+
+        cp debian/percona-xtradb-cluster-client.README.Debian debian/percona-xtradb-cluster-client-pro.README.Debian
+        cp debian/percona-xtradb-cluster-client.dirs debian/percona-xtradb-cluster-client-pro.dirs
+        cp debian/percona-xtradb-cluster-client.docs debian/percona-xtradb-cluster-client-pro.docs
+        cp debian/percona-xtradb-cluster-client.install debian/percona-xtradb-cluster-client-pro.install
+        cp debian/percona-xtradb-cluster-client.links debian/percona-xtradb-cluster-client-pro.links
+        cp debian/percona-xtradb-cluster-client.lintian-overrides debian/percona-xtradb-cluster-client-pro.lintian-overrides
+        cp debian/percona-xtradb-cluster-common.dirs debian/percona-xtradb-cluster-pro-common.dirs
+        cp debian/percona-xtradb-cluster-common.install debian/percona-xtradb-cluster-pro-common.install
+        cp debian/percona-xtradb-cluster-common.lintian-overrides debian/percona-xtradb-cluster-pro-common.lintian-overrides
+        cp debian/percona-xtradb-cluster-common.postinst debian/percona-xtradb-cluster-pro-common.postinst
+        cp debian/percona-xtradb-cluster-common.postrm debian/percona-xtradb-cluster-pro-common.postrm
+        cp debian/percona-xtradb-cluster-garbd-debug.install debian/percona-xtradb-cluster-garbd-pro-debug.install
+        cp debian/percona-xtradb-cluster-garbd.docs debian/percona-xtradb-cluster-garbd-pro.docs
+        cp debian/percona-xtradb-cluster-garbd.garbd.default debian/percona-xtradb-cluster-garbd-pro.garbd.default
+        cp debian/percona-xtradb-cluster-garbd.garbd.init debian/percona-xtradb-cluster-garbd-pro.garbd.init
+        cp debian/percona-xtradb-cluster-garbd.install debian/percona-xtradb-cluster-garbd-pro.install
+        cp debian/percona-xtradb-cluster-garbd.postinst debian/percona-xtradb-cluster-garbd-pro.postinst
+        cp debian/percona-xtradb-cluster-garbd.preinst debian/percona-xtradb-cluster-garbd-pro.preinst
+        cp debian/percona-xtradb-cluster-garbd.prerm debian/percona-xtradb-cluster-garbd-pro.prerm
+        cp debian/percona-xtradb-cluster-server-debug.install debian/percona-xtradb-cluster-server-pro-debug.install
+        cp debian/percona-xtradb-cluster-server.README.Debian debian/percona-xtradb-cluster-server-pro.README.Debian
+        cp debian/percona-xtradb-cluster-server.config debian/percona-xtradb-cluster-server-pro.config
+        cp debian/percona-xtradb-cluster-server.dirs debian/percona-xtradb-cluster-server-pro.dirs
+        cp debian/percona-xtradb-cluster-server.docs debian/percona-xtradb-cluster-server-pro.docs
+        cp debian/percona-xtradb-cluster-server.install debian/percona-xtradb-cluster-server-pro.install
+        cp debian/percona-xtradb-cluster-server.links debian/percona-xtradb-cluster-server-pro.links
+        cp debian/percona-xtradb-cluster-server.lintian-overrides debian/percona-xtradb-cluster-server-pro.lintian-overrides
+        cp debian/percona-xtradb-cluster-server.logcheck.ignore.paranoid debian/percona-xtradb-cluster-server-pro.logcheck.ignore.paranoid
+        cp debian/percona-xtradb-cluster-server.logcheck.ignore.server debian/percona-xtradb-cluster-server-pro.logcheck.ignore.server
+        cp debian/percona-xtradb-cluster-server.logcheck.ignore.workstation debian/percona-xtradb-cluster-server-pro.logcheck.ignore.workstation
+        cp debian/percona-xtradb-cluster-server.mysql.init debian/percona-xtradb-cluster-server-pro.mysql.init
+        cp debian/percona-xtradb-cluster-server.postinst debian/percona-xtradb-cluster-server-pro.postinst
+        cp debian/percona-xtradb-cluster-server.postrm debian/percona-xtradb-cluster-server-pro.postrm
+        cp debian/percona-xtradb-cluster-server.preinst debian/percona-xtradb-cluster-server-pro.preinst
+        cp debian/percona-xtradb-cluster-server.prerm debian/percona-xtradb-cluster-server-pro.prerm
+        cp debian/percona-xtradb-cluster-server.templates debian/percona-xtradb-cluster-server-pro.templates
+        cp debian/percona-xtradb-cluster-source.install debian/percona-xtradb-cluster-pro-source.install
+        cp debian/percona-xtradb-cluster-test.install debian/percona-xtradb-cluster-test-pro.install
+        cp debian/percona-xtradb-cluster-test.links debian/percona-xtradb-cluster-test-pro.links
+        cp debian/percona-xtradb-cluster-test.lintian-overrides debian/percona-xtradb-cluster-test-pro.lintian-overrides
+    fi
     sed -i "s:@@MYSQL_VERSION@@:${VERSION}:g" debian/changelog
     sed -i "s:@@PERCONA_VERSION@@:${RELEASE}:g" debian/changelog
 
@@ -865,7 +1044,11 @@ build_deb(){
     export CXXFLAGS=" $COMMON_FLAGS -Wno-virtual-move-assign  ${CXXFLAGS:-}"
 
     DSC=$(basename $(find . -name '*.dsc' | sort | tail -n 1))
-    DIRNAME=$(echo ${DSC} | sed -e 's:_:-:g' | awk -F'-' '{print $1"-"$2"-"$3"-"$4"-"$5}' | sed -e s:.dsc::)
+    if [[ "x${FIPSMODE}" == "x1" ]]; then
+        DIRNAME=$(echo ${DSC} | sed -e 's:_:-:g' | awk -F'-' '{print $1"-"$2"-"$3"-"$4"-"$5"-"$6}' | sed -e s:.dsc::)
+    else
+        DIRNAME=$(echo ${DSC} | sed -e 's:_:-:g' | awk -F'-' '{print $1"-"$2"-"$3"-"$4"-"$5}' | sed -e s:.dsc::)
+    fi
     rm -rf $DIRNAME
 
     #
@@ -876,18 +1059,27 @@ build_deb(){
 
     cd ${DIRNAME} || exit
 
-    mkdir pxb-2.4
-    mkdir pxb-8.0
-    dpkg-deb -R /var/cache/apt/archives/percona-xtrabackup-24* pxb-2.4
+    # (1) PXB compatible with previous PXC LTS version
+    mkdir -p pxb-8.0
+    # (2) PXB compatible with this PXC version (LTS or Innovative)
+    mkdir -p pxb-8.4
+
+
     dpkg-deb -R /var/cache/apt/archives/percona-xtrabackup-80* pxb-8.0
-    cd pxb-2.4 || exit
+    dpkg-deb -R /var/cache/apt/archives/percona-xtrabackup-84* pxb-8.4
+
+    #  (1)
+    cd pxb-8.0 || exit
         mv usr/bin ./
         mv usr/lib* ./
         rm -rf usr *.deb DEBIAN
-    cd ../pxb-8.0 || exit
+
+    # (2)
+    cd ../pxb-8.4 || exit
         mv usr/bin ./
         mv usr/lib* ./
         rm -rf usr *.deb DEBIAN
+
     cd ../ || exit
 
     if [[ "x$DEBIAN_VERSION" == "xbionic" || "x$DEBIAN_VERSION" == "xstretch" || "x$DEBIAN_VERSION" == "xfocal" || "x$DEBIAN_VERSION" == "xbullseye" || "x$DEBIAN_VERSION" == "xjammy" || "x$DEBIAN_VERSION" == "xbookworm" || "x$DEBIAN_VERSION" == "xnoble" ]]; then
@@ -909,6 +1101,10 @@ build_deb(){
     #
 
     postfix=""
+    if [ x"${FIPSMODE}" == x1 ]; then
+        postfix="-pro"
+    fi
+
     cd debian/
         wget https://raw.githubusercontent.com/Percona-Lab/telemetry-agent/phase-0/call-home.sh
         sed -i 's:exit 0::' percona-xtradb-cluster-server"${postfix}".postinst
@@ -928,12 +1124,6 @@ build_deb(){
         sed -i 's/export CXXFLAGS=/export CXXFLAGS=-Wno-error=nonnull-compare /' debian/rules
     fi
 
-    if [ ${DEBIAN_VERSION} = "noble" -a ${ARCH} = "aarch64" ]; then
-        sed -i 's:dh_strip --dbg-package=percona-xtradb-cluster-dbg:mv debian/percona-xtradb-cluster-server/usr/lib/mysql/plugin/authentication_fido.so /tmp\n\tdh_strip --dbg-package=percona-xtradb-cluster-dbg\n\tmv /tmp/authentication_fido.so debian/percona-xtradb-cluster-server/usr/lib/mysql/plugin/authentication_fido.so:' debian/rules
-        sed -i 's:dh_strip -Xlibprotobuf-lite:dh_strip -Xlibprotobuf-lite --exclude=debian/percona-xtradb-cluster-server/usr/lib/mysql/plugin/authentication_fido.so:' debian/rules
-        cat debian/rules
-    fi
-
     GALERA_REVNO="${GALERA_REVNO}" SCONS_ARGS=' strict_build_flags=0'  MAKE_JFLAG=-j4  dpkg-buildpackage -rfakeroot -uc -us -b
     #
     cd ${WORKSPACE} || exit
@@ -999,6 +1189,10 @@ build_tarball(){
     #
     cd ${NAME}-${VERSION}-${MYSQL_RELEASE} || exit
 
+    if [[ "x${FIPSMODE}" == "x1" ]]; then
+        sed -i "s/FIPSMODE=0/FIPSMODE=1/g" build-ps/build-binary.sh
+    fi
+
     BUILD_NUMBER=$(date "+%Y%m%d-%H%M%S")
     mkdir -p $BUILD_NUMBER
 
@@ -1007,19 +1201,7 @@ build_tarball(){
     CURDIR=$(pwd)
     cd ${BUILD_ROOT} || exit
     if [ -f /etc/redhat-release ]; then
-        mkdir pxb-2.4
-        pushd pxb-2.4
-        yumdownloader percona-xtrabackup-24-2.4.29
-        rpm2cpio *.rpm | cpio --extract --make-directories --verbose
-        mv usr/bin ./
-        mv usr/lib* ./
-        mv lib64 lib
-        mv lib/xtrabackup/* lib/ || true
-        rm -rf lib/xtrabackup
-        rm -rf usr
-        rm -f *.rpm
-        popd
-
+        # (1)
         mkdir pxb-8.0
         pushd pxb-8.0
         yumdownloader percona-xtrabackup-80-8.0.35
@@ -1033,29 +1215,50 @@ build_tarball(){
         rm -rf usr
         rm -f *.rpm
         popd
-        tar -zcvf  percona-xtrabackup-2.4.tar.gz pxb-2.4
+
+        # (2)
+        mkdir pxb-8.4
+        pushd pxb-8.4
+        yumdownloader percona-xtrabackup-84-8.4.0
+        rpm2cpio *.rpm | cpio --extract --make-directories --verbose
+        mv usr/bin ./
+        mv usr/lib* ./
+        mv lib64 lib
+        mv lib/xtrabackup/* lib/ || true
+        rm -rf lib/xtrabackup
+        rm -rf usr
+        rm -f *.rpm
+        popd
+
         tar -zcvf  percona-xtrabackup-8.0.tar.gz pxb-8.0
+        tar -zcvf  percona-xtrabackup-8.4.tar.gz pxb-8.4
     else
-        mkdir pxb-2.4
         mkdir pxb-8.0
-        dpkg-deb -R /var/cache/apt/archives/percona-xtrabackup-24* pxb-2.4
+        mkdir pxb-8.4
         dpkg-deb -R /var/cache/apt/archives/percona-xtrabackup-80* pxb-8.0
-        cd pxb-2.4 || exit
+        dpkg-deb -R /var/cache/apt/archives/percona-xtrabackup-84* pxb-8.4
+        
+        # (1)
+        pushd pxb-8.0
             mv usr/bin ./
             mv usr/lib* ./
             rm -rf usr *.deb DEBIAN
-        cd ../pxb-8.0 || exit
+        popd
+
+        # (2)
+        pushd pxb-8.4
             mv usr/bin ./
             mv usr/lib* ./
             rm -rf usr *.deb DEBIAN
-        cd ../ || exit
-        tar -zcvf  percona-xtrabackup-2.4.tar.gz pxb-2.4
+        popd
+        
         tar -zcvf  percona-xtrabackup-8.0.tar.gz pxb-8.0
+        tar -zcvf  percona-xtrabackup-8.4.tar.gz pxb-8.4
     fi
     mkdir -p ${BUILD_ROOT}/target/pxc_extra/
     cp *.tar.gz ${BUILD_ROOT}/target/pxc_extra/
     cp *.tar.gz ${BUILD_ROOT}/target
-    rm -rf pxb-8.0 pxb-2.4
+    rm -rf pxb-8.0 pxb-8.4
     cd ${CURDIR} || exit
     rm -rf jemalloc
     wget https://github.com/jemalloc/jemalloc/releases/download/$JVERSION/jemalloc-$JVERSION.tar.bz2
