@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
@@ -65,8 +65,8 @@ enum enum_sql_command {
   SQLCOM_SHOW_ENGINE_STATUS,
   SQLCOM_SHOW_ENGINE_MUTEX,
   SQLCOM_SHOW_PROCESSLIST,
-  SQLCOM_SHOW_MASTER_STAT,
-  SQLCOM_SHOW_SLAVE_STAT,
+  SQLCOM_SHOW_BINLOG_STATUS,
+  SQLCOM_SHOW_REPLICA_STATUS,
   SQLCOM_SHOW_GRANTS,
   SQLCOM_SHOW_CREATE,
   SQLCOM_SHOW_CHARSETS,
@@ -101,12 +101,12 @@ enum enum_sql_command {
   SQLCOM_COMMIT,
   SQLCOM_SAVEPOINT,
   SQLCOM_RELEASE_SAVEPOINT,
-  SQLCOM_SLAVE_START,
-  SQLCOM_SLAVE_STOP,
+  SQLCOM_REPLICA_START,
+  SQLCOM_REPLICA_STOP,
   SQLCOM_START_GROUP_REPLICATION,
   SQLCOM_STOP_GROUP_REPLICATION,
   SQLCOM_BEGIN,
-  SQLCOM_CHANGE_MASTER,
+  SQLCOM_CHANGE_REPLICATION_SOURCE,
   SQLCOM_CHANGE_REPLICATION_FILTER,
   SQLCOM_RENAME_TABLE,
   SQLCOM_RESET,
@@ -117,7 +117,7 @@ enum enum_sql_command {
   SQLCOM_HA_OPEN,
   SQLCOM_HA_CLOSE,
   SQLCOM_HA_READ,
-  SQLCOM_SHOW_SLAVE_HOSTS,
+  SQLCOM_SHOW_REPLICAS,
   SQLCOM_DELETE_MULTI,
   SQLCOM_UPDATE_MULTI,
   SQLCOM_SHOW_BINLOG_EVENTS,
@@ -203,6 +203,7 @@ enum enum_sql_command {
   SQLCOM_RESTART_SERVER,
   SQLCOM_CREATE_SRS,
   SQLCOM_DROP_SRS,
+  SQLCOM_SHOW_PARSE_TREE,
   SQLCOM_SHOW_USER_STATS,
   SQLCOM_SHOW_TABLE_STATS,
   SQLCOM_SHOW_INDEX_STATS,
