@@ -54,12 +54,12 @@
 #include <time.h>
 
 #include <atomic>  // error_handler_hook
+#include <cstring>
 
-#include "m_string.h" /* IWYU pragma: keep */
 #include "my_compiler.h"
 #include "my_compress.h"
 #include "my_inttypes.h"
-#include "my_loglevel.h"
+#include "mysql/my_loglevel.h"
 
 /* HAVE_PSI_*_INTERFACE */
 #include "my_psi_config.h" /* IWYU pragma: keep */
@@ -71,11 +71,15 @@
 #include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/components/services/bits/psi_file_bits.h"
 #include "mysql/components/services/bits/psi_memory_bits.h"
+#include "mysql/components/services/bits/psi_metric_bits.h"
 #include "mysql/components/services/bits/psi_stage_bits.h"
 #include "sql/stream_cipher.h"
+#include "string_with_len.h"
+
+class MY_CHARSET_LOADER;
 
 struct CHARSET_INFO;
-struct MY_CHARSET_LOADER;
+struct MY_CHARSET_ERRMSG;
 
 struct PSI_cond_bootstrap;
 struct PSI_data_lock_bootstrap;
@@ -84,6 +88,7 @@ struct PSI_file_bootstrap;
 struct PSI_idle_bootstrap;
 struct PSI_mdl_bootstrap;
 struct PSI_memory_bootstrap;
+struct PSI_metric_bootstrap;
 struct PSI_mutex_bootstrap;
 struct PSI_rwlock_bootstrap;
 struct PSI_socket_bootstrap;
@@ -262,8 +267,8 @@ extern int (*is_killed_hook)(const void *opaque_thd);
 /* charsets */
 #define MY_ALL_CHARSETS_SIZE 2048
 extern MYSQL_PLUGIN_IMPORT CHARSET_INFO *default_charset_info;
-extern MYSQL_PLUGIN_IMPORT CHARSET_INFO *all_charsets[MY_ALL_CHARSETS_SIZE];
-extern CHARSET_INFO compiled_charsets[];
+extern MYSQL_PLUGIN_IMPORT const CHARSET_INFO
+    *all_charsets[MY_ALL_CHARSETS_SIZE];
 
 /* statistics */
 extern ulong my_tmp_file_created;
@@ -478,8 +483,9 @@ struct ST_FILE_ID {
 };
 
 typedef void (*my_error_reporter)(enum loglevel level, uint ecode, ...);
+typedef void (*my_error_vreporter)(enum loglevel level, uint ecode, va_list);
 
-extern my_error_reporter my_charset_error_reporter;
+extern my_error_vreporter my_charset_error_reporter;
 
 /* defines for mf_iocache */
 extern PSI_file_key key_file_io_cache;
@@ -494,6 +500,7 @@ inline bool my_b_inited(const IO_CACHE *info) {
 constexpr int my_b_EOF = INT_MIN;
 
 inline int my_b_read(IO_CACHE *info, uchar *buffer, size_t count) {
+  assert(info->type != WRITE_CACHE);
   if ((size_t)(info->read_end - info->read_pos) >= count) {
     memcpy(buffer, info->read_pos, count);
     info->read_pos += count;
@@ -503,6 +510,7 @@ inline int my_b_read(IO_CACHE *info, uchar *buffer, size_t count) {
 }
 
 inline int my_b_write(IO_CACHE *info, const uchar *buffer, size_t count) {
+  assert(info->type != READ_CACHE);
   if ((size_t)(info->write_end - info->write_pos) >= count) {
     memcpy(info->write_pos, buffer, count);
     info->write_pos += count;
@@ -521,22 +529,22 @@ inline int my_b_get(IO_CACHE *info) {
   return _my_b_get(info);
 }
 
-MY_NODISCARD
+[[nodiscard]]
 inline my_off_t my_b_tell(const IO_CACHE *info) {
   return info->pos_in_file + *info->current_pos - info->request_pos;
 }
 
-MY_NODISCARD
+[[nodiscard]]
 inline uchar *my_b_get_buffer_start(const IO_CACHE *info) {
   return info->request_pos;
 }
 
-MY_NODISCARD
+[[nodiscard]]
 inline size_t my_b_get_bytes_in_buffer(const IO_CACHE *info) {
   return info->read_end - my_b_get_buffer_start(info);
 }
 
-MY_NODISCARD
+[[nodiscard]]
 inline my_off_t my_b_get_pos_in_file(const IO_CACHE *info) {
   return info->pos_in_file;
 }
@@ -544,7 +552,7 @@ inline my_off_t my_b_get_pos_in_file(const IO_CACHE *info) {
 /* tell write offset in the SEQ_APPEND cache */
 int my_b_copy_to_file(IO_CACHE *cache, FILE *file);
 
-MY_NODISCARD
+[[nodiscard]]
 inline size_t my_b_bytes_in_cache(const IO_CACHE *info) {
   return *info->current_end - *info->current_pos;
 }
@@ -740,31 +748,31 @@ extern bool array_append_string_unique(const char *str, const char **array,
 
 void my_store_ptr(uchar *buff, size_t pack_length, my_off_t pos);
 my_off_t my_get_ptr(uchar *ptr, size_t pack_length);
-MY_NODISCARD
+[[nodiscard]]
 extern int init_io_cache_ext(IO_CACHE *info, File file, size_t cachesize,
                              enum cache_type type, my_off_t seek_offset,
                              bool use_async_io, myf cache_myflags,
                              PSI_file_key file_key);
-MY_NODISCARD
+[[nodiscard]]
 extern int init_io_cache(IO_CACHE *info, File file, size_t cachesize,
                          enum cache_type type, my_off_t seek_offset,
                          bool use_async_io, myf cache_myflags);
-MY_NODISCARD
+[[nodiscard]]
 extern bool reinit_io_cache(IO_CACHE *info, enum cache_type type,
                             my_off_t seek_offset, bool use_async_io,
                             bool clear_cache);
 extern void setup_io_cache(IO_CACHE *info);
-MY_NODISCARD
+[[nodiscard]]
 extern int _my_b_read(IO_CACHE *info, uchar *Buffer, size_t Count);
-MY_NODISCARD
+[[nodiscard]]
 extern int _my_b_read_r(IO_CACHE *info, uchar *Buffer, size_t Count);
 extern void init_io_cache_share(IO_CACHE *read_cache, IO_CACHE_SHARE *cshare,
                                 IO_CACHE *write_cache, uint num_threads);
 extern void remove_io_thread(IO_CACHE *info);
-MY_NODISCARD
+[[nodiscard]]
 extern int _my_b_seq_read(IO_CACHE *info, uchar *Buffer, size_t Count);
 extern int _my_b_net_read(IO_CACHE *info, uchar *Buffer, size_t Count);
-MY_NODISCARD
+[[nodiscard]]
 extern int _my_b_write(IO_CACHE *info, const uchar *Buffer, size_t Count);
 extern int my_b_append(IO_CACHE *info, const uchar *Buffer, size_t Count);
 extern int my_b_safe_write(IO_CACHE *info, const uchar *Buffer, size_t Count);
@@ -898,25 +906,23 @@ int my_msync(int, void *, size_t, int);
 /* character sets */
 extern uint get_charset_number(const char *cs_name, uint cs_flags);
 extern uint get_collation_number(const char *name);
-extern const char *get_collation_name(uint cs_number);
+extern const char *get_collation_name(uint charset_number);
 
 extern CHARSET_INFO *get_charset(uint cs_number, myf flags);
 extern CHARSET_INFO *get_charset_by_name(const char *cs_name, myf flags);
-extern CHARSET_INFO *my_collation_get_by_name(MY_CHARSET_LOADER *loader,
-                                              const char *name, myf flags);
+extern CHARSET_INFO *my_collation_get_by_name(const char *collation_name,
+                                              myf flags, MY_CHARSET_ERRMSG *);
 extern CHARSET_INFO *get_charset_by_csname(const char *cs_name, uint cs_flags,
                                            myf my_flags);
-extern CHARSET_INFO *my_charset_get_by_name(MY_CHARSET_LOADER *loader,
-                                            const char *name, uint cs_flags,
-                                            myf my_flags);
+extern CHARSET_INFO *my_charset_get_by_name(const char *name, uint cs_flags,
+                                            myf my_flags, MY_CHARSET_ERRMSG *);
 extern bool resolve_charset(const char *cs_name, const CHARSET_INFO *default_cs,
                             const CHARSET_INFO **cs);
 extern bool resolve_collation(const char *cl_name,
                               const CHARSET_INFO *default_cl,
                               const CHARSET_INFO **cl);
 extern char *get_charsets_dir(char *buf);
-extern bool init_compiled_charsets(myf flags);
-extern void add_compiled_collation(CHARSET_INFO *cs);
+
 extern size_t escape_string_for_mysql(const CHARSET_INFO *charset_info,
                                       char *to, size_t to_length,
                                       const char *from, size_t length);
@@ -970,6 +976,8 @@ extern void set_psi_mdl_service(void *psi);
 extern MYSQL_PLUGIN_IMPORT PSI_memory_bootstrap *psi_memory_hook;
 extern void set_psi_memory_service(void *psi);
 extern MYSQL_PLUGIN_IMPORT PSI_mutex_bootstrap *psi_mutex_hook;
+extern void set_psi_metric_service(void *psi);
+extern MYSQL_PLUGIN_IMPORT PSI_metric_bootstrap *psi_metric_hook;
 extern void set_psi_mutex_service(void *psi);
 extern MYSQL_PLUGIN_IMPORT PSI_rwlock_bootstrap *psi_rwlock_hook;
 extern void set_psi_rwlock_service(void *psi);
@@ -991,6 +999,13 @@ extern MYSQL_PLUGIN_IMPORT PSI_tls_channel_bootstrap *psi_tls_channel_hook;
 extern void set_psi_tls_channel_service(void *psi);
 #endif /* HAVE_PSI_INTERFACE */
 
+/* Total physical memory available */
+[[nodiscard]] extern unsigned long long my_physical_memory();
+
+/* Compares versions and determine if clone is allowed */
+[[nodiscard]] extern bool are_versions_clone_compatible(std::string ver1,
+                                                        std::string ver2);
+
 /**
   @} (end of group MYSYS)
 */
