@@ -1,4 +1,4 @@
-/* Copyright (c) 2005, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2005, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -166,8 +166,11 @@ struct MYSQL_XID {
                          builtin_##NAME##_sizeof_struct_st_plugin,        \
                          builtin_##NAME##_plugin)
 
-#define mysql_declare_plugin_end                 \
-  , { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 } \
+#define mysql_declare_plugin_end                                            \
+  , {                                                                       \
+    0, nullptr, nullptr, nullptr, nullptr, 0, nullptr, nullptr, nullptr, 0, \
+        nullptr, nullptr, nullptr, 0                                        \
+  }                                                                         \
   }
 
 /*
