@@ -275,6 +275,7 @@ enum enum_ssl_init_error {
   SSL_FIPS_MODE_FAILED,
   SSL_INITERR_ECDHFAIL,
   SSL_INITERR_X509_VERIFY_PARAM,
+  SSL_INITERR_INVALID_CERTIFICATES,
   SSL_INITERR_LASTERR
 };
 const char *sslGetErrString(enum enum_ssl_init_error err);
@@ -286,7 +287,8 @@ struct st_VioSSLFd {
 int sslaccept(struct st_VioSSLFd *, MYSQL_VIO, long timeout,
               unsigned long *errptr);
 int sslconnect(struct st_VioSSLFd *, MYSQL_VIO, long timeout,
-               SSL_SESSION *session, unsigned long *errptr, SSL **ssl);
+               SSL_SESSION *session, unsigned long *errptr, SSL **ssl,
+               const char *sni_servername);
 
 struct st_VioSSLFd *new_VioSSLConnectorFd(
     const char *key_file, const char *cert_file, const char *ca_file,
