@@ -1,5 +1,5 @@
 # -*- cperl -*-
-# Copyright (c) 2007, 2025, Oracle and/or its affiliates.
+# Copyright (c) 2007, 2024, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0,
@@ -366,6 +366,7 @@ my @mysqld_rules = (
   { 'tmpdir'                                       => \&fix_tmpdir },
   { 'loose-sha256_password_auto_generate_rsa_keys' => "0" },
   { 'loose-caching_sha2_password_auto_generate_rsa_keys' => "0" },
+  { 'loose-mysql-native-password' => "ON" },
 
   { '#mtr_basedir' => sub { return shift->{ARGS}->{basedir}; }
   },
@@ -421,10 +422,6 @@ my @ndbd_rules = ({ 'BackupDataDir' => \&fix_cluster_backup_dir },
                   { 'HostName'      => \&fix_host },
                   { '#cpubind'      => \&fix_cpubind },);
 
-# Rules to run for each memcached in the config
-#  - will be run in order listed here
-my @memcached_rules = ({ '#host' => \&fix_host }, { 'port' => \&fix_port },);
-
 # Rules to run for each cluster_config section
 #  - will be run in order listed here
 my @cluster_config_rules = ({ 'mysqld'   => \&fix_host },
@@ -533,7 +530,7 @@ sub post_check_client_groups {
 }
 
 sub resolve_at_variable {
-  my ($self, $config, $group, $option, $worker) = @_;
+  my ($self, $config, $group, $option) = @_;
   local $_ = $option->value();
   my ($res, $after);
 
@@ -545,9 +542,7 @@ sub resolve_at_variable {
     $group_name =~ s/^\@//; # Remove at
     my $value;
 
-    if ($group_name =~ "envarray") {
-      $value = $ENV{$option_name.$worker};
-    } elsif ($group_name =~ "env") {
+    if ($group_name =~ "env") {
       $value = $ENV{$option_name};
     } else {
       my $from_group= $config->group($group_name)
@@ -563,12 +558,12 @@ sub resolve_at_variable {
 }
 
 sub post_fix_resolve_at_variables {
-  my ($self, $config, $worker) = @_;
+  my ($self, $config) = @_;
 
   foreach my $group ($config->groups()) {
     foreach my $option ($group->options()) {
       next unless defined $option->value();
-      $self->resolve_at_variable($config, $group, $option, $worker)
+      $self->resolve_at_variable($config, $group, $option)
 	    if ($option->value() =~ /\@/);
     }
   }
@@ -758,7 +753,7 @@ sub run_generate_sections_from_cluster_config {
 sub new_config {
   my ($class, $args) = @_;
 
-  my @required_args = ('basedir', 'baseport', 'vardir', 'template_path', 'testdir', 'tmpdir', 'worker');
+  my @required_args = ('basedir', 'baseport', 'vardir', 'template_path', 'testdir', 'tmpdir');
 
   foreach my $required (@required_args) {
     croak "you must pass '$required'" unless defined $args->{$required};
@@ -813,8 +808,6 @@ sub new_config {
 
   $self->run_section_rules($config, 'mysqld.', @mysqld_rules);
 
-  $self->run_section_rules($config, 'memcached.', @memcached_rules);
-
   # [mysqlbinlog] need additional settings
   $self->run_rules_for_group($config, $config->insert('mysqlbinlog'),
                              @mysqlbinlog_rules);
@@ -838,12 +831,9 @@ sub new_config {
     push(@post_rules, \&post_check_secondary_engine_mysqld_group);
   }
 
-  # Worker ID
-  my $worker = $args->{'worker'};
-
   # Run post rules
   foreach my $rule (@post_rules) {
-    &$rule($self, $config, $worker);
+    &$rule($self, $config);
   }
 
   return $config;
