@@ -150,7 +150,7 @@ namespace galera
         void process_conf_change(void* recv_ctx, const struct gcs_action& cc);
         void process_state_req(void* recv_ctx, const void* req,
                                size_t req_size, wsrep_seqno_t seqno_l,
-                               wsrep_seqno_t donor_seq);
+                               wsrep_seqno_t donor_seq, const char* requestor_id);
         void process_join(wsrep_seqno_t seqno, wsrep_seqno_t seqno_l);
         void process_sync(wsrep_seqno_t seqno_l);
         void process_vote(wsrep_seqno_t seq, int64_t code,wsrep_seqno_t seqno_l);
@@ -193,9 +193,9 @@ namespace galera
 
         // IST Action handler interface
         void ist_trx(const TrxHandleSlavePtr& ts, bool must_apply,
-                     bool preload);
-        void ist_cc(const gcs_action&, bool must_apply, bool preload);
-        void ist_end(int error);
+                     bool preload) override;
+        void ist_cc(const gcs_action&, bool must_apply, bool preload) override;
+        void ist_end(const ist::Result&) override;
 
         // Cancel local and enter apply monitors for TrxHandle
         void cancel_monitors_for_local(const TrxHandleSlave& ts)
@@ -286,7 +286,7 @@ namespace galera
                 mutex_(),
                 cond_(),
                 eof_(false),
-                error_(0),
+                result_(0, ""),
                 queue_()
 #ifdef PXC
                 , processed_upto_(0)
@@ -294,13 +294,15 @@ namespace galera
             { }
 #ifdef PXC
             int is_eof() { return eof_; }
+            void reset() { eof_ = false; result_ = ist::Result{0, ""}; processed_upto_ = 0; }
+#else
+            void reset() { eof_ = false; result_ = ist::Result{0, ""}; }
 #endif /* PXC */
-            void reset() { eof_ = false; error_ = 0; processed_upto_ = 0;}
-            void eof(int error)
+            void eof(const ist::Result& result)
             {
                 gu::Lock lock(mutex_);
                 eof_ = true;
-                error_ = error;
+                result_ = result;
                 cond_.broadcast();
             }
 
@@ -344,12 +346,14 @@ namespace galera
                 }
                 else
                 {
-                    if (error_)
+                    if (result_.error)
                     {
-                        int err(error_);
-                        error_ = 0; // Make just one thread to detect the failure
+                        int err(result_.error);
+                        // Make just one thread to detect the failure
+                        result_.error = 0;
                         gu_throw_error(err)
-                            << "IST receiver reported failure";
+                            << "IST receiver reported failure: '"
+                            << result_.error_str << "' (" << err << ")";
                     }
                 }
 
@@ -367,7 +371,7 @@ namespace galera
             gu::Mutex mutex_;
             gu::Cond  cond_;
             bool eof_;
-            int error_;
+            ist::Result result_;
             std::queue<ISTEvent> queue_;
 #ifdef PXC
             wsrep_seqno_t processed_upto_;
@@ -970,7 +974,7 @@ namespace galera
 #endif /* PXC */
 
         /* resume reception of GCS events */
-        void resume_recv() { gcs_.resume_recv(); ist_end(0); }
+        void resume_recv() { gcs_.resume_recv(); ist_end(ist::Result{0, ""}); }
 
         /* These methods facilitate closing procedure.
          * They must be called under closing_mutex_ lock */
@@ -1030,6 +1034,7 @@ namespace galera
          * |                 9 | SS keys   4 |              2 |               2 |
          * | 4.x            10 | PA range/ 5 | CC events /  3 |               2 |
          * |                   | UPD keys    | idx preload    |                 |
+         * |                11 | SRV keys  6 |              3 |               2 |
          * |--------------------------------------------------------------------|
          *
          * Note: str_proto_ver is decided in replicator_str.cpp based on
