@@ -45,6 +45,7 @@ Config::Config (int argc, char* argv[])
       log_     (),
       cfg_     (),
       recv_script_ (),
+      post_recv_script_(),
       workdir_ (),
 #if defined(WITH_COREDUMPER) && WITH_COREDUMPER
       coredumper_ (),
@@ -72,6 +73,7 @@ Config::Config (int argc, char* argv[])
         ("options,o",   po::value<std::string>(&options_),     "GCS/GCOMM option list")
         ("log,l",       po::value<std::string>(&log_),         "Log file")
         ("recv-script", po::value<std::string>(&recv_script_), "SST request receive script")
+        ("post-recv-script", po::value<std::string>(&post_recv_script_), "Post SST script")
         ("workdir,w",po::value<std::string>(&workdir_),
          "Daemon working directory")
         ;
@@ -152,6 +154,7 @@ Config::Config (int argc, char* argv[])
     strip_quotes(group_);
     strip_quotes(sst_);
     strip_quotes(recv_script_);
+    strip_quotes(post_recv_script_);
     strip_quotes(donor_);
     strip_quotes(options_);
     strip_quotes(log_);
