@@ -52,9 +52,11 @@ namespace gcache
 
         while (seqno2ptr.index_begin() <= seqno && !seqno2ptr.empty())
         {
+#ifdef PXC
             /* Skip purge from this seqno onwards. */
             if (params.skip_purge(seqno2ptr.index_begin()))
                 return false;
+#endif /* PXC */
 
             BufferHeader* const bh(ptr2BH(seqno2ptr.front()));
 
@@ -145,9 +147,8 @@ namespace gcache
             {
                 log_fatal << "OOO release: seqno_released " << seqno_released
                           << ", releasing " << bh->seqno_g;
+                assert(0);
             }
-            assert(seqno_released + 1 == bh->seqno_g ||
-                   SEQNO_NONE == seqno_released);
 #endif
             new_released = bh->seqno_g;
         }
