@@ -44,13 +44,10 @@ namespace gcache
     {
         write_preamble(false);
 
-        for (seqno2ptr_iter_t i = seqno2ptr_.begin(); i != seqno2ptr_.end();)
+        for (seqno2ptr_iter_t i = seqno2ptr_.begin(); i != seqno2ptr_.end(); ++i)
         {
             if (ptr2BH(*i)->ctx == BH_ctx_t(this)) {
-                i = seqno2ptr_.erase(i);
-            }
-            else {
-                ++i;
+                seqno2ptr_.erase(i);
             }
         }
 
