@@ -1,4 +1,4 @@
-/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -605,8 +605,8 @@ void Client::pfs_end_state(uint32_t err_num, const char *err_mesg) {
   mysql_mutex_lock(&s_table_mutex);
   assert(s_num_clones == 1);
 
-  bool provisioning = (get_data_dir() == nullptr);
-  bool failed = (err_num != 0);
+  const bool provisioning = (get_data_dir() == nullptr);
+  const bool failed = (err_num != 0);
 
   /* In case provisioning is successful, clone operation is still
   in progress and will continue after restart. */
@@ -661,7 +661,7 @@ void Client::uninit_pfs() {
 
 uint32_t Client::limit_buffer(uint32_t buffer_size) {
   /* Limit total buffer size to 128 M */
-  uint32_t max_buffer_size = 128 * 1024 * 1024;
+  const uint32_t max_buffer_size = 128 * 1024 * 1024;
   auto num_tasks = get_max_concurrency();
 
   auto limit = max_buffer_size / num_tasks;
@@ -679,7 +679,7 @@ uint32_t Client::limit_workers(uint32_t num_workers) {
     /* Zero is also valid result for the limit. Workers are over and above
     the master task. So, anything less than 64M would mean no workers to
     spawn immediately. */
-    uint32_t limit = clone_max_network_bandwidth / 64;
+    const uint32_t limit = clone_max_network_bandwidth / 64;
     if (num_workers > limit) {
       num_workers = limit;
     }
@@ -691,7 +691,7 @@ uint32_t Client::limit_workers(uint32_t num_workers) {
     /* Zero is also valid result for the limit. Workers are over and above
     the master task. So, anything less than 64M would mean no workers to
     spawn immediately. */
-    uint32_t limit = clone_max_io_bandwidth / 64;
+    const uint32_t limit = clone_max_io_bandwidth / 64;
     if (num_workers > limit) {
       num_workers = limit;
     }
@@ -830,7 +830,7 @@ int Client::clone() {
     }
 
     /* If clone is successful, clear any error happened during exit. */
-    bool clear_err = (err == 0);
+    const bool clear_err = (err == 0);
     mysql_service_clone_protocol->mysql_clone_disconnect(get_thd(), m_conn,
                                                          abort_net, clear_err);
 
@@ -1083,8 +1083,19 @@ int Client::validate_remote_params() {
       continue;
     }
 
-    /* Built-in plugins with no shared object should already be installed. */
-    assert(!so_name.empty());
+    /*
+      Regardless of whether a plugin is builtin or not, donor will always
+      send plugin name (always non-empty) and dynamic library name (can be
+      empty).
+
+      The assumption is that if plugin is not installed, it must have
+      a corresponding dynamic library.
+
+      This is true for all plugins except mysql_native_password which
+      does not have a corresponding shared library but is disabled by
+      default.
+    */
+    assert(!so_name.empty() || plugin_name == "mysql_native_password");
 
     if (so_name.empty() || plugin_is_loadable(so_name)) {
       continue;
@@ -1154,7 +1165,7 @@ int Client::extract_string(const uchar *&packet, size_t &length,
     }
   }
   /* purecov: begin deadcode */
-  int err = ER_CLONE_PROTOCOL;
+  const int err = ER_CLONE_PROTOCOL;
   my_error(err, MYF(0), "Wrong Clone RPC response length for parameters");
   return (err);
   /* purecov: end */
@@ -1432,7 +1443,7 @@ int Client::receive_response(Command_RPC com, bool use_aux) {
   auto &info = get_thread_info();
 
   /* Skip setting returned locators for restart */
-  bool skip_apply = (com == COM_REINIT);
+  const bool skip_apply = (com == COM_REINIT);
 
   /* For graceful exit we wait for remote to send
   the end of command message */
@@ -1772,7 +1783,7 @@ int Client::set_error(const uchar *buffer, size_t length) {
   buffer += 4;
   length -= 4;
 
-  int err = ER_CLONE_DONOR;
+  const int err = ER_CLONE_DONOR;
 
   if (is_master()) {
     char err_buf[MYSYS_ERRMSG_SIZE];
