@@ -1,4 +1,4 @@
-/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -224,7 +224,7 @@ static bool scan_donor_list(const std::string &donor_list,
         entry_len = comma_pos - begin_pos;
       }
 
-      std::string entry = donor_list.substr(begin_pos, entry_len);
+      const std::string entry = donor_list.substr(begin_pos, entry_len);
       auto colon_pos = entry.find(":");
 
       /* Bad entry if no separator is found or found in beginning. */
@@ -334,7 +334,7 @@ static int check_donor_addr_format(MYSQL_THD thd, SYS_VAR *var [[maybe_unused]],
     /* purecov: end */
   }
 
-  std::string addrs(addrs_cstring);
+  const std::string addrs(addrs_cstring);
 
   Donor_Callback callback = [](std::string, uint32_t) { return (false); };
 
