@@ -1,4 +1,4 @@
-/* Copyright (c) 2013, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2013, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -43,6 +43,7 @@
 #ifndef NDEBUG
 #include "plugin/group_replication/include/sql_service/sql_command_test.h"
 #endif
+#include "string_with_len.h"
 
 /*
   Buffer to read the write_set value as a string.
@@ -50,26 +51,26 @@
 */
 #define BUFFER_READ_PKE 8
 
-void cleanup_transaction_write_set(
-    Transaction_write_set *transaction_write_set) {
+int add_write_set(Transaction_context_log_event *tcle, std::vector<uint64> *set,
+                  const THD *thd) {
   DBUG_TRACE;
-  if (transaction_write_set != nullptr) {
-    my_free(transaction_write_set->write_set);
-    my_free(transaction_write_set);
-  }
-}
-
-int add_write_set(Transaction_context_log_event *tcle,
-                  Transaction_write_set *set) {
-  DBUG_TRACE;
-  int iterator = set->write_set_size;
-  for (int i = 0; i < iterator; i++) {
+  for (std::vector<uint64>::iterator it = set->begin(); it != set->end();
+       ++it) {
     uchar buff[BUFFER_READ_PKE];
-    int8store(buff, set->write_set[i]);
+    int8store(buff, *it);
     uint64 const tmp_str_sz =
         base64_needed_encoded_length((uint64)BUFFER_READ_PKE);
     char *write_set_value =
         (char *)my_malloc(key_write_set_encoded, tmp_str_sz, MYF(MY_WME));
+    // key_write_set_encoded is being tracked for connection memory limits, if
+    // exceed thread will be killed
+    if (thd->is_killed()) {
+      LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_CONN_KILLED,
+                   "Generate write identification hash failed");
+      my_free(write_set_value);
+      return 1;
+    }
+
     if (!write_set_value) {
       /* purecov: begin inspected */
       LogPluginErr(ERROR_LEVEL,
@@ -123,14 +124,6 @@ int group_replication_trans_before_dml(Trans_param *param, int &out) {
     return 0;
   }
 
-  if ((out += (param->trans_ctx_info.transaction_write_set_extraction ==
-               HASH_ALGORITHM_OFF))) {
-    /* purecov: begin inspected */
-    LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_TRANS_WRITE_SET_EXTRACTION_NOT_SET);
-    return 0;
-    /* purecov: end */
-  }
-
   if (local_member_info->has_enforces_update_everywhere_checks() &&
       (out += (param->trans_ctx_info.tx_isolation == ISO_SERIALIZABLE))) {
     LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_UNSUPPORTED_TRANS_ISOLATION);
@@ -176,7 +169,6 @@ int group_replication_trans_before_commit(Trans_param *param) {
   int error = 0;
   const int pre_wait_error = 1;
   const int post_wait_error = 2;
-  int64 last_committed = 0;
   int64 sequence_number = 1;
 
   DBUG_EXECUTE_IF("group_replication_force_error_on_before_commit_listener",
@@ -289,15 +281,20 @@ int group_replication_trans_before_commit(Trans_param *param) {
   const ulong transaction_size_limit = get_transaction_size_limit();
   my_off_t transaction_size = 0;
 
-  const bool is_gtid_specified = param->gtid_info.type == ASSIGNED_GTID;
+  bool is_gtid_specified = param->gtid_info.type == ASSIGNED_GTID;
+
+  mysql::gtid::Tag_plain automatic_tag;
+  automatic_tag.clear();
+
   Gtid gtid = {param->gtid_info.sidno, param->gtid_info.gno};
   if (!is_gtid_specified) {
-    // Dummy values that will be replaced after certification.
+    // sidno and gno are dummy values that will be replaced after certification
     gtid.sidno = 1;
     gtid.gno = 1;
+    automatic_tag = param->gtid_info.automatic_tag;
   }
 
-  const Gtid_specification gtid_specification = {ASSIGNED_GTID, gtid};
+  Gtid_specification gtid_specification = {ASSIGNED_GTID, gtid, automatic_tag};
   Gtid_log_event *gle = nullptr;
 
   Transaction_context_log_event *tcle = nullptr;
@@ -363,14 +360,16 @@ int group_replication_trans_before_commit(Trans_param *param) {
   }
 
   if (is_dml) {
-    Transaction_write_set *write_set =
-        get_transaction_write_set(param->thread_id);
+    Rpl_transaction_write_set_ctx *transaction_write_set_ctx =
+        param->thd->get_transaction()->get_transaction_write_set_ctx();
+
+    std::vector<uint64> *write_set = transaction_write_set_ctx->get_write_set();
     /*
       When GTID is specified we may have empty transactions, that is,
       a transaction may have not write set at all because it didn't
       change any data, it will just persist that GTID as applied.
     */
-    if ((write_set == nullptr) && (!is_gtid_specified) &&
+    if ((write_set->size() == 0) && (!is_gtid_specified) &&
         (!param->is_create_table_as_query_block)) {
       LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_FAILED_TO_EXTRACT_TRANS_WRITE_SET,
                    param->thread_id);
@@ -378,17 +377,13 @@ int group_replication_trans_before_commit(Trans_param *param) {
       goto err;
     }
 
-    if (write_set != nullptr) {
-      if (add_write_set(tcle, write_set)) {
-        /* purecov: begin inspected */
-        cleanup_transaction_write_set(write_set);
+    if (write_set->size() > 0) {
+      if (add_write_set(tcle, write_set, param->thd)) {
         LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_FAILED_TO_GATHER_TRANS_WRITE_SET,
                      param->thread_id);
         error = pre_wait_error;
         goto err;
-        /* purecov: end */
       }
-      cleanup_transaction_write_set(write_set);
       assert(is_gtid_specified || (tcle->get_write_set()->size() > 0));
     } else {
       /*
@@ -396,15 +391,6 @@ int group_replication_trans_before_commit(Trans_param *param) {
         comment at binlog_cache_data::may_have_sbr_stmts().
       */
       may_have_sbr_stmts = true;
-
-      /*
-        Empty transactions, despite not having write-set, can be
-        applied in parallel with any other transaction, as such we
-        flag those transactions with `last_committed = -1`.
-      */
-      if (is_gtid_specified && !param->is_create_table_as_query_block) {
-        last_committed = -1;
-      }
     }
 
     DBUG_EXECUTE_IF("group_replication_after_add_write_set", {
@@ -460,8 +446,8 @@ int group_replication_trans_before_commit(Trans_param *param) {
 
   // Notice the GTID of atomic DDL is written to the trans cache as well.
   gle = new Gtid_log_event(
-      param->server_id, is_dml || param->is_atomic_ddl, last_committed,
-      sequence_number, may_have_sbr_stmts, *(param->original_commit_timestamp),
+      param->server_id, is_dml || param->is_atomic_ddl, 0, sequence_number,
+      may_have_sbr_stmts, *(param->original_commit_timestamp),
       immediate_commit_timestamp, gtid_specification,
       *(param->original_server_version), *(param->immediate_server_version));
   /*
@@ -505,6 +491,15 @@ int group_replication_trans_before_commit(Trans_param *param) {
           transaction_size, consistency_level);
     }
 
+    // transaction_message use memory key being tracked for connection limits,
+    // if exceed thread will be killed
+    if (param->thd->is_killed()) {
+      LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_CONN_KILLED,
+                   "Generate transaction message failed");
+      error = pre_wait_error;
+      goto err;
+    }
+
     if (binary_event_serialize(tcle, transaction_msg) ||
         binary_event_serialize(gle, transaction_msg)) {
       LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_WRITE_TO_TRANSACTION_MESSAGE_FAILED,
@@ -707,7 +702,7 @@ int group_replication_trans_begin(Trans_param *param, int &out) {
        *transaction_observers) {
     out = transaction_observer->before_transaction_begin(
         param->thread_id, param->group_replication_consistency,
-        param->hold_timeout, param->rpl_channel_type);
+        param->hold_timeout, param->rpl_channel_type, param->thd);
     if (out) break;
   }
   group_transaction_observation_manager->unlock_observer_list();
