@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -32,14 +32,13 @@
 #include <sys/types.h>
 #include <string>
 
-#include "m_string.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
-#include "my_loglevel.h"
 #include "my_sys.h"
 #include "my_thread.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/components/services/log_shared.h"
+#include "mysql/my_loglevel.h"
 #include "mysql/psi/mysql_file.h"
 #include "mysql/psi/mysql_thread.h"
 #include "mysql_com.h"
@@ -53,6 +52,7 @@
 #include "sql/mysqld_thd_manager.h"  // Global_THD_manager
 #include "sql/protocol_classic.h"
 #include "sql/query_options.h"
+#include "sql/sd_notify.h"  // for sysd::notify(..) calls
 #include "sql/set_var.h"
 #include "sql/sql_bootstrap.h"
 #include "sql/sql_class.h"    // THD
@@ -66,6 +66,7 @@
 #include "sql/system_variables.h"
 #include "sql/thd_raii.h"
 #include "sql/transaction_info.h"
+#include "string_with_len.h"
 
 namespace bootstrap {
 
@@ -141,6 +142,8 @@ static bool handle_bootstrap_impl(handle_bootstrap_args *args) {
     */
     assert(thd->system_thread == SYSTEM_THREAD_SERVER_INITIALIZE);
 
+    sysd::notify("STATUS=Initialization of MySQL system tables in progress\n");
+
     /*
       The server must avoid logging compiled statements into the binary log
       (and generating GTIDs for them when GTID_MODE is ON) during bootstrap/
@@ -148,14 +151,17 @@ static bool handle_bootstrap_impl(handle_bootstrap_args *args) {
       We disable SQL_LOG_BIN session variable while processing compiled
       statements.
     */
-    Disable_binlog_guard disable_binlog(thd);
-    Disable_sql_log_bin_guard disable_sql_log_bin(thd);
+    const Disable_binlog_guard disable_binlog(thd);
+    const Disable_sql_log_bin_guard disable_sql_log_bin(thd);
 
     Compiled_in_command_iterator comp_iter;
     rc = process_iterator(thd, &comp_iter, true);
 
     thd->system_thread = SYSTEM_THREAD_INIT_FILE;
 
+    sysd::notify("STATUS=Initialization of MySQL system tables ",
+                 rc ? "unsuccessful" : "successful", "\n");
+
     if (rc != 0) {
       return true;
     }
@@ -170,9 +176,15 @@ static bool handle_bootstrap_impl(handle_bootstrap_args *args) {
     */
     assert(thd->system_thread == SYSTEM_THREAD_INIT_FILE);
 
+    sysd::notify(
+        "STATUS=Execution of SQL Commands from Init-file in progress\n");
+
     File_command_iterator file_iter(args->m_file_name, args->m_file,
                                     mysql_file_fgets_fn);
     rc = process_iterator(thd, &file_iter, false);
+
+    sysd::notify("STATUS=Execution of SQL Commands from Init-file ",
+                 rc ? "unsuccessful" : "successful", "\n");
     if (rc != 0) {
       return true;
     }
@@ -441,8 +453,8 @@ bool run_bootstrap_thread(const char *file_name, MYSQL_FILE *file,
 
   my_thread_handle thread_handle;
   // What about setting THD::real_id?
-  int error = mysql_thread_create(key_thread_bootstrap, &thread_handle,
-                                  &thr_attr, handle_bootstrap, &args);
+  const int error = mysql_thread_create(key_thread_bootstrap, &thread_handle,
+                                        &thr_attr, handle_bootstrap, &args);
   if (error) {
     /* purecov: begin inspected */
     LogErr(WARNING_LEVEL, ER_BOOTSTRAP_CANT_THREAD, errno).os_errno(errno);
