@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2013, 2025, Oracle and/or its affiliates.
+   Copyright (c) 2013, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -92,6 +92,11 @@ class Per_thread_connection_handler : public Connection_handler {
   */
   static void modify_thread_cache_size(const ulong thread_cache_size);
 
+  /**
+    Return blocked threads count.
+  */
+  static ulong get_blocked_pthread_count();
+
   Per_thread_connection_handler() = default;
   ~Per_thread_connection_handler() override = default;
 
