@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -42,6 +42,7 @@ class THD;
 namespace dd {
 class Schema;
 }  // namespace dd
+struct CHARSET_INFO;
 struct TABLE;
 
 class Sql_check_constraint_spec;
@@ -480,22 +481,22 @@ bool rename_check_constraints(const char *old_table_name, dd::Table *new_tab);
 bool uses_general_tablespace(const Table &t);
 
 /**
-  Throw deprecation warnings if table uses prefix keys in the partitioning
-  function.
+  Check and give error if table uses prefix keys in the partitioning function.
 
-  @param  thd             Thread handler
   @param  schema_name     Schema name
   @param  orig_table_name Original table name (required in case of ALTER TABLE,
   since temporary table name is created)
   @param  table           dd::Table instance
-  @param  is_upgrade      True if this is called during upgrade. Warning will be
+  @param  is_upgrade      True if this is called during upgrade. Error will be
   sent to error log instead of the client.
 
+  @return true  - On failure
+  @return false - On success
+
 */
-void warn_on_deprecated_prefix_key_partition(THD *thd, const char *schema_name,
-                                             const char *orig_table_name,
-                                             const Table *table,
-                                             const bool is_upgrade);
+bool prefix_key_partition_exists(const char *schema_name,
+                                 const char *orig_table_name,
+                                 const Table *table, const bool is_upgrade);
 
 /**
   Get the autoextend_size option value for implicit tablespaces
@@ -510,20 +511,12 @@ bool get_implicit_tablespace_options(THD *thd, const Table *table,
                                      ulonglong *autoextend_size);
 
 /**
-  Check whether a table has constraints, virtual columns,
-  default, or partitions that may use functions. If so, we may
-  wish to try to open the table during upgrade to see whether
-  we can (or whether any of the functions have changed in such
-  a way that opening the table is no longer possible).
+  Validate if table uses foreign keys referring to proper index.
+  FK cannot refer to non unique index and partial elements in index.
 
-  @param  table_def    The table to examine.
-  @param  schema_name  The name of the schema, or nullptr.
-  @param  debug        A String_type to return debug_info in, or nullptr.
-  @return true         The table definition potentially uses functions.
-  @return false        The table is basic.
+  @param  thd             Thread handle.
+  @param  table           dd::Table instance of referenced table
 */
-bool uses_functions(const Table *table_def, const char *schema_name = nullptr,
-                    String_type *debug = nullptr);
-
+bool check_non_standard_key_exists_in_fk(THD *thd, const Table *table);
 }  // namespace dd
 #endif  // DD_TABLE_INCLUDED
