@@ -1,4 +1,4 @@
-/* Copyright (c) 2019, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2019, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -25,26 +25,42 @@
 
 #include <errno.h>
 #include <fcntl.h>
+#include <stdint.h>
 #include <sys/types.h>
+#include <chrono>
+#include <iomanip>
+#include <sstream>
+#include <string>
 
-#include "server.h"
 #include "sql/dd/upgrade/server.h"
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #include <vector>
 
-#include "lex_string.h"
 #include "my_dbug.h"
+#include "my_rapidjson_size_t.h"
 #include "mysql/components/services/log_builtins.h"
+#include "mysql/psi/mysql_file.h"
+#include "mysql/strings/m_ctype.h"
+#include "nulls.h"
+#include "rapidjson/document.h"
+#include "rapidjson/prettywriter.h"
+#include "rapidjson/stringbuffer.h"
 #include "scripts/mysql_fix_privilege_tables_sql.h"
 #include "scripts/sql_commands_system_tables_data_fix.h"
-#include "scripts/sql_firewall_stored_procedures.h"
+#include "scripts/sql_firewall_sp_firewall_group_delist.h"
+#include "scripts/sql_firewall_sp_firewall_group_enlist.h"
+#include "scripts/sql_firewall_sp_reload_firewall_group_rules.h"
+#include "scripts/sql_firewall_sp_reload_firewall_rules.h"
+#include "scripts/sql_firewall_sp_set_firewall_group_mode.h"
+#include "scripts/sql_firewall_sp_set_firewall_group_mode_and_user.h"
+#include "scripts/sql_firewall_sp_set_firewall_mode.h"
 #include "sql/dd/cache/dictionary_client.h"  // dd::cache::Dictionary_client
 #include "sql/dd/dd_schema.h"                // dd::Schema_MDL_locker
-#include "sql/dd/dd_table.h"  // dd::warn_on_deprecated_prefix_key_partition
-#include "sql/dd/dd_tablespace.h"                 // dd::fill_table_and_parts...
-#include "sql/dd/dd_trigger.h"                    // dd::create_trigger
+#include "sql/dd/dd_table.h"                 // dd::prefix_key_partition_exists
+#include "sql/dd/dd_tablespace.h"            // dd::fill_table_and_parts...
+#include "sql/dd/dd_trigger.h"               // dd::create_trigger
 #include "sql/dd/impl/bootstrap/bootstrap_ctx.h"  // dd::DD_bootstrap_ctx
 #include "sql/dd/impl/bootstrap/bootstrapper.h"
 #include "sql/dd/impl/tables/dd_properties.h"  // dd::tables::DD_properties
@@ -60,18 +76,20 @@
 #include "sql/sp.h"         // Stored_routine_creation_ctx
 #include "sql/sp_head.h"    // sp_head
 #include "sql/sql_base.h"
-#include "sql/sql_prepare.h"
+#include "sql/sql_table.h"
+#include "sql/statement/ed_connection.h"
 #include "sql/strfunc.h"
 #include "sql/table_trigger_dispatcher.h"  // Table_trigger_dispatcher
 #include "sql/thd_raii.h"
 #include "sql/trigger.h"  // Trigger
 #include "sql/trigger_def.h"
+#include "string_with_len.h"
 
 #ifdef HAVE_PERCONA_TELEMETRY
 #include "sql/dd/dd_utility.h"  // check_if_server_ddse_readonly
 #endif
 
-typedef ulonglong sql_mode_t;
+using sql_mode_t = uint64_t;
 extern const char *mysql_sys_schema[];
 extern const char *fill_help_tables[];
 
@@ -79,11 +97,6 @@ const char *upgrade_modes[] = {"NONE", "MINIMAL", "AUTO", "FORCE", NullS};
 TYPELIB upgrade_mode_typelib = {array_elements(upgrade_modes) - 1, "",
                                 upgrade_modes, nullptr};
 
-const char *check_table_fun_modes[] = {"WARN", "ABORT", NullS};
-TYPELIB check_table_fun_mode_typelib = {
-    array_elements(check_table_fun_modes) - 1, "", check_table_fun_modes,
-    nullptr};
-
 namespace dd {
 namespace upgrade {
 
@@ -196,6 +209,22 @@ bool Syntax_error_handler::handle_condition(
   return false;
 }
 
+void Syntax_error_handler::reset_last_condition() {
+  // This method was created to handle a corner case in upgrades to 8.4
+  // where old servers contain routines with unsupported terminology.
+  // TODO: As upgrades to 9.0 only come from 8.4+, this code can be removed
+  // in 9.0
+  assert(
+      dd::bootstrap::DD_bootstrap_ctx::instance().is_server_upgrade_from_before(
+          bootstrap::SERVER_VERSION_80400) &&
+      strstr(reason.c_str(), " to use near 'SLAVE STATUS"));
+
+  parse_error_count--;
+  if (m_global_counter) (*m_global_counter)--;
+  is_parse_error = false;
+  reason = "";
+}
+
 bool Syntax_error_handler::has_too_many_errors() {
   return parse_error_count > MAX_SERVER_CHECK_FAILS;
 }
@@ -216,6 +245,10 @@ Upgrade_error_counter Upgrade_error_counter::operator++(int) {
   m_error_count++;
   return *this;
 }
+Upgrade_error_counter Upgrade_error_counter::operator--(int) {
+  m_error_count--;
+  return *this;
+}
 
 namespace {
 
@@ -432,9 +465,11 @@ class MySQL_check {
 
 #ifdef WITH_WSREP
 bool ignore_error_and_execute(THD *thd, const char *query_ptr,
+                              bool print_err = true,
                               const bool pxc_upgrade = false) {
 #else
-bool ignore_error_and_execute(THD *thd, const char *query_ptr) {
+bool ignore_error_and_execute(THD *thd, const char *query_ptr,
+                              bool print_err = true) {
 #endif /* WITH_WSREP */
   Ed_connection con(thd);
   LEX_STRING str;
@@ -447,8 +482,9 @@ bool ignore_error_and_execute(THD *thd, const char *query_ptr) {
 #ifdef WITH_WSREP
     if (!pxc_upgrade) {
 #endif /* WITH_WSREP */
-      LogErr(ERROR_LEVEL, ER_DD_INITIALIZE_SQL_ERROR, query_ptr,
-             con.get_last_errno(), con.get_last_error());
+      if (print_err)
+        LogErr(ERROR_LEVEL, ER_DD_INITIALIZE_SQL_ERROR, query_ptr,
+              con.get_last_errno(), con.get_last_error());
       return true;
 #ifdef WITH_WSREP
     }
@@ -457,10 +493,103 @@ bool ignore_error_and_execute(THD *thd, const char *query_ptr) {
   return false;
 }
 
+/**
+ * This function will create the firewall's stored procedures.
+ *
+ * @param[in]        thd                thread context
+ * @param[in]        drop_query         DROP statement to drop procedure
+ * @param[in]        fw_proc            stored procedure's SQL definition
+ *
+ * @retval           false              execution of query successful
+ * @retval           true               execution of query failed
+ */
+static bool reinstall_firewall_procedures(THD *thd, const char *drop_query,
+                                          const char *fw_proc[]) {
+  if (!ignore_error_and_execute(thd, drop_query, false))
+    for (auto query = fw_proc; *query != nullptr; query++)
+      if (ignore_error_and_execute(thd, *query)) return true;
+
+  return false;
+}
+
+/**
+ * This function will check and create the firewall's stored procedures.
+ *
+ * @param[in]        thd                thread context
+ *
+ * @retval           false              execution of query successful
+ * @retval           true               execution of query failed
+ */
+static bool upgrade_firewall_procedures(THD *thd) {
+  struct firewall_installer {
+    const char *drop_query;
+    const char **fwproc;
+  };
+
+  static firewall_installer fw_commands[] = {
+      {"DROP PROCEDURE sp_set_firewall_mode", firewall_sp_set_firewall_mode},
+      {"DROP PROCEDURE sp_reload_firewall_rules",
+       firewall_sp_reload_firewall_rules},
+      {"DROP PROCEDURE sp_set_firewall_group_mode",
+       firewall_sp_set_firewall_group_mode},
+      {"DROP PROCEDURE sp_set_firewall_group_mode_and_user",
+       firewall_sp_set_firewall_group_mode_and_user},
+      {"DROP PROCEDURE sp_reload_firewall_group_rules",
+       firewall_sp_reload_firewall_group_rules},
+      {"DROP PROCEDURE sp_firewall_group_enlist",
+       firewall_sp_firewall_group_enlist},
+      {"DROP PROCEDURE sp_firewall_group_delist",
+       firewall_sp_firewall_group_delist},
+  };
+
+  for (auto &fw : fw_commands)
+    if (reinstall_firewall_procedures(thd, fw.drop_query, fw.fwproc))
+      return true;
+
+  return false;
+}
+
+/**
+ * This function will switch to the schema which is pointed by the
+ * mysql-firewall-database variable by executing USE.
+ *
+ * @param[in]        thd                thread context
+ * @param[out]       fw_schema          value of mysql-firewall-database
+ *
+ * @retval           false              execution of USE successful
+ * @retval           true               execution of USE failed
+ */
+static bool switch_to_firewall_schema(THD *thd, std::string &fw_schema) {
+  LEX_STRING firewall_schema_name = {nullptr, 0};
+  Ed_connection conn(thd);
+
+  lex_string_strmake(thd->mem_root, &firewall_schema_name,
+                     STRING_WITH_LEN("SELECT @@mysql_firewall_database"));
+
+  if (conn.execute_direct(firewall_schema_name)) return true;
+
+  const List<Ed_row> rows = *(conn.get_result_sets());
+  const MYSQL_LEX_STRING *result = rows[0]->get_column(0);
+  fw_schema = result->str;
+
+  // if firewall is installed in a schema other than mysql
+  // then switch to that schema
+
+  std::string command = "USE ";
+  command.append(fw_schema);  // forms USE <schema>
+
+  if (strcmp("mysql", fw_schema.c_str()))
+    if (ignore_error_and_execute(thd, command.c_str())) return true;
+
+  return false;
+}
+
 /** upgrades Firewall stored procedures */
 static bool upgrade_firewall(THD *thd) {
   bool has_old_firewall_tables{false};
   bool has_new_firewall_tables{false};
+  bool error{false};
+  std::string fw_schema("mysql");
 
   {
     // lock required tables before checking their existence
@@ -471,7 +600,7 @@ static bool upgrade_firewall(THD *thd) {
                      "firewall_groups", MDL_SHARED, MDL_TRANSACTION);
 
     // check whether firewall tables exist
-    bool error =
+    error =
         (thd->mdl_context.acquire_lock(&request1,
                                        thd->variables.lock_wait_timeout) ||
          thd->mdl_context.acquire_lock(&request2,
@@ -486,13 +615,17 @@ static bool upgrade_firewall(THD *thd) {
     if (error) return true;
   }
 
-  // upgrade stored procedures, leave on error
-  if (has_old_firewall_tables && !has_new_firewall_tables)
-    for (auto query = &firewall_stored_procedures[0]; *query != nullptr;
-         query++)
-      if (ignore_error_and_execute(thd, *query)) return true;
+  // upgrade the procedures
+  if (has_old_firewall_tables || has_new_firewall_tables) {
+    error = switch_to_firewall_schema(thd, fw_schema) ||
+            upgrade_firewall_procedures(thd);
 
-  return false;
+    // we might have switched to another schema during fw upgrade
+    // go back to mysql
+    if (fw_schema != "mysql")
+      error |= ignore_error_and_execute(thd, "USE mysql");
+  }
+  return error;
 }
 
 bool fix_sys_schema(THD *thd) {
@@ -522,7 +655,10 @@ bool fix_sys_schema(THD *thd) {
     return false;
 
   const char **query_ptr;
-  LogErr(INFORMATION_LEVEL, ER_SERVER_UPGRADE_SYS_SCHEMA);
+  LogErr(INFORMATION_LEVEL,
+         dd::bootstrap::DD_bootstrap_ctx::instance().is_server_patch_downgrade()
+             ? ER_SERVER_DOWNGRADE_SYS_SCHEMA
+             : ER_SERVER_UPGRADE_SYS_SCHEMA);
   for (query_ptr = &mysql_sys_schema[0]; *query_ptr != nullptr; query_ptr++)
     if (ignore_error_and_execute(thd, *query_ptr)) return true;
   thd->mem_root->Clear();
@@ -565,36 +701,38 @@ bool fix_mysql_tables(THD *thd) {
 }
 
 bool upgrade_help_tables(THD *thd) {
+  // know if it's upgrade or downgrade
+  bool is_downgrade =
+      dd::bootstrap::DD_bootstrap_ctx::instance().is_server_patch_downgrade();
+
   if (dd::execute_query(thd, "USE mysql")) {
     LogErr(ERROR_LEVEL, ER_DD_UPGRADE_FAILED_FIND_VALID_DATA_DIR);
     return true;
   }
-  LogErr(INFORMATION_LEVEL, ER_SERVER_UPGRADE_HELP_TABLE_STATUS, "started");
+
+  LogErr(INFORMATION_LEVEL,
+         is_downgrade ? ER_SERVER_DOWNGRADE_HELP_TABLE_STATUS
+                      : ER_SERVER_UPGRADE_HELP_TABLE_STATUS,
+         "started");
+
   for (const char **query_ptr = &fill_help_tables[0]; *query_ptr != nullptr;
        query_ptr++)
     if (dd::execute_query(thd, *query_ptr)) {
-      LogErr(ERROR_LEVEL, ER_SERVER_UPGRADE_HELP_TABLE_STATUS, "failed");
+      LogErr(ERROR_LEVEL,
+             is_downgrade ? ER_SERVER_DOWNGRADE_HELP_TABLE_STATUS
+                          : ER_SERVER_UPGRADE_HELP_TABLE_STATUS,
+             "failed");
       return true;
     }
-  LogErr(INFORMATION_LEVEL, ER_SERVER_UPGRADE_HELP_TABLE_STATUS, "completed");
+
+  LogErr(INFORMATION_LEVEL,
+         is_downgrade ? ER_SERVER_DOWNGRADE_HELP_TABLE_STATUS
+                      : ER_SERVER_UPGRADE_HELP_TABLE_STATUS,
+         "completed");
+
   return false;
 }
 
-static void create_upgrade_file() {
-  FILE *out;
-  char upgrade_info_file[FN_REFLEN] = {0};
-  fn_format(upgrade_info_file, "mysql_upgrade_info", mysql_real_data_home_ptr,
-            "", MYF(0));
-
-  if ((out = my_fopen(upgrade_info_file, O_TRUNC | O_WRONLY, MYF(0)))) {
-    /* Write new version to file */
-    fputs(MYSQL_SERVER_VERSION, out);
-    my_fclose(out, MYF(0));
-    return;
-  }
-  LogErr(WARNING_LEVEL, ER_SERVER_UPGRADE_INFO_FILE, upgrade_info_file);
-}
-
 static bool get_shared_tablespace_names(
     THD *thd, std::set<dd::String_type> *shared_spaces) {
   assert(innodb_hton != nullptr && innodb_hton->get_tablespace_type);
@@ -614,128 +752,6 @@ static bool get_shared_tablespace_names(
   return thd->dd_client()->foreach<dd::Tablespace>(nullptr, process_spaces);
 }
 
-/*
-  SQL error handler to use during check_table_funs, i.e.
-  while we check table definitions for newly-broken SQL functions.
-  It downgrades errors to warnings, and increases a counter each
-  time each time it does so.
-*/
-class Sql_fun_error_handler : public Internal_error_handler {
-  uint *m_error_count;  ///< Count of downgraded errors.
-
- public:
-  /**
-     Creates a new Sql_fun_error_handler.
-
-     @param error_count  Address of the error count
-  */
-  explicit Sql_fun_error_handler(uint *error_count)
-      : m_error_count(error_count) {}
-
- public:
-  bool handle_condition(THD *, uint, const char *,
-                        Sql_condition::enum_severity_level *sl,
-                        const char *) override {
-    if (*sl == Sql_condition::SL_ERROR) {
-      (*m_error_count)++;
-      return true;
-    }
-    return false;
-  }
-};
-
-/**
-  Check table definitions for SQL functions.
-
-  Sometimes, improving a SQL function's behaviour or diagnostics
-  may result in an error being thrown in situations where this
-  wasn't the case. If that call is part of a table's definition,
-  the error will be thrown during the opening of the table,
-  which will then fail.
-
-  This is something the user will generally want to know before
-  putting an upgraded database into production. Therefore, we
-  inspect all user tables that (potentially) have SQL functions
-  in DEFAULT clauses, PARTITIONing, virtual columns, or indexes.
-  Any such table we try to open. This may take a while, but it
-  is preferable to not being aware of breakage.
-
-  @param  thd          The THD to use.
-  @param  schema       The schema whose tables to examine.
-  @param  error_count  Count of errors (total for all check functions).
-  @return false        True if too many errors were detected, false otherwise.
-*/
-static bool check_table_funs(THD *thd, std::unique_ptr<Schema> &schema,
-                             Upgrade_error_counter *error_count) {
-  uint sql_fun_errors = 0;
-
-  // Function called on each table to validate it.
-  auto process_cf_table = [&](std::unique_ptr<dd::Table> &table) {
-    // Skip non-InnoDB tables as their search engine may not be available yet.
-    if (my_strcasecmp(system_charset_info, table->engine().c_str(), "InnoDB"))
-      return false;
-
-    // Are SQL functions used in table def (defaults, virtual columns, etc.)?
-    if (dd::uses_functions(table.get(), schema->name().c_str())) {
-      Open_table_context ot_ctx(
-          thd, MYSQL_OPEN_GET_NEW_TABLE | MYSQL_OPEN_NO_NEW_TABLE_IN_SE);
-
-      char db_name[NAME_LEN + 1];
-      char table_name[NAME_LEN + 1];
-      my_stpncpy(db_name, schema->name().c_str(), NAME_LEN);
-      my_stpncpy(table_name, table->name().c_str(), NAME_LEN);
-
-      if (lower_case_table_names > 1) {
-        my_casedn_str(system_charset_info, db_name);
-        my_casedn_str(system_charset_info, table_name);
-      }
-
-      Table_ref tr(db_name, table_name, TL_READ);
-
-      // Did trying to open this table throw any new errors?
-      uint old_errors = sql_fun_errors;
-      open_table(thd, &tr, &ot_ctx);
-
-      // Did we catch any errors that would have prevented open_table()?
-      if (sql_fun_errors > old_errors) {
-        // Log that the table has problems.
-        LogErr(WARNING_LEVEL, ER_CHECK_TABLE_FUNCTIONS, schema->name().c_str(),
-               table->name().c_str());
-
-        // On higher log levels, create a detailed description of the table.
-        dd::String_type debug_info;
-        dd::uses_functions(table.get(), schema->name().c_str(), &debug_info);
-        LogErr(INFORMATION_LEVEL, ER_CHECK_TABLE_FUNCTIONS_DETAIL,
-               debug_info.c_str());
-
-        // increase global error count
-        if (opt_check_table_funs == CHECK_TABLE_FUN_ABORT) (*error_count)++;
-      }
-    }
-
-    return error_count->has_too_many_errors();
-  };
-
-  // Skip pfs.
-  if (0 == schema->name().compare("performance_schema")) return false;
-
-  // Our error handler counts errors and downgrades them to warnings.
-  Sql_fun_error_handler error_handler(&sql_fun_errors);
-  thd->push_internal_handler(&error_handler);
-
-  std::unique_ptr<dd::Object_key> table_key(
-      dd::Table::DD_table::create_key_by_schema_id(schema->id()));
-
-  // Iterate over tables in this schema.
-  bool res =
-      thd->dd_client()->foreach<dd::Table>(table_key.get(), process_cf_table);
-
-  // Clean up.
-  thd->pop_internal_handler();
-
-  return res;
-}
-
 static bool check_tables(THD *thd, std::unique_ptr<Schema> &schema,
                          const std::set<dd::String_type> *shared_spaces,
                          Upgrade_error_counter *error_count) {
@@ -754,8 +770,11 @@ static bool check_tables(THD *thd, std::unique_ptr<Schema> &schema,
     }
 
     // Check for usage of prefix key index in PARTITION BY KEY() function.
-    dd::warn_on_deprecated_prefix_key_partition(
-        thd, schema->name().c_str(), table->name().c_str(), table.get(), true);
+    if (dd::prefix_key_partition_exists(
+            schema->name().c_str(), table->name().c_str(), table.get(), true))
+      return true;
+
+    dd::check_non_standard_key_exists_in_fk(thd, table.get());
 
     // Check for partitioned innodb tables using shared spaces.
     if (!shared_spaces->empty() &&
@@ -776,7 +795,21 @@ static bool check_tables(THD *thd, std::unique_ptr<Schema> &schema,
         }
       }
     }
+
+    // Check if AUTO_INCREMENT is used with DOUBLE/FLOAT
+    for (const auto &col : *table->columns()) {
+      if (col->is_auto_increment() &&
+          (col->type() == enum_column_types::DOUBLE ||
+           col->type() == enum_column_types::FLOAT)) {
+        (*error_count)++;
+        LogErr(ERROR_LEVEL, ER_AUTO_INCREMENT_NOT_SUPPORTED_FOR_FLOAT_DOUBLE,
+               schema->name().c_str(), table->name().c_str(),
+               col->name().c_str());
+      }
+    }
+
     DBUG_EXECUTE_IF("upgrade_failed_during_init", (*error_count)++;);
+
     return error_count->has_too_many_errors();
   };
 
@@ -804,15 +837,30 @@ static bool check_events(THD *thd, std::unique_ptr<Schema> &schema,
 }
 
 static bool check_routines(THD *thd, std::unique_ptr<Schema> &schema,
+                           Syntax_error_handler &error_handler,
                            Upgrade_error_counter *error_count) {
   std::unique_ptr<Object_key> routine_key(
       dd::Routine::DD_table::create_key_by_schema_id(schema->id()));
 
   auto process_routine = [&](std::unique_ptr<dd::Routine> &routine) {
-    if (invalid_routine(thd, *schema, *routine))
-      LogErr(ERROR_LEVEL, ER_UPGRADE_PARSE_ERROR, "Routine",
-             schema->name().c_str(), routine->name().c_str(),
-             Syntax_error_handler::error_message());
+    if (invalid_routine(thd, *schema, *routine)) {
+      // This is a corner case in upgrades to 8.4 where old servers contain
+      // routines with unsupported terminology.
+      // TODO: As upgrades to 9.0 only come from 8.4+, this code can be removed
+      // in 9.0
+      if (dd::bootstrap::DD_bootstrap_ctx::instance()
+              .is_server_upgrade_from_before(bootstrap::SERVER_VERSION_80400) &&
+          schema->name() == "sys" && routine->name() == "diagnostics" &&
+          strstr(Syntax_error_handler::error_message(),
+                 " to use near 'SLAVE STATUS")) {
+        error_handler.reset_last_condition();
+        thd->clear_error();
+      } else {
+        LogErr(ERROR_LEVEL, ER_UPGRADE_PARSE_ERROR, "Routine",
+               schema->name().c_str(), routine->name().c_str(),
+               Syntax_error_handler::error_message());
+      }
+    }
     return error_count->has_too_many_errors();
   };
 
@@ -836,6 +884,14 @@ static bool check_views(THD *thd, std::unique_ptr<Schema> &schema,
   return thd->dd_client()->foreach<dd::View>(view_key.get(), process_view);
 }
 
+/* Make sure the old unsupported "mysql_upgrade_info" file is removed. */
+static void remove_legacy_upgrade_info_file() {
+  char upgrade_file[FN_REFLEN] = {0};
+  fn_format(upgrade_file, "mysql_upgrade_info", mysql_real_data_home_ptr, "",
+            MYF(0));
+  if (!my_access(upgrade_file, F_OK))
+    std::ignore = mysql_file_delete(key_file_misc, upgrade_file, MYF(0));
+}
 }  // namespace
 
 #ifdef WITH_WSREP
@@ -862,7 +918,7 @@ bool pxc_fix_mysql_tables(THD *thd) {
     }
 
     if (in_pxc_section) {
-      if (ignore_error_and_execute(thd, *query_ptr, true)) return true;
+      if (ignore_error_and_execute(thd, *query_ptr, true, true)) return true;
     }
   }
   return false;
@@ -909,6 +965,135 @@ bool upgrade_pxc_only(THD *thd) {
 
 #endif /* WITH_WSREP */
 
+/*
+  Maintain a file named "mysql_upgrade_history" in the data directory.
+
+  The file will contain one entry for each upgrade. The format is structured
+  text on JSON format.
+
+  Errors will be written as warnings to the error log; if we e.g. fail to
+  open the upgrade history file, we will not abort the server since this file
+  is not considered a critical feature of the server.
+
+  @param initialize   If this is the initialization of the data directory.
+*/
+void update_upgrade_history_file(bool initialize) {
+  /* Name of the "mysql_upgrade_history" file. */
+  char upgrade_file[FN_REFLEN] = {0};
+  fn_format(upgrade_file, "mysql_upgrade_history", mysql_real_data_home_ptr, "",
+            MYF(0));
+
+  /* JSON keys. */
+  constexpr char k_file_format[] = "file_format";
+  constexpr char k_upgrade_history[] = "upgrade_history";
+  constexpr char k_date[] = "date";
+  constexpr char k_version[] = "version";
+  constexpr char k_maturity[] = "maturity";
+  constexpr char k_initialize[] = "initialize";
+  constexpr char v_file_format[] = "1";
+
+  /* If > max entries, we keep the first and the (max - 1)  last ones. */
+  constexpr int MAX_HISTORY_SIZE = 1000;
+  static_assert(MAX_HISTORY_SIZE >= 2,
+                "The upgrade history should contain at least the first "
+                "and last entry.");
+  using namespace rapidjson;
+  Document doc;
+
+  /* Open file if it exists, auto close on return. */
+  auto deleter = [&](FILE *ptr) {
+    if (ptr != nullptr) my_fclose(ptr, MYF(0));
+  };
+  std::unique_ptr<FILE, decltype(deleter)> fp(nullptr, deleter);
+
+  MY_STAT sa;
+  char errbuf[MYSYS_STRERROR_SIZE];
+  bool file_exists = (my_stat(upgrade_file, &sa, MYF(0)) != nullptr);
+  bool append = file_exists;  // Append to current doc if possible.
+
+  /* If the file exists, read the doc and see if it is valid. */
+  if (file_exists) {
+    fp.reset(my_fopen(upgrade_file, O_RDONLY, MYF(0)));
+    if (fp == nullptr) {
+      LogErr(WARNING_LEVEL, ER_SERVER_CANT_OPEN_FILE, upgrade_file, my_errno(),
+             my_strerror(errbuf, sizeof(errbuf), my_errno()));
+      return;
+    }
+
+    /* Read contents into buffer. */
+    char buff[512] = {0};
+    std::string parsed_value;
+    do {
+      parsed_value.append(buff);
+      buff[0] = '\0';
+    } while (fgets(buff, sizeof(buff) - 1, fp.get()));
+
+    /* Parse JSON, check expected format. */
+    ParseResult ok = doc.Parse(parsed_value.c_str());
+    if (!(ok && doc.IsObject() && doc[k_file_format].IsString() &&
+          doc[k_upgrade_history].IsArray())) {
+      LogErr(WARNING_LEVEL, ER_INVALID_FILE_FORMAT, upgrade_file);
+      append = false;  // Cannot append, must overwrite with an empty doc.
+    }
+  }
+
+  /* If the file existed with valid contents, append, otherwise, overwrite. */
+  if (append) {
+    /* If current version is same as last entry, return. */
+    Value &hist = doc[k_upgrade_history].GetArray();
+    int count = hist.Size();
+    if (count > 0 &&
+        !strcmp(hist[count - 1][k_version].GetString(), server_version))
+      return;
+
+    /* If the doc contains too many entries, remove from the second and on. */
+    int remove_count = (count - MAX_HISTORY_SIZE) + 1;
+    if (remove_count > 0) {
+      hist.Erase(hist.Begin() + 1, hist.Begin() + remove_count + 1);
+    }
+  } else {
+    /* Otherwise, if no file existed, initialize an empty JSON document. */
+    doc.SetObject();
+    doc.AddMember(k_file_format, v_file_format, doc.GetAllocator());
+    Value history(kArrayType);
+    doc.AddMember(k_upgrade_history, history, doc.GetAllocator());
+  }
+
+  /* Append timestamp, MYSQL_SERVER_VERSION and LTS info to version array. */
+  std::stringstream str;
+  std::time_t sec = my_micro_time() / 1000000;
+  str << std::put_time(std::gmtime(&sec), "%F %T");
+  Value date(str.str().c_str(), str.str().size(), doc.GetAllocator());
+  Value version(server_version, strlen(server_version), doc.GetAllocator());
+  Value maturity(MYSQL_VERSION_MATURITY);
+
+  Value new_version;
+  new_version.SetObject();
+  new_version.AddMember(k_date, date, doc.GetAllocator());
+  new_version.AddMember(k_version, version, doc.GetAllocator());
+  new_version.AddMember(k_maturity, maturity, doc.GetAllocator());
+  if (initialize) {
+    Value init(true);
+    new_version.AddMember(k_initialize, init, doc.GetAllocator());
+  }
+  doc[k_upgrade_history].GetArray().PushBack(new_version, doc.GetAllocator());
+
+  /* Reopen the file, which is auto closed on function return. */
+  fp.reset(my_fopen(upgrade_file, O_CREAT | O_TRUNC | O_WRONLY, MYF(0)));
+  if (fp == nullptr) {
+    LogErr(WARNING_LEVEL, ER_SERVER_CANT_OPEN_FILE, upgrade_file, my_errno(),
+           my_strerror(errbuf, sizeof(errbuf), my_errno()));
+    return;
+  }
+
+  /* Write JSON document to a buffer and further to file with a newline. */
+  rapidjson::StringBuffer buffer;
+  rapidjson::Writer<rapidjson::StringBuffer> writer(buffer);
+  doc.Accept(writer);
+  fputs(buffer.GetString(), fp.get());
+  fputs("\n", fp.get());
+}
+
 /*
   This function runs checks on the database before running the upgrade to make
   sure that the database is ready to be upgraded to a newer version. New checks
@@ -952,9 +1137,8 @@ bool do_server_upgrade_checks(THD *thd) {
   auto process_schema = [&](std::unique_ptr<Schema> &schema) {
     return check_tables(thd, schema, &shared_spaces, &error_count) ||
            check_events(thd, schema, &error_count) ||
-           check_routines(thd, schema, &error_count) ||
-           check_views(thd, schema, &error_count) ||
-           check_table_funs(thd, schema, &error_count);
+           check_routines(thd, schema, error_handler, &error_count) ||
+           check_views(thd, schema, &error_count);
   };
 
   if (thd->dd_client()->foreach<dd::Schema>(nullptr, process_schema) ||
@@ -962,7 +1146,12 @@ bool do_server_upgrade_checks(THD *thd) {
     thd->pop_internal_handler();
     return dd::end_transaction(thd, true);
   }
-
+  ulong non_std_key_cnt = deprecated_use_fk_on_non_standard_key_count;
+  if (non_std_key_cnt != 0) {
+    LogErr(WARNING_LEVEL, ER_USAGE_DEPRECATION_COUNTER,
+           "foreign key referring to a non-unique or partial key",
+           std::to_string(non_std_key_cnt).c_str(), "during upgrade");
+  }
   thd->pop_internal_handler();
   return false;
 }
@@ -1137,7 +1326,8 @@ bool upgrade_system_schemas(THD *thd) {
           dd::tables::DD_properties::instance().set(
               thd, "MYSQLD_VERSION_UPGRADED", MYSQL_VERSION_ID);
   }
-  create_upgrade_file();
+
+  remove_legacy_upgrade_info_file();
   bootstrap_error_handler.set_log_error(true);
 
   if (dd::bootstrap::DD_bootstrap_ctx::instance().is_server_patch_downgrade()) {
