@@ -1,4 +1,4 @@
-/* Copyright (c) 2005, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2005, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -33,22 +33,23 @@
 #include <vector>
 
 #include "lex_string.h"
-#include "m_ctype.h"  // CHARSET_INFO
 #include "m_string.h"
 #include "my_dbug.h"
-#include "my_loglevel.h"
 #include "my_macros.h"
 #include "my_sys.h"
 #include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/components/services/bits/psi_memory_bits.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/components/services/log_shared.h"
+#include "mysql/my_loglevel.h"
 #include "mysql/psi/mysql_cond.h"
 #include "mysql/psi/mysql_memory.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql/psi/mysql_sp.h"
 #include "mysql/psi/mysql_stage.h"
 #include "mysql/psi/mysql_thread.h"
+#include "mysql/strings/int2str.h"
+#include "mysql/strings/m_ctype.h"  // CHARSET_INFO
 #include "mysql_com.h"
 #include "mysqld_error.h"  // ER_*
 #include "sql/auth/auth_acls.h"
@@ -85,6 +86,7 @@
 #include "sql/transaction.h"
 #include "sql/tztime.h"  // Time_zone
 #include "sql_string.h"  // String
+#include "string_with_len.h"
 
 #ifdef WITH_WSREP
 #include "sql/sql_parse.h"
@@ -99,13 +101,13 @@
  TODO list :
  - CREATE EVENT should not go into binary log! Does it now? The SQL statements
    issued by the EVENT are replicated.
-   I have an idea how to solve the problem at failover. So the status field
-   will be ENUM('DISABLED', 'ENABLED', 'SLAVESIDE_DISABLED').
+   I have an idea how to solve the problem at failover. So the internal table
+   status field will be ENUM('DISABLED', 'ENABLED', 'SLAVESIDE_DISABLED').
    In this case when CREATE EVENT is replicated it should go into the binary
-   as SLAVESIDE_DISABLED if it is ENABLED, when it's created as DISABLEd it
+   as DISABLE ON REPLICA if it is ENABLED, when it's created as DISABLEd it
    should be replicated as disabled. If an event is ALTERed as DISABLED the
    query should go untouched into the binary log, when ALTERed as enable then
-   it should go as SLAVESIDE_DISABLED. This is regarding the SQL interface.
+   it should go as DISABLE ON REPLICA. This is regarding the SQL interface.
    TT routines however modify mysql.event internally and this does not go the
    log so in this case queries has to be injected into the log...somehow... or
    maybe a solution is RBR for this case, because the event may go only from
@@ -377,9 +379,9 @@ bool Events::create_event(THD *thd, Event_parse_data *parse_data,
     When we are going out of the function scope, the original binary
     format state will be restored.
   */
-  Save_and_Restore_binlog_format_state binlog_format_state(thd);
+  const Save_and_Restore_binlog_format_state binlog_format_state(thd);
 
-  dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
+  const dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
   if (Event_db_repository::create_event(thd, parse_data, if_not_exists,
                                         &event_already_exists)) {
     /* On error conditions my_error() is called so no need to handle here */
@@ -535,9 +537,9 @@ bool Events::update_event(THD *thd, Event_parse_data *parse_data,
     When we are going out of the function scope, the original binary
     format state will be restored.
   */
-  Save_and_Restore_binlog_format_state binlog_format_state(thd);
+  const Save_and_Restore_binlog_format_state binlog_format_state(thd);
 
-  dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
+  const dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
   if (Event_db_repository::update_event(thd, parse_data, new_dbname,
                                         new_name)) {
     /* On error conditions my_error() is called so no need to handle here */
@@ -552,8 +554,8 @@ bool Events::update_event(THD *thd, Event_parse_data *parse_data,
       goto err_with_rollback;
     }
 
-    LEX_CSTRING dbname = new_dbname ? *new_dbname : parse_data->dbname;
-    LEX_CSTRING name = new_name ? *new_name : parse_data->name;
+    const LEX_CSTRING dbname = new_dbname ? *new_dbname : parse_data->dbname;
+    const LEX_CSTRING name = new_name ? *new_name : parse_data->name;
     if (Event_db_repository::load_named_event(thd, dbname, name,
                                               new_element.get()))
       goto err_with_rollback;
@@ -658,7 +660,7 @@ bool Events::drop_event(THD *thd, LEX_CSTRING dbname, LEX_CSTRING name,
 
   DEBUG_SYNC(thd, "after_acquiring_exclusive_lock_on_the_event");
 
-  dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
+  const dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
   bool event_exists;
   if (Event_db_repository::drop_event(thd, dbname, name, if_exists,
                                       &event_exists)) {
@@ -1160,7 +1162,7 @@ static bool load_events_from_db(THD *thd, Event_queue *event_queue) {
   DBUG_TRACE;
   DBUG_PRINT("enter", ("thd: %p", thd));
 
-  dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
+  const dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
 
   // Fetch all Schemas
   std::vector<const dd::Schema *> schemas;
@@ -1214,7 +1216,7 @@ static bool load_events_from_db(THD *thd, Event_queue *event_queue) {
         WSREP_DEBUG("Disabling non-native event (name: %s)",
                     et->m_event_name.str);
 
-        if (et->m_status == Event_parse_data::SLAVESIDE_DISABLED) continue;
+        if (et->m_status == Event_parse_data::REPLICA_SIDE_DISABLED) continue;
 
         if (lock_object_name(thd, MDL_key::EVENT, et->m_schema_name.str,
                              et->m_event_name.str)) {
@@ -1229,7 +1231,7 @@ static bool load_events_from_db(THD *thd, Event_queue *event_queue) {
 
         (void)Event_db_repository::update_timing_fields_for_event(
             thd, et->m_schema_name, et->m_event_name, et->m_last_executed,
-            Event_parse_data::SLAVESIDE_DISABLED);
+            Event_parse_data::REPLICA_SIDE_DISABLED);
 
         thd->mdl_context.release_transactional_locks();
 
