@@ -1,15 +1,16 @@
-/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -58,7 +59,6 @@
 #include "sql/sql_class.h"
 #include "sql/sql_const.h"
 #include "sql/sql_executor.h"
-#include "sql/sql_join_buffer.h"
 #include "sql/sql_lex.h"
 #include "sql/sql_opt_exec_shared.h"
 #include "sql/sql_optimizer.h"  // JOIN
@@ -485,16 +485,14 @@ DynamicRangeIterator::DynamicRangeIterator(THD *thd, TABLE *table,
       m_qep_tab(qep_tab),
       m_mem_root(key_memory_test_quick_select_exec,
                  thd->variables.range_alloc_block_size),
-      m_examined_rows(examined_rows),
-      m_read_set_without_base_columns(table->read_set) {
-  add_virtual_gcol_base_cols(table, thd->mem_root,
-                             &m_read_set_with_base_columns);
-}
+      m_examined_rows(examined_rows) {}
 
 DynamicRangeIterator::~DynamicRangeIterator() {
   // This is owned by our MEM_ROOT.
-  destroy(m_qep_tab->range_scan());
-  m_qep_tab->set_range_scan(nullptr);
+  if (m_qep_tab->range_scan() != nullptr) {
+    ::destroy_at(m_qep_tab->range_scan());
+    m_qep_tab->set_range_scan(nullptr);
+  }
 }
 
 bool DynamicRangeIterator::Init() {
@@ -580,29 +578,13 @@ bool DynamicRangeIterator::Init() {
     return false;
   }
 
-  // Create the required Iterator based on the strategy chosen. Also set the
-  // read set to be used while accessing the table. Unlike a regular range
-  // scan, as the access strategy keeps changing for a dynamic range scan,
-  // optimizer cannot know if the read set should include base columns of
-  // virtually generated columns or not. As a result, this Iterator maintains
-  // two different read sets, to be used once the access strategy is chosen
-  // here.
+  // Create the required Iterator based on the strategy chosen.
   if (qck) {
     m_iterator = std::move(qck);
-    // If the range optimizer chose index merge scan or a range scan with
-    // covering index, use the read set without base columns. Otherwise we use
-    // the read set with base columns included.
-    if (used_index(range_scan) == MAX_KEY ||
-        table()->covering_keys.is_set(used_index(range_scan)))
-      table()->read_set = m_read_set_without_base_columns;
-    else
-      table()->read_set = &m_read_set_with_base_columns;
   } else {
     m_iterator = NewIterator<TableScanIterator>(
         thd(), &m_mem_root, table(), m_qep_tab->position()->rows_fetched,
         m_examined_rows);
-    // For a table scan, include base columns in read set.
-    table()->read_set = &m_read_set_with_base_columns;
   }
   return m_iterator->Init();
 }
@@ -633,7 +615,7 @@ FullTextSearchIterator::FullTextSearchIterator(THD *thd, TABLE *table,
   // hand, maintains alternative plans with and without index scans throughout
   // the planning, and doesn't determine whether it should use the indexed or
   // non-indexed plan until the full query plan has been constructed.
-  if (thd->lex->using_hypergraph_optimizer) {
+  if (thd->lex->using_hypergraph_optimizer()) {
     // Should not already be enabled.
     assert(!ft_func->score_from_index_scan);
     // Should operate on the main object.
@@ -795,8 +777,7 @@ AlternativeIterator::AlternativeIterator(
     : RowIterator(thd),
       m_source_iterator(std::move(source)),
       m_table_scan_iterator(std::move(table_scan_iterator)),
-      m_table(table),
-      m_original_read_set(table->read_set) {
+      m_table(table) {
   for (unsigned key_part_idx = 0; key_part_idx < ref->key_parts;
        ++key_part_idx) {
     bool *cond_guard = ref->cond_guards[key_part_idx];
@@ -805,17 +786,13 @@ AlternativeIterator::AlternativeIterator(
     }
   }
   assert(!m_applicable_cond_guards.empty());
-
-  add_virtual_gcol_base_cols(table, thd->mem_root, &m_table_scan_read_set);
 }
 
 bool AlternativeIterator::Init() {
   m_iterator = m_source_iterator.get();
-  m_table->read_set = m_original_read_set;
   for (bool *cond_guard : m_applicable_cond_guards) {
     if (!*cond_guard) {
       m_iterator = m_table_scan_iterator.get();
-      m_table->read_set = &m_table_scan_read_set;
       break;
     }
   }
@@ -934,11 +911,11 @@ int ZeroRowsAggregatedIterator::Read() {
 TableValueConstructorIterator::TableValueConstructorIterator(
     THD *thd, ha_rows *examined_rows,
     const mem_root_deque<mem_root_deque<Item *> *> &row_value_list,
-    mem_root_deque<Item *> *join_fields)
+    Mem_root_array<Item_values_column *> *output_refs)
     : RowIterator(thd),
       m_examined_rows(examined_rows),
       m_row_value_list(row_value_list),
-      m_output_refs(join_fields) {
+      m_output_refs(output_refs) {
   assert(examined_rows != nullptr);
 }
 
@@ -948,17 +925,15 @@ bool TableValueConstructorIterator::Init() {
 }
 
 int TableValueConstructorIterator::Read() {
-  if (*m_examined_rows == m_row_value_list.size()) return -1;
+  if (m_row_it == m_row_value_list.end()) return -1;
 
   // If the TVC has a single row, we don't create Item_values_column reference
   // objects during resolving. We will instead use the single row directly from
   // Query_block::item_list, such that we don't have to change references here.
-  if (m_row_value_list.size() != 1) {
-    auto output_refs_it = VisibleFields(*m_output_refs).begin();
+  if (m_output_refs != nullptr) {
+    Item_values_column **output_refs_it = m_output_refs->begin();
     for (const Item *value : **m_row_it) {
-      Item_values_column *ref =
-          down_cast<Item_values_column *>(*output_refs_it);
-      ++output_refs_it;
+      Item_values_column *ref = *output_refs_it++;
 
       // Ideally we would not be casting away constness here. However, as the
       // evaluation of Item objects during execution is not const (i.e. none of
@@ -966,9 +941,9 @@ int TableValueConstructorIterator::Read() {
       // Item_values_column object cannot be const.
       ref->set_value(const_cast<Item *>(value));
     }
-    ++m_row_it;
   }
 
+  ++m_row_it;
   ++*m_examined_rows;
   return 0;
 }
