@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -84,7 +84,6 @@
 #include <atomic>
 
 #include "lex_string.h"
-#include "m_ctype.h"
 #include "m_string.h"
 #include "my_base.h"
 #include "my_dbug.h"
@@ -92,6 +91,7 @@
 #include "my_sqlcommand.h"
 #include "my_sys.h"
 #include "mysql/service_mysql_alloc.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
 #include "sql/auth/auth_common.h"  // SUPER_ACL
@@ -327,9 +327,9 @@ MYSQL_LOCK *mysql_lock_tables(THD *thd, TABLE **tables, size_t count,
                               uint flags) {
   int rc;
   MYSQL_LOCK *sql_lock;
-  ulong timeout = (flags & MYSQL_LOCK_IGNORE_TIMEOUT)
-                      ? LONG_TIMEOUT
-                      : thd->variables.lock_wait_timeout;
+  const ulong timeout = (flags & MYSQL_LOCK_IGNORE_TIMEOUT)
+                            ? LONG_TIMEOUT
+                            : thd->variables.lock_wait_timeout;
 
   DBUG_TRACE;
 
@@ -341,7 +341,7 @@ MYSQL_LOCK *mysql_lock_tables(THD *thd, TABLE **tables, size_t count,
   if (!(thd->state_flags & Open_tables_state::SYSTEM_TABLES))
     THD_STAGE_INFO(thd, stage_system_lock);
 
-  ulonglong lock_start_usec = my_micro_time();
+  const ulonglong lock_start_usec = my_micro_time();
 
   DBUG_PRINT("info", ("thd->proc_info %s", thd->proc_info()));
   if (sql_lock->table_count &&
@@ -384,7 +384,7 @@ end:
   if (thd->variables.session_track_transaction_info > TX_TRACK_NONE)
     track_table_access(thd, tables, count);
 
-  ulonglong lock_end_usec = my_micro_time();
+  const ulonglong lock_end_usec = my_micro_time();
   thd->inc_lock_usec(lock_end_usec - lock_start_usec);
 
   return sql_lock;
@@ -959,8 +959,6 @@ static void print_lock_error(int error, const char *table) {
   }
 }
 
-std::atomic<int32> Global_read_lock::m_atomic_active_requests;
-
 /****************************************************************************
   Handling of global read locks
 
@@ -984,7 +982,7 @@ std::atomic<int32> Global_read_lock::m_atomic_active_requests;
   if possible).
 
   Why does FLUSH TABLES WITH READ LOCK need to block COMMIT: because it's used
-  to read a non-moving SHOW MASTER STATUS, and a COMMIT writes to the binary
+  to read a non-moving SHOW BINARY LOG STATUS, and a COMMIT writes to the binary
   log.
 
   Why getting the global read lock is two steps and not one. Because FLUSH
@@ -1092,12 +1090,8 @@ bool Global_read_lock::lock_global_read_lock(THD *thd) {
                      MDL_EXPLICIT);
 #endif /* WITH_WSREP */
 
-    /* Increment static variable first to signal innodb memcached server
-       to release mdl locks held by it */
-    Global_read_lock::m_atomic_active_requests++;
     if (thd->mdl_context.acquire_lock(&mdl_request,
                                       thd->variables.lock_wait_timeout)) {
-      Global_read_lock::m_atomic_active_requests--;
       return true;
     }
 
@@ -1159,7 +1153,6 @@ void Global_read_lock::unlock_global_read_lock(THD *thd) {
 #endif /* WITH_WSREP */
   }
   thd->mdl_context.release_lock(m_mdl_global_shared_lock);
-  Global_read_lock::m_atomic_active_requests--;
   m_mdl_global_shared_lock = nullptr;
   m_state = GRL_NONE;
 }
