@@ -1,6 +1,6 @@
 #ifndef MDL_H
 #define MDL_H
-/* Copyright (c) 2009, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2009, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -30,9 +30,9 @@
 #include <new>
 #include <unordered_map>
 
-#include "m_string.h"
 #include "my_alloc.h"
 #include "my_compiler.h"
+#include "strmake.h"
 
 #include "my_inttypes.h"
 #include "my_psi_config.h"
@@ -538,10 +538,10 @@ struct MDL_key {
     end = strmake(start, name, NAME_LEN);
     m_object_name_length = static_cast<uint16>(end - start);
 
-    size_t col_len = strlen(column_name);
+    const size_t col_len = strlen(column_name);
     assert(col_len <= NAME_LEN);
     start = end + 1;
-    size_t remaining =
+    const size_t remaining =
         MAX_MDLKEY_LENGTH - m_db_name_length - m_object_name_length - 3;
     uint16 extra_length = 0;
 
@@ -713,9 +713,10 @@ struct MDL_key {
     m_object_name_length = m_length - m_db_name_length - 3;
   }
   void mdl_key_init(const MDL_key *rhs) {
-    uint16 copy_length = rhs->use_normalized_object_name()
-                             ? rhs->m_length + rhs->m_object_name_length + 1
-                             : rhs->m_length;
+    const uint16 copy_length =
+        rhs->use_normalized_object_name()
+            ? rhs->m_length + rhs->m_object_name_length + 1
+            : rhs->m_length;
     memcpy(m_ptr, rhs->m_ptr, copy_length);
     m_length = rhs->m_length;
     m_db_name_length = rhs->m_db_name_length;
