@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -379,15 +379,28 @@ void Global_THD_manager::do_for_all_thd_copy(Do_THD_Impl *func) {
 }
 
 void Global_THD_manager::do_for_all_thd(Do_THD_Impl *func) {
-  Do_THD doit(func);
+  const Do_THD doit(func);
   for (int i = 0; i < NUM_PARTITIONS; i++) {
     MUTEX_LOCK(lock, &LOCK_thd_list[i]);
     std::for_each(thd_list[i].begin(), thd_list[i].end(), doit);
   }
 }
 
+void Global_THD_manager::do_for_first_n_thd(Do_THD_Impl *func, uint n) {
+  Do_THD doit(func);
+  uint num_processed_thds = 0;
+  for (int i = 0; i < NUM_PARTITIONS; i++) {
+    MUTEX_LOCK(lock, &LOCK_thd_list[i]);
+    for (auto it = thd_list[i].begin(); it != thd_list[i].end(); it++) {
+      if (num_processed_thds == n) return;
+      doit(*it);
+      num_processed_thds++;
+    }
+  }
+}
+
 THD_ptr Global_THD_manager::find_thd(Find_THD_Impl *func) {
-  Find_THD find_thd(func);
+  const Find_THD find_thd(func);
   for (int i = 0; i < NUM_PARTITIONS; i++) {
     MUTEX_LOCK(lock, &LOCK_thd_list[i]);
     THD_array::const_iterator it =
@@ -404,7 +417,7 @@ THD_ptr Global_THD_manager::find_thd(Find_THD_Impl *func) {
 // Optimized version of the above function for when we know
 // the thread_id of the THD we are looking for.
 THD_ptr Global_THD_manager::find_thd(Find_thd_with_id *func) {
-  Find_THD find_thd(func);
+  const Find_THD find_thd(func);
   // Since we know the thread_id, we can check the correct
   // partition directly.
   const int partition = thd_partition(func->m_thread_id);
