@@ -1,4 +1,4 @@
-/* Copyright (c) 2013, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2013, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -356,6 +356,14 @@ class Global_THD_manager {
   */
   void do_for_all_thd(Do_THD_Impl *func);
 
+  /**
+   * This function calls func() for all first "n" THDs across all THD list
+   * partitions.
+   * @param func Object of class which overrides operator()
+   * @param n number of elements we want to call func for
+   */
+  void do_for_first_n_thd(Do_THD_Impl *func, uint n);
+
   /**
     Returns a THD_ptr containing first THD for which operator() returns true.
 
