@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -29,11 +29,10 @@
 #include <cstring>
 
 #include "lex_string.h"
-#include "m_ctype.h"
-#include "m_string.h"
 #include "my_alloc.h"
 
 #include "my_sqlcommand.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysqld_error.h"
 #include "sql/derror.h"
 #include "sql/item_subselect.h"
@@ -47,6 +46,7 @@
 #include "sql/sql_const.h"
 #include "sql/sql_error.h"
 #include "sql/sql_lex.h"
+#include "string_with_len.h"
 
 extern struct st_opt_hint_info opt_hint_info[];
 
@@ -137,7 +137,7 @@ static Opt_hints_qb *find_qb_hints(Parse_context *pc,
     }
   }
 
-  if (qb == nullptr && hint != nullptr)
+  if (qb == nullptr)
     hint->print_warn(pc->thd, ER_WARN_UNKNOWN_QB_NAME, qb_name, nullptr,
                      nullptr, nullptr);
 
@@ -222,8 +222,8 @@ void PT_hint::print_warn(THD *thd, uint err_code,
                       ER_THD_NONCONST(thd, err_code), str.c_ptr_safe());
 }
 
-bool PT_qb_level_hint::contextualize(Parse_context *pc) {
-  if (super::contextualize(pc)) return true;
+bool PT_qb_level_hint::do_contextualize(Parse_context *pc) {
+  if (super::do_contextualize(pc)) return true;
 
   Opt_hints_qb *qb = find_qb_hints(pc, &qb_name, this);
   if (qb == nullptr) return false;  // TODO: Should this generate a warning?
@@ -335,13 +335,8 @@ void PT_qb_level_hint::append_args(const THD *thd, String *str) const {
   }
 }
 
-bool PT_hint_list::contextualize(Parse_context *pc) {
-  if (!pc->thd->lex->opt_hints_global)
-    pc->thd->lex->opt_hints_global =
-        new (pc->thd->mem_root) Opt_hints_global(pc->thd->mem_root);
-  if (pc->thd->lex->opt_hints_global->deferred_hints_flag != true) {
-    if (super::contextualize(pc)) return true;
-  }
+bool PT_hint_list::do_contextualize(Parse_context *pc) {
+  if (super::do_contextualize(pc)) return true;
 
   if (!get_qb_hints(pc, pc->select)) return true;
 
@@ -350,18 +345,14 @@ bool PT_hint_list::contextualize(Parse_context *pc) {
       if (pc->thd->lex->sql_command == SQLCOM_CREATE_VIEW &&
           !(*h)->supports_view())
         continue;
-      if ((*h)->contextualize(pc)) {
-        if (pc->thd->lex->opt_hints_global->deferred_hints_flag != true) {
-          pc->thd->lex->opt_hints_global->deferred_hints->push_back(*h);
-        }
-      }
+      if ((*h)->contextualize(pc)) return true;
     }
   }
   return false;
 }
 
-bool PT_table_level_hint::contextualize(Parse_context *pc) {
-  if (super::contextualize(pc)) return true;
+bool PT_table_level_hint::do_contextualize(Parse_context *pc) {
+  if (super::do_contextualize(pc)) return true;
 
   if (table_list.empty())  // Query block level hint
   {
@@ -412,23 +403,11 @@ void PT_key_level_hint::append_args(const THD *thd, String *str) const {
   }
 }
 
-bool PT_key_level_hint::contextualize(Parse_context *pc) {
-  Opt_hints_qb *qb = find_qb_hints(pc, &table_name.opt_query_block, nullptr);
-  if (qb == nullptr &&
-      pc->thd->lex->opt_hints_global->deferred_hints_flag == true) {
-    this->print_warn(pc->thd, ER_WARN_UNKNOWN_QB_NAME,
-                     &table_name.opt_query_block, nullptr, nullptr, nullptr);
-    if (super::contextualize(pc)) return true;
-    return false;
-  }
-  if (qb == nullptr && this->type() != INDEX_HINT_ENUM) {
-    this->print_warn(pc->thd, ER_WARN_UNKNOWN_QB_NAME,
-                     &table_name.opt_query_block, nullptr, nullptr, nullptr);
-    if (super::contextualize(pc)) return true;
-    return false;
-  }
-  if (qb == nullptr) return true;
-  if (super::contextualize(pc)) return true;
+bool PT_key_level_hint::do_contextualize(Parse_context *pc) {
+  if (super::do_contextualize(pc)) return true;
+
+  Opt_hints_qb *qb = find_qb_hints(pc, &table_name.opt_query_block, this);
+  if (qb == nullptr) return false;
 
   Opt_hints_table *tab = get_table_hints(pc, &table_name, qb);
   if (!tab) return true;
@@ -496,8 +475,8 @@ bool PT_key_level_hint::contextualize(Parse_context *pc) {
   return false;
 }
 
-bool PT_hint_qb_name::contextualize(Parse_context *pc) {
-  if (super::contextualize(pc)) return true;
+bool PT_hint_qb_name::do_contextualize(Parse_context *pc) {
+  if (super::do_contextualize(pc)) return true;
 
   Opt_hints_qb *qb = pc->select->opt_hints_qb;
 
@@ -515,8 +494,8 @@ bool PT_hint_qb_name::contextualize(Parse_context *pc) {
   return false;
 }
 
-bool PT_hint_max_execution_time::contextualize(Parse_context *pc) {
-  if (super::contextualize(pc)) return true;
+bool PT_hint_max_execution_time::do_contextualize(Parse_context *pc) {
+  if (super::do_contextualize(pc)) return true;
 
   if (pc->thd->lex->sql_command != SQLCOM_SELECT ||  // not a SELECT statement
       pc->thd->lex->sphead ||                        // or in a SP/trigger/event
@@ -542,7 +521,7 @@ bool PT_hint_max_execution_time::contextualize(Parse_context *pc) {
   return false;
 }
 
-bool PT_hint_sys_var::contextualize(Parse_context *pc) {
+bool PT_hint_sys_var::do_contextualize(Parse_context *pc) {
   if (!sys_var_value) {
     // No warning here, warning is issued by parser.
     return false;
@@ -584,8 +563,8 @@ bool PT_hint_sys_var::contextualize(Parse_context *pc) {
                                             sys_var_value);
 }
 
-bool PT_hint_resource_group::contextualize(Parse_context *pc) {
-  if (super::contextualize(pc)) return true;
+bool PT_hint_resource_group::do_contextualize(Parse_context *pc) {
+  if (super::do_contextualize(pc)) return true;
 
   auto res_grp_mgr = resourcegroups::Resource_group_mgr::instance();
   if (!res_grp_mgr->resource_group_support()) {
