@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -27,13 +27,12 @@
 #include <set>
 #include <utility>
 
-#include "m_ctype.h"
 #include "m_string.h"
 #include "memory_debugging.h"
 #include "my_dbug.h"
-#include "my_loglevel.h"
 #include "my_sqlcommand.h"
 #include "mysql/components/services/log_builtins.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysqld_error.h"
 #include "sql/handler.h"
 #include "sql/key.h"
@@ -576,6 +575,10 @@ SEL_TREE *tree_and(RANGE_OPT_PARAM *param, SEL_TREE *tree1, SEL_TREE *tree2) {
 
   /* ok, both trees are index_merge trees */
   imerge_list_and_list(&tree1->merges, &tree2->merges);
+  // An index merge is a union/OR, so it cannot exactly represent an
+  // intersection/AND.
+  tree1->inexact |= !tree1->merges.is_empty();
+
   return tree1;
 }
 
@@ -668,7 +671,7 @@ static bool remove_nonrange_trees(RANGE_OPT_PARAM *param, SEL_TREE *tree) {
   for (uint i = 0; i < param->keys; i++) {
     if (tree->keys[i]) {
       if (tree->keys[i]->root->part) {
-        tree->keys[i] = NULL;
+        tree->keys[i] = nullptr;
         tree->keys_map.clear_bit(i);
       } else
         res = true;
@@ -708,7 +711,7 @@ SEL_TREE *tree_or(RANGE_OPT_PARAM *param, bool remove_jump_scans,
   */
   if (!tree1->merges.is_empty()) {
     for (uint i = 0; i < param->keys; i++)
-      if (tree1->keys[i] != NULL &&
+      if (tree1->keys[i] != nullptr &&
           tree1->keys[i]->type == SEL_ROOT::Type::KEY_RANGE) {
         tree1->merges.clear();
         break;
@@ -716,7 +719,7 @@ SEL_TREE *tree_or(RANGE_OPT_PARAM *param, bool remove_jump_scans,
   }
   if (!tree2->merges.is_empty()) {
     for (uint i = 0; i < param->keys; i++)
-      if (tree2->keys[i] != NULL &&
+      if (tree2->keys[i] != nullptr &&
           tree2->keys[i]->type == SEL_ROOT::Type::KEY_RANGE) {
         tree2->merges.clear();
         break;
