@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -35,12 +35,12 @@
 #include "my_compiler.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
-#include "my_loglevel.h"
 #include "my_sys.h"
 #include "my_thread.h"
 #include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/components/services/bits/psi_stage_bits.h"
 #include "mysql/components/services/log_builtins.h"
+#include "mysql/my_loglevel.h"
 #include "mysql/psi/mysql_cond.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql/service_mysql_alloc.h"
@@ -92,13 +92,6 @@ int channel_stop(Master_info *mi, int threads_to_stop, long timeout);
 int initialize_channel_service_interface() {
   DBUG_TRACE;
 
-  // master info and relay log repositories must be TABLE
-  if (opt_mi_repository_id != INFO_REPOSITORY_TABLE ||
-      opt_rli_repository_id != INFO_REPOSITORY_TABLE) {
-    LogErr(ERROR_LEVEL, ER_RPL_CHANNELS_REQUIRE_TABLES_AS_INFO_REPOSITORIES);
-    return 1;
-  }
-
   // server id must be different from 0
   if (server_id == 0) {
     LogErr(ERROR_LEVEL, ER_RPL_CHANNELS_REQUIRE_NON_ZERO_SERVER_ID);
@@ -152,8 +145,9 @@ static void set_mi_settings(Master_info *mi,
     Group replication applier channel shall not use checksum on its relay log
     files.
   */
-  if (channel_map.is_group_replication_channel_name(mi->get_channel(), true)) {
-    fde->footer()->checksum_alg = binary_log::BINLOG_CHECKSUM_ALG_OFF;
+  if (channel_map.is_group_replication_applier_channel_name(
+          mi->get_channel())) {
+    fde->footer()->checksum_alg = mysql::binlog::event::BINLOG_CHECKSUM_ALG_OFF;
     /*
       When the receiver thread connects to the master, it gets its current
       binlog checksum algorithm, but as GR applier channel has no receiver
@@ -161,7 +155,7 @@ static void set_mi_settings(Master_info *mi,
       here to BINLOG_CHECKSUM_ALG_OFF as events queued after certification have
       no checksum information.
     */
-    mi->checksum_alg_before_fd = binary_log::BINLOG_CHECKSUM_ALG_OFF;
+    mi->checksum_alg_before_fd = mysql::binlog::event::BINLOG_CHECKSUM_ALG_OFF;
   }
   mi->set_mi_description_event(fde);
 
@@ -234,10 +228,10 @@ void initialize_channel_connection_info(Channel_connection_info *channel_info) {
   channel_info->view_id = nullptr;
 }
 
-static void set_mi_ssl_options(LEX_MASTER_INFO *lex_mi,
+static void set_mi_ssl_options(LEX_SOURCE_INFO *lex_mi,
                                Channel_ssl_info *channel_ssl_info) {
-  lex_mi->ssl = (channel_ssl_info->use_ssl) ? LEX_MASTER_INFO::LEX_MI_ENABLE
-                                            : LEX_MASTER_INFO::LEX_MI_DISABLE;
+  lex_mi->ssl = (channel_ssl_info->use_ssl) ? LEX_SOURCE_INFO::LEX_MI_ENABLE
+                                            : LEX_SOURCE_INFO::LEX_MI_DISABLE;
 
   if (channel_ssl_info->ssl_ca_file_name != nullptr) {
     lex_mi->ssl_ca = channel_ssl_info->ssl_ca_file_name;
@@ -272,15 +266,15 @@ static void set_mi_ssl_options(LEX_MASTER_INFO *lex_mi,
   }
 
   if (channel_ssl_info->tls_ciphersuites != nullptr) {
-    lex_mi->tls_ciphersuites = LEX_MASTER_INFO::SPECIFIED_STRING;
+    lex_mi->tls_ciphersuites = LEX_SOURCE_INFO::SPECIFIED_STRING;
     lex_mi->tls_ciphersuites_string = channel_ssl_info->tls_ciphersuites;
   } else {
-    lex_mi->tls_ciphersuites = LEX_MASTER_INFO::SPECIFIED_NULL;
+    lex_mi->tls_ciphersuites = LEX_SOURCE_INFO::SPECIFIED_NULL;
   }
 
   lex_mi->ssl_verify_server_cert = (channel_ssl_info->ssl_verify_server_cert)
-                                       ? LEX_MASTER_INFO::LEX_MI_ENABLE
-                                       : LEX_MASTER_INFO::LEX_MI_DISABLE;
+                                       ? LEX_SOURCE_INFO::LEX_MI_ENABLE
+                                       : LEX_SOURCE_INFO::LEX_MI_DISABLE;
 }
 
 int channel_create(const char *channel, Channel_creation_info *channel_info) {
@@ -288,7 +282,7 @@ int channel_create(const char *channel, Channel_creation_info *channel_info) {
 
   Master_info *mi = nullptr;
   int error = 0;
-  LEX_MASTER_INFO *lex_mi = nullptr;
+  LEX_SOURCE_INFO *lex_mi = nullptr;
 
   bool thd_created = false;
   THD *thd = current_thd;
@@ -314,7 +308,7 @@ int channel_create(const char *channel, Channel_creation_info *channel_info) {
     if ((error = add_new_channel(&mi, channel))) goto err;
   }
 
-  lex_mi = new LEX_MASTER_INFO();
+  lex_mi = new LEX_SOURCE_INFO();
   lex_mi->channel = channel;
   lex_mi->host = channel_info->hostname;
   /*
@@ -322,23 +316,24 @@ int channel_create(const char *channel, Channel_creation_info *channel_info) {
     or 'group_replication_applier' channel wants to set the port number
     to '0' as there is no actual network usage on these channels.
   */
-  lex_mi->port_opt = LEX_MASTER_INFO::LEX_MI_ENABLE;
+  lex_mi->port_opt = LEX_SOURCE_INFO::LEX_MI_ENABLE;
   lex_mi->port = channel_info->port;
   lex_mi->user = channel_info->user;
   lex_mi->password = channel_info->password;
   lex_mi->sql_delay = channel_info->sql_delay;
   lex_mi->connect_retry = channel_info->connect_retry;
   if (channel_info->retry_count) {
-    lex_mi->retry_count_opt = LEX_MASTER_INFO::LEX_MI_ENABLE;
+    lex_mi->retry_count_opt = LEX_SOURCE_INFO::LEX_MI_ENABLE;
     lex_mi->retry_count = channel_info->retry_count;
   }
 
   if (channel_info->auto_position) {
-    lex_mi->auto_position = LEX_MASTER_INFO::LEX_MI_ENABLE;
+    lex_mi->auto_position = LEX_SOURCE_INFO::LEX_MI_ENABLE;
     if ((mi && mi->is_auto_position()) ||
         channel_info->auto_position == RPL_SERVICE_SERVER_DEFAULT) {
-      // So change master allows new configurations with a running SQL thread
-      lex_mi->auto_position = LEX_MASTER_INFO::LEX_MI_UNCHANGED;
+      // So change replication source allows new configurations with a running
+      // SQL thread
+      lex_mi->auto_position = LEX_SOURCE_INFO::LEX_MI_UNCHANGED;
     }
   }
 
@@ -347,16 +342,18 @@ int channel_create(const char *channel, Channel_creation_info *channel_info) {
   }
 
   if (channel_info->get_public_key) {
-    lex_mi->get_public_key = LEX_MASTER_INFO::LEX_MI_ENABLE;
+    lex_mi->get_public_key = LEX_SOURCE_INFO::LEX_MI_ENABLE;
     if (mi && mi->get_public_key) {
-      // So change master allows new configurations with a running SQL thread
-      lex_mi->get_public_key = LEX_MASTER_INFO::LEX_MI_UNCHANGED;
+      // So change replication source allows new configurations with a running
+      // SQL thread
+      lex_mi->get_public_key = LEX_SOURCE_INFO::LEX_MI_UNCHANGED;
     }
   } else {
-    lex_mi->get_public_key = LEX_MASTER_INFO::LEX_MI_DISABLE;
+    lex_mi->get_public_key = LEX_SOURCE_INFO::LEX_MI_DISABLE;
     if (mi && !mi->get_public_key) {
-      // So change master allows new configurations with a running SQL thread
-      lex_mi->get_public_key = LEX_MASTER_INFO::LEX_MI_UNCHANGED;
+      // So change replication source allows new configurations with a running
+      // SQL thread
+      lex_mi->get_public_key = LEX_SOURCE_INFO::LEX_MI_UNCHANGED;
     }
   }
 
@@ -367,16 +364,16 @@ int channel_create(const char *channel, Channel_creation_info *channel_info) {
     lex_mi->zstd_compression_level = channel_info->zstd_compression_level;
   }
 
-  lex_mi->m_source_connection_auto_failover = LEX_MASTER_INFO::LEX_MI_UNCHANGED;
+  lex_mi->m_source_connection_auto_failover = LEX_SOURCE_INFO::LEX_MI_UNCHANGED;
   if (channel_info->m_source_connection_auto_failover) {
     if (mi && !mi->is_source_connection_auto_failover()) {
       lex_mi->m_source_connection_auto_failover =
-          LEX_MASTER_INFO::LEX_MI_ENABLE;
+          LEX_SOURCE_INFO::LEX_MI_ENABLE;
     }
   } else {
     if (mi && mi->is_source_connection_auto_failover()) {
       lex_mi->m_source_connection_auto_failover =
-          LEX_MASTER_INFO::LEX_MI_DISABLE;
+          LEX_SOURCE_INFO::LEX_MI_DISABLE;
     }
   }
 
@@ -423,7 +420,7 @@ int channel_start(const char *channel, Channel_connection_info *connection_info,
   DBUG_TRACE;
   int error = 0;
   int thread_mask = 0;
-  LEX_MASTER_INFO lex_mi;
+  LEX_SOURCE_INFO lex_mi;
   ulong thread_start_id = 0;
   bool thd_created = false;
   THD *thd = current_thd;
@@ -450,16 +447,16 @@ int channel_start(const char *channel, Channel_connection_info *connection_info,
   }
 
   if (threads_to_start & CHANNEL_APPLIER_THREAD) {
-    thread_mask |= SLAVE_SQL;
+    thread_mask |= REPLICA_SQL;
   }
   if (threads_to_start & CHANNEL_RECEIVER_THREAD) {
-    thread_mask |= SLAVE_IO;
+    thread_mask |= REPLICA_IO;
   }
 
   // Nothing to be done here
   if (!thread_mask) goto err;
 
-  LEX_SLAVE_CONNECTION lex_connection;
+  LEX_REPLICA_CONNECTION lex_connection;
   lex_connection.reset();
 
   if (!Rpl_channel_credentials::get_instance().get_credentials(channel, user,
@@ -475,18 +472,18 @@ int channel_start(const char *channel, Channel_connection_info *connection_info,
   if (connection_info->until_condition != CHANNEL_NO_UNTIL_CONDITION) {
     switch (connection_info->until_condition) {
       case CHANNEL_UNTIL_APPLIER_AFTER_GTIDS:
-        lex_mi.gtid_until_condition = LEX_MASTER_INFO::UNTIL_SQL_AFTER_GTIDS;
+        lex_mi.gtid_until_condition = LEX_SOURCE_INFO::UNTIL_SQL_AFTER_GTIDS;
         lex_mi.gtid = connection_info->gtid;
         break;
       case CHANNEL_UNTIL_APPLIER_BEFORE_GTIDS:
-        lex_mi.gtid_until_condition = LEX_MASTER_INFO::UNTIL_SQL_BEFORE_GTIDS;
+        lex_mi.gtid_until_condition = LEX_SOURCE_INFO::UNTIL_SQL_BEFORE_GTIDS;
         lex_mi.gtid = connection_info->gtid;
         break;
       case CHANNEL_UNTIL_APPLIER_AFTER_GAPS:
         lex_mi.until_after_gaps = true;
         break;
       case CHANNEL_UNTIL_VIEW_ID:
-        assert((thread_mask & SLAVE_SQL) && connection_info->view_id);
+        assert((thread_mask & REPLICA_SQL) && connection_info->view_id);
         lex_mi.view_id = connection_info->view_id;
         break;
       default:
@@ -494,7 +491,7 @@ int channel_start(const char *channel, Channel_connection_info *connection_info,
     }
   }
 
-  if (wait_for_connection && (thread_mask & SLAVE_IO))
+  if (wait_for_connection && (thread_mask & REPLICA_IO))
     thread_start_id = mi->slave_run_id;
 
   if (!thd) {
@@ -505,7 +502,7 @@ int channel_start(const char *channel, Channel_connection_info *connection_info,
   error = start_slave(thd, &lex_connection, &lex_mi, thread_mask, mi,
                       use_server_mta_configuration);
 
-  if (wait_for_connection && (thread_mask & SLAVE_IO) && !error) {
+  if (wait_for_connection && (thread_mask & REPLICA_IO) && !error) {
     mysql_mutex_lock(&mi->run_lock);
     /*
       If the ids are still equal this means the start thread method did not
@@ -556,12 +553,12 @@ int channel_stop(Master_info *mi, int threads_to_stop, long timeout) {
   init_thread_mask(&server_thd_mask, mi, false /* not inverse*/);
 
   if ((threads_to_stop & CHANNEL_APPLIER_THREAD) &&
-      (server_thd_mask & SLAVE_SQL)) {
-    thread_mask |= SLAVE_SQL;
+      (server_thd_mask & REPLICA_SQL)) {
+    thread_mask |= REPLICA_SQL;
   }
   if ((threads_to_stop & CHANNEL_RECEIVER_THREAD) &&
-      (server_thd_mask & SLAVE_IO)) {
-    thread_mask |= SLAVE_IO;
+      (server_thd_mask & REPLICA_IO)) {
+    thread_mask |= REPLICA_IO;
   }
   if ((threads_to_stop & CHANNEL_RECEIVER_THREAD) &&
       (server_thd_mask & SLAVE_MONITOR)) {
@@ -721,9 +718,9 @@ bool channel_is_active(const char *channel,
     case CHANNEL_NO_THD:
       return true;  // return true as the channel exists
     case CHANNEL_RECEIVER_THREAD:
-      return thread_mask & SLAVE_IO;
+      return thread_mask & REPLICA_IO;
     case CHANNEL_APPLIER_THREAD:
-      return thread_mask & SLAVE_SQL;
+      return thread_mask & REPLICA_SQL;
     default:
       assert(0);
   }
@@ -837,17 +834,17 @@ long long channel_get_last_delivered_gno(const char *channel, int sidno) {
 
   rpl_gno last_gno = 0;
 
-  Checkable_rwlock *sid_lock = mi->rli->get_sid_lock();
-  sid_lock->rdlock();
+  Checkable_rwlock *tsid_lock = mi->rli->get_tsid_lock();
+  tsid_lock->rdlock();
   last_gno = mi->rli->get_gtid_set()->get_last_gno(sidno);
-  sid_lock->unlock();
+  tsid_lock->unlock();
 
 #if !defined(NDEBUG)
   const Gtid_set *retrieved_gtid_set = mi->rli->get_gtid_set();
   char *retrieved_gtid_set_string = nullptr;
-  sid_lock->wrlock();
+  tsid_lock->wrlock();
   retrieved_gtid_set->to_string(&retrieved_gtid_set_string);
-  sid_lock->unlock();
+  tsid_lock->unlock();
   DBUG_PRINT("info", ("get_last_delivered_gno retrieved_set_string: %s",
                       retrieved_gtid_set_string));
   my_free(retrieved_gtid_set_string);
@@ -868,12 +865,12 @@ int channel_add_executed_gtids_to_received_gtids(const char *channel) {
     return RPL_CHANNEL_SERVICE_CHANNEL_DOES_NOT_EXISTS_ERROR;
   }
 
-  global_sid_lock->wrlock();
+  global_tsid_lock->wrlock();
 
   enum_return_status return_status =
       mi->rli->add_gtid_set(gtid_state->get_executed_gtids());
 
-  global_sid_lock->unlock();
+  global_tsid_lock->unlock();
   channel_map.unlock();
 
   return return_status != RETURN_STATUS_OK;
@@ -916,16 +913,16 @@ int channel_wait_until_apply_queue_applied(const char *channel,
   channel_map.unlock();
 
   /*
-    The retrieved_gtid_set (rli->get_gtid_set) has its own sid_map/sid_lock
-    and do not use global_sid_map/global_sid_lock. Instead of blocking both
+    The retrieved_gtid_set (rli->get_gtid_set) has its own tsid_map/tsid_lock
+    and do not use global_tsid_map/global_tsid_lock. Instead of blocking both
     sid locks on each wait iteration at rli->wait_for_gtid_set(Gtid_set), it
     would be better to use rli->wait_for_gtid_set(char *) that will create a
-    new Gtid_set based on global_sid_map.
+    new Gtid_set based on global_tsid_map.
   */
   char *retrieved_gtid_set_buf;
-  mi->rli->get_sid_lock()->wrlock();
+  mi->rli->get_tsid_lock()->wrlock();
   mi->rli->get_gtid_set()->to_string(&retrieved_gtid_set_buf);
-  mi->rli->get_sid_lock()->unlock();
+  mi->rli->get_tsid_lock()->unlock();
 
   int error = mi->rli->wait_for_gtid_set(current_thd, retrieved_gtid_set_buf,
                                          timeout, false);
@@ -1133,6 +1130,53 @@ int channel_get_network_namespace(const char *channel, std::string &net_ns) {
   return 0;
 }
 
+int channel_get_gtid_set_to_apply(const char *channel,
+                                  std::string &gtid_set_to_apply) {
+  DBUG_TRACE;
+
+  int error = 0;
+  channel_map.rdlock();
+  Master_info *mi = channel_map.get_mi(channel);
+
+  if (mi == nullptr) {
+    channel_map.unlock();
+    return RPL_CHANNEL_SERVICE_CHANNEL_DOES_NOT_EXISTS_ERROR;
+  }
+
+  mi->inc_reference();
+  channel_map.unlock();
+  Tsid_map tsid_map{nullptr};
+  Gtid_set gtids{&tsid_map, nullptr};
+
+  /* RECEIVED_TRANSACTION_SET */
+  mi->rli->get_tsid_lock()->wrlock();
+  if (RETURN_STATUS_OK != gtids.add_gtid_set(mi->rli->get_gtid_set())) {
+    error = 1;
+  }
+  mi->rli->get_tsid_lock()->unlock();
+
+  mi->dec_reference();
+
+  if (error) {
+    return error;
+  }
+
+  /* GTID_EXECUTED */
+  global_tsid_lock->wrlock();
+  gtids.remove_gtid_set(gtid_state->get_executed_gtids());
+  global_tsid_lock->unlock();
+
+  char *gtids_string = nullptr;
+  if (-1 != gtids.to_string(&gtids_string)) {
+    gtid_set_to_apply.assign(gtids_string);
+  } else {
+    error = ER_OUTOFMEMORY;
+  }
+  my_free(gtids_string);
+
+  return error;
+}
+
 bool channel_is_stopping(const char *channel,
                          enum_channel_thread_types thd_type) {
   bool is_stopping = false;
@@ -1211,7 +1255,7 @@ bool is_any_slave_channel_running(int thread_mask) {
     mi = it->second;
 
     if (mi) {
-      if ((thread_mask & SLAVE_IO) != 0) {
+      if ((thread_mask & REPLICA_IO) != 0) {
         mysql_mutex_lock(&mi->run_lock);
         is_running = mi->slave_running;
         mysql_mutex_unlock(&mi->run_lock);
@@ -1221,7 +1265,7 @@ bool is_any_slave_channel_running(int thread_mask) {
         }
       }
 
-      if ((thread_mask & SLAVE_SQL) != 0) {
+      if ((thread_mask & REPLICA_SQL) != 0) {
         mysql_mutex_lock(&mi->rli->run_lock);
         is_running = mi->rli->slave_running;
         mysql_mutex_unlock(&mi->rli->run_lock);
@@ -1250,7 +1294,7 @@ bool is_any_slave_channel_running_with_failover_enabled(int thread_mask) {
 
     if (mi && Master_info::is_configured(mi) &&
         mi->is_source_connection_auto_failover()) {
-      if ((thread_mask & SLAVE_IO) != 0) {
+      if ((thread_mask & REPLICA_IO) != 0) {
         mysql_mutex_lock(&mi->run_lock);
         is_running = mi->slave_running;
         mysql_mutex_unlock(&mi->run_lock);
@@ -1260,7 +1304,7 @@ bool is_any_slave_channel_running_with_failover_enabled(int thread_mask) {
         }
       }
 
-      if ((thread_mask & SLAVE_SQL) != 0) {
+      if ((thread_mask & REPLICA_SQL) != 0) {
         mysql_mutex_lock(&mi->rli->run_lock);
         is_running = mi->rli->slave_running;
         mysql_mutex_unlock(&mi->rli->run_lock);
