@@ -1,4 +1,4 @@
-/* Copyright (c) 2011, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -50,8 +50,11 @@
 bool set_gtid_next(THD *thd, const Gtid_specification &spec) {
   DBUG_TRACE;
 
+  bool is_tagged = false;
+  mysql::gtid::Tsid tsid;
+
   spec.dbug_print();
-  global_sid_lock->assert_some_lock();
+  global_tsid_lock->assert_some_lock();
   // we may acquire and release locks throughout this function; this
   // variable tells the error handler how many are left to release
   int lock_count = 1;
@@ -63,12 +66,12 @@ bool set_gtid_next(THD *thd, const Gtid_specification &spec) {
     char buf[Gtid::MAX_TEXT_LENGTH + 1];
     if (thd->owned_gtid.sidno > 0) {
 #ifndef NDEBUG
-      global_sid_lock->unlock();
-      global_sid_lock->wrlock();
+      global_tsid_lock->unlock();
+      global_tsid_lock->wrlock();
       assert(gtid_state->get_owned_gtids()->thread_owns_anything(
           thd->thread_id()));
 #endif
-      thd->owned_gtid.to_string(thd->owned_sid, buf);
+      thd->owned_gtid.to_string(thd->owned_tsid, buf);
     } else {
       assert(gtid_state->get_anonymous_ownership_count() > 0);
       strcpy(buf, "ANONYMOUS");
@@ -82,7 +85,16 @@ bool set_gtid_next(THD *thd, const Gtid_specification &spec) {
 
   switch (spec.type) {
     case AUTOMATIC_GTID:
-      thd->variables.gtid_next.set_automatic();
+      if (spec.is_automatic_tagged()) {
+        if (global_gtid_mode.get() != Gtid_mode::ON &&
+            global_gtid_mode.get() != Gtid_mode::ON_PERMISSIVE) {
+          my_error(
+              ER_CANT_SET_GTID_NEXT_TO_AUTOMATIC_TAGGED_WHEN_GTID_MODE_IS_OFF,
+              MYF(0));
+          goto err;
+        }
+      }
+      thd->variables.gtid_next.set(spec);
       break;
 
     case ANONYMOUS_GTID:
@@ -111,13 +123,19 @@ bool set_gtid_next(THD *thd, const Gtid_specification &spec) {
       assert(spec.gtid.sidno >= 1);
       assert(spec.gtid.gno >= 1);
       assert(spec.gtid.gno < GNO_END);
+      tsid = global_tsid_map->sidno_to_tsid(spec.gtid.sidno);
+      is_tagged = tsid.is_tagged();
       while (true) {
-        // loop invariant: we should always hold global_sid_lock.rdlock
+        // loop invariant: we should always hold global_tsid_lock.rdlock
         assert(lock_count == 1);
-        global_sid_lock->assert_some_lock();
+        global_tsid_lock->assert_some_lock();
 
         if (global_gtid_mode.get() == Gtid_mode::OFF) {
-          my_error(ER_CANT_SET_GTID_NEXT_TO_GTID_WHEN_GTID_MODE_IS_OFF, MYF(0));
+          auto err = ER_CANT_SET_GTID_NEXT_TO_GTID_WHEN_GTID_MODE_IS_OFF;
+          if (is_tagged) {
+            err = ER_GTID_NEXT_TAG_GTID_MODE_OFF;
+          }
+          my_error(err, MYF(0));
           goto err;
         }
 
@@ -146,11 +164,11 @@ bool set_gtid_next(THD *thd, const Gtid_specification &spec) {
         }
         // GTID owned by someone (other thread)
         else {
-          // The call below releases the read lock on global_sid_lock and
+          // The call below releases the read lock on global_tsid_lock and
           // the mutex lock on SIDNO.
           gtid_state->wait_for_gtid(thd, spec.gtid);
 
-          // global_sid_lock and mutex are now released
+          // global_tsid_lock and mutex are now released
           lock_count = 0;
 
           // Check if thread was killed.
@@ -159,7 +177,7 @@ bool set_gtid_next(THD *thd, const Gtid_specification &spec) {
           }
           // If this thread is a slave SQL thread or slave SQL worker
           // thread, we need this additional condition to determine if it
-          // has been stopped by STOP SLAVE [SQL_THREAD].
+          // has been stopped by STOP REPLICA [SQL_THREAD].
           if ((thd->system_thread &
                (SYSTEM_THREAD_SLAVE_SQL | SYSTEM_THREAD_SLAVE_WORKER)) != 0) {
             // TODO: error is *not* reported on cancel
@@ -169,7 +187,7 @@ bool set_gtid_next(THD *thd, const Gtid_specification &spec) {
               goto err;
             }
           }
-          global_sid_lock->rdlock();
+          global_tsid_lock->rdlock();
           lock_count = 1;
         }
       }  // end while (true)
@@ -193,12 +211,20 @@ bool set_gtid_next(THD *thd, const Gtid_specification &spec) {
       break;
   }
 
+  if (thd->has_incremented_gtid_automatic_count) {
+    gtid_state->decrease_gtid_automatic_tagged_count();
+  }
+  if (spec.is_automatic_tagged()) {
+    gtid_state->increase_gtid_automatic_tagged_count();
+  }
+  thd->has_incremented_gtid_automatic_count = spec.is_automatic_tagged();
+
   ret = false;
 
 err:
   if (lock_count == 2) gtid_state->unlock_sidno(spec.gtid.sidno);
 
-  if (lock_count >= 1) global_sid_lock->unlock();
+  if (lock_count >= 1) global_tsid_lock->unlock();
 
   if (!ret) gtid_set_performance_schema_values(thd);
   thd->owned_gtid.dbug_print(nullptr, "Set owned_gtid in set_gtid_next");
@@ -221,7 +247,7 @@ int gtid_acquire_ownership_multiple(THD *thd) {
     Gtid g = git.get();
     my_thread_id owner = 0;
     rpl_sidno last_sidno = 0;
-    global_sid_lock->rdlock();
+    global_tsid_lock->rdlock();
     while (g.sidno != 0) {
       // lock all SIDNOs in order
       if (g.sidno != last_sidno) gtid_state->lock_sidno(g.sidno);
@@ -249,18 +275,18 @@ int gtid_acquire_ownership_multiple(THD *thd) {
       if (gtid_next_list->contains_sidno(sidno))
         gtid_state->unlock_sidno(sidno);
 
-    // wait. this call releases the read lock on global_sid_lock and
+    // wait. this call releases the read lock on global_tsid_lock and
     // the mutex lock on SIDNO
     gtid_state->wait_for_gtid(thd, g);
 
-    // global_sid_lock and mutex are now released
+    // global_tsid_lock and mutex are now released
 
     // at this point, we don't hold any locks. re-acquire the global
     // read lock that was held when this function was invoked
     if (thd->killed || connection_events_loop_aborted()) return 1;
     // If this thread is a slave SQL thread or slave SQL worker
     // thread, we need this additional condition to determine if it
-    // has been stopped by STOP SLAVE [SQL_THREAD].
+    // has been stopped by STOP REPLICA [SQL_THREAD].
     if ((thd->system_thread &
          (SYSTEM_THREAD_SLAVE_SQL | SYSTEM_THREAD_SLAVE_WORKER)) != 0) {
       assert(thd->rli_slave != nullptr);
@@ -269,13 +295,13 @@ int gtid_acquire_ownership_multiple(THD *thd) {
     }
   }
 
-  // global_sid_lock is now held
+  // global_tsid_lock is now held
   thd->owned_gtid_set.ensure_sidno(greatest_sidno);
 
   /*
     Now the following hold:
      - None of the GTIDs in GTID_NEXT_LIST is owned by any thread.
-     - We hold a lock on global_sid_lock.
+     - We hold a lock on global_tsid_lock.
      - We hold a lock on all SIDNOs in GTID_NEXT_LIST.
     So we acquire ownership of all GTIDs that we need.
   */
@@ -300,7 +326,7 @@ int gtid_acquire_ownership_multiple(THD *thd) {
   for (rpl_sidno sidno = 1; sidno <= max_sidno; sidno++)
     if (gtid_next_list->contains_sidno(sidno)) gtid_state->unlock_sidno(sidno);
 
-  global_sid_lock->unlock();
+  global_tsid_lock->unlock();
 
   /*
     TODO: If this code is enabled, set the GTID in the Performance Schema,
@@ -362,29 +388,24 @@ static inline void skip_statement(THD *thd) {
 #ifdef WITH_WSREP
   /* Despite the transaction was skipped, it needs to be updated in the
    * Wsrep_async_monitor */
-  if (thd->system_thread == SYSTEM_THREAD_SLAVE_WORKER && !thd->wsrep_applier) {
+  if (thd->system_thread == SYSTEM_THREAD_SLAVE_WORKER) {
     Slave_worker *sw = dynamic_cast<Slave_worker *>(thd->rli_slave);
-    // It should never happen. If this is SYSTEM_THREAD_SLAVE_WORKER, but it
-    // is not wsrep applier, it has to be Slave_worker.
-    assert(sw != nullptr);
-    if (sw) {
-      Wsrep_async_monitor *wsrep_async_monitor{sw->get_wsrep_async_monitor()};
-      if (wsrep_async_monitor) {
-        auto seqno = sw->sequence_number();
-        assert(seqno > 0);
-        wsrep_async_monitor->skip(seqno);
-      }
+    Wsrep_async_monitor *wsrep_async_monitor{sw->get_wsrep_async_monitor()};
+    if (wsrep_async_monitor) {
+      auto seqno = sw->sequence_number();
+      assert(seqno > 0);
+      wsrep_async_monitor->skip(seqno);
     }
   }
 #endif /* WITH_WSREP */
 
 #ifndef NDEBUG
   const Gtid_set *executed_gtids = gtid_state->get_executed_gtids();
-  global_sid_lock->rdlock();
+  global_tsid_lock->rdlock();
   gtid_state->lock_sidno(thd->variables.gtid_next.gtid.sidno);
   assert(executed_gtids->contains_gtid(thd->variables.gtid_next.gtid));
   gtid_state->unlock_sidno(thd->variables.gtid_next.gtid.sidno);
-  global_sid_lock->unlock();
+  global_tsid_lock->unlock();
 #endif
 }
 
@@ -407,8 +428,8 @@ bool gtid_reacquire_ownership_if_anonymous(THD *thd) {
     spec.set_anonymous();
     DBUG_PRINT("info", ("acquiring ANONYMOUS ownership"));
 
-    global_sid_lock->rdlock();
-    // set_gtid_next releases global_sid_lock
+    global_tsid_lock->rdlock();
+    // set_gtid_next releases global_tsid_lock
     if (set_gtid_next(thd, spec))
       // this can happen if gtid_mode=on
       return true;
@@ -526,9 +547,9 @@ enum_gtid_statement_status gtid_pre_statement_checks(THD *thd) {
   */
   if (UNDEFINED_GTID == gtid_next->type) {
     char buf[Gtid::MAX_TEXT_LENGTH + 1];
-    global_sid_lock->rdlock();
-    gtid_next->to_string(global_sid_map, buf);
-    global_sid_lock->unlock();
+    global_tsid_lock->rdlock();
+    gtid_next->to_string(global_tsid_map, buf);
+    global_tsid_lock->unlock();
     my_error(ER_GTID_NEXT_TYPE_UNDEFINED_GTID, MYF(0), buf);
     return GTID_STATEMENT_CANCEL;
   }
@@ -612,21 +633,20 @@ void gtid_set_performance_schema_values(const THD *thd [[maybe_unused]]) {
 
     // Thread owns GTID.
     if (thd->owned_gtid.sidno >= 1) {
-      spec.type = ASSIGNED_GTID;
-      spec.gtid = thd->owned_gtid;
+      spec.set(thd->owned_gtid.sidno, thd->owned_gtid.gno);
     }
 
     // Thread owns ANONYMOUS.
     else if (thd->owned_gtid.sidno == THD::OWNED_SIDNO_ANONYMOUS) {
-      spec.type = ANONYMOUS_GTID;
+      spec.set_anonymous();
     }
 
     // Thread does not own anything.
     else {
       assert(thd->owned_gtid.sidno == 0);
-      spec.type = AUTOMATIC_GTID;
+      spec = thd->variables.gtid_next;
     }
-    MYSQL_SET_TRANSACTION_GTID(thd->m_transaction_psi, &thd->owned_sid, &spec);
+    MYSQL_SET_TRANSACTION_GTID(thd->m_transaction_psi, &thd->owned_tsid, &spec);
   }
 #endif
 }
