@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -43,6 +43,7 @@
 #include "sql/sql_class.h"
 #include "sql/sql_error.h"
 #include "sql/sql_lex.h"
+#include "string_with_len.h"
 
 Commit_order_manager::Commit_order_manager(uint32 worker_numbers)
     : m_workers(worker_numbers) {
@@ -233,8 +234,8 @@ bool Commit_order_manager::wait_on_graph(Slave_worker *worker) {
       if (Commit_stage_manager::get_instance()
               .is_ticket_on_its_turn_and_back_ticket_incremented(worker_thd)) {
         char gtid_buf[Gtid::MAX_TEXT_LENGTH + 1];
-        worker_thd->variables.gtid_next.gtid.to_string(global_sid_map, gtid_buf,
-                                                       true);
+        worker_thd->variables.gtid_next.gtid.to_string(global_tsid_map,
+                                                       gtid_buf, true);
         LogErr(WARNING_LEVEL, ER_RPL_MTA_ALLOW_COMMIT_OUT_OF_ORDER, gtid_buf);
         allow_commit_out_of_order = true;
         return false;
@@ -265,6 +266,10 @@ bool Commit_order_manager::wait_on_graph(Slave_worker *worker) {
     worker->report_commit_order_deadlock();
     rollback_status = true;
     return true;
+  } else {
+    // If the worker is at the head of the queue, then no point in killing
+    // a transaction due to an ephemeral deadlock that might have happened.
+    worker->reset_commit_order_deadlock();
   }
   return false;
 }
