@@ -1,4 +1,4 @@
-/* Copyright (c) 2011, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -30,18 +30,18 @@
 #include <atomic>
 #include <tuple>
 
-#include "libbinlogevents/include/binlog_event.h"
 #include "my_bitmap.h"
 #include "my_compiler.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_io.h"
-#include "my_loglevel.h"
 #include "my_psi_config.h"
+#include "mysql/binlog/event/binlog_event.h"
 #include "mysql/components/services/bits/mysql_cond_bits.h"
 #include "mysql/components/services/bits/mysql_mutex_bits.h"
 #include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/components/services/bits/psi_mutex_bits.h"
+#include "mysql/my_loglevel.h"
 #include "mysql/service_mysql_alloc.h"
 #include "prealloced_array.h"  // Prealloced_array
 #include "sql/log_event.h"     // Format_description_log_event
@@ -196,7 +196,7 @@ struct Slave_job_group {
   char *checkpoint_relay_log_name;
   std::atomic<int32> done;  // Flag raised by W,  read and reset by Coordinator
   ulong shifted;            // shift the last CP bitmap at receiving a new CP
-  time_t ts;                // Group's timestamp to update Seconds_behind_master
+  time_t ts;                // Group's timestamp to update Seconds_behind_source
 #ifndef NDEBUG
   bool notified{false};  // to debug group_master_log_name change notification
 #endif
@@ -535,7 +535,7 @@ class Slave_worker : public Relay_log_info {
   ulong wq_empty_waits;            // how many times got idle
   ulong events_done;               // how many events (statements) processed
   ulong groups_done;               // how many groups (transactions) processed
-  volatile int curr_jobs;          // number of active  assignments
+  std::atomic<int> curr_jobs;      // number of active  assignments
   // number of partitions allocated to the worker at point in time
   long usage_partition;
   // symmetric to rli->mts_end_group_sets_max_dbs
@@ -596,14 +596,14 @@ class Slave_worker : public Relay_log_info {
     ERROR_LEAVING = 2,  // is set by Worker
     STOP = 3,           // is set by Coordinator upon receiving STOP
     STOP_ACCEPTED =
-        4  // is set by worker upon completing job when STOP SLAVE is issued
+        4  // is set by worker upon completing job when STOP REPLICA is issued
   };
 
   /*
     This function is used to make a copy of the worker object before we
-    destroy it on STOP SLAVE. This new object is then used to report the
-    worker status until next START SLAVE following which the new worker objects
-    will be used.
+    destroy it on STOP REPLICA. This new object is then used to report the
+    worker status until next START REPLICA following which the new worker
+    objects will be used.
   */
   void copy_values_for_PFS(ulong worker_id, en_running_state running_status,
                            THD *worker_thd, const Error &last_error,
@@ -764,8 +764,8 @@ class Slave_worker : public Relay_log_info {
         gtid_next=automatic. In this scenario, there is no reason to execute a
         Format_description_log_event. So we generate an error.
       */
-      if (info_thd->variables.gtid_next.type == AUTOMATIC_GTID ||
-          info_thd->variables.gtid_next.type == UNDEFINED_GTID) {
+      if (info_thd->variables.gtid_next.is_automatic() ||
+          info_thd->variables.gtid_next.is_undefined()) {
         bool in_active_multi_stmt =
             info_thd->in_active_multi_stmt_transaction();
 
@@ -918,6 +918,11 @@ class Slave_worker : public Relay_log_info {
                  va_list v_args) const override
       MY_ATTRIBUTE((format(printf, 4, 0)));
 
+  void do_report(loglevel level, int err_code,
+                 const Gtid_specification *gtid_next, const char *msg,
+                 va_list v_args) const override
+      MY_ATTRIBUTE((format(printf, 5, 0)));
+
  private:
   ulong gaq_index;           // GAQ index of the current assignment
   ulonglong master_log_pos;  // event's cached log_pos for possible error report
@@ -940,9 +945,12 @@ class Slave_worker : public Relay_log_info {
                              const char *end_event_relay_log_name);
   void assign_partition_db(Log_event *ev);
 
+ public:
+  /**
+    Set the flag the signals a deadlock to false
+  */
   void reset_commit_order_deadlock();
 
- public:
   /**
      Returns an array with the expected column numbers of the primary key
      fields of the table repository.
