@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -588,4 +588,17 @@ class Open_table_context {
 bool is_acl_table_in_non_LTM(const Table_ref *tl,
                              enum enum_locked_tables_mode ltm);
 
+/**
+  Create a table cache/table definition cache key for a table in a
+  secondary storage engine.
+
+  The key is constructed by appending a single byte with the value 1
+  to the key generated by #create_table_def_key().
+
+  @param db_name     the database name
+  @param table_name  the table name
+  @return the key
+*/
+std::string create_table_def_key_secondary(const char *db_name,
+                                           const char *table_name);
 #endif /* SQL_BASE_INCLUDED */
