@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+   Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -42,7 +42,6 @@
 #include <vector>
 
 #include "lex_string.h"
-#include "m_ctype.h"
 #include "m_string.h"
 #include "my_command.h"
 #include "my_dbug.h"
@@ -57,9 +56,12 @@
 #include "mysql/psi/mysql_file.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql/service_mysql_alloc.h"
+#include "mysql/strings/int2str.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
 #include "mysys_err.h"  // EE_*
+#include "nulls.h"
 #include "sql/auth/auth_acls.h"
 #include "sql/auth/auth_common.h"  // SELECT_ACL
 #include "sql/auth/sql_security_ctx.h"
@@ -72,11 +74,10 @@
 #include "sql/dd/string_type.h"
 #include "sql/dd/types/abstract_table.h"
 #include "sql/dd/types/schema.h"
-#include "sql/dd/upgrade_57/upgrade.h"  // dd::upgrade::in_progress
-#include "sql/debug_sync.h"             // DEBUG_SYNC
-#include "sql/derror.h"                 // ER_THD
-#include "sql/error_handler.h"          // Drop_table_error_handler
-#include "sql/events.h"                 // Events
+#include "sql/debug_sync.h"     // DEBUG_SYNC
+#include "sql/derror.h"         // ER_THD
+#include "sql/error_handler.h"  // Drop_table_error_handler
+#include "sql/events.h"         // Events
 #include "sql/handler.h"
 #include "sql/lock.h"       // lock_schema_name
 #include "sql/log.h"        // log_*()
@@ -99,6 +100,9 @@
 #include "sql/thd_raii.h"
 #include "sql/transaction.h"  // trans_rollback_stmt
 #include "sql_string.h"
+#include "string_with_len.h"
+#include "strmake.h"
+#include "strxmov.h"
 #include "typelib.h"
 
 /*
@@ -149,7 +153,7 @@ bool get_default_db_collation(THD *thd, const char *db_name,
                               const CHARSET_INFO **collation) {
   // We must make sure the schema is released and unlocked in the right order.
   dd::Schema_MDL_locker mdl_handler(thd);
-  dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
+  const dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
   const dd::Schema *sch_obj = nullptr;
 
   if (mdl_handler.ensure_locked(db_name) ||
@@ -237,7 +241,7 @@ bool check_schema_readonly(THD *thd, const char *schema_name,
   }
 
   dd::Schema_MDL_locker mdl_handler(thd);
-  dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
+  const dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
   const dd::Schema *sch_obj = nullptr;
 
   if (mdl_handler.ensure_locked(schema_name) ||
@@ -265,7 +269,7 @@ bool check_schema_readonly(THD *thd, const char *schema_name,
 
 static bool write_db_cmd_to_binlog(THD *thd, const char *db, bool trx_cache) {
   if (mysql_bin_log.is_open()) {
-    int errcode = query_error_code(thd, true);
+    const int errcode = query_error_code(thd, true);
     Query_log_event qinfo(thd, thd->query().str, thd->query().length, trx_cache,
                           false, /* suppress_use */ true, errcode);
     /*
@@ -332,7 +336,7 @@ bool mysql_create_db(THD *thd, const char *db, HA_CREATE_INFO *create_info) {
     Use Auto_releaser to keep uncommitted object for database until
     trans_commit() call.
   */
-  dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
+  const dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
 
   // Reject creation of the system schema except for system threads.
   if (!thd->is_dd_system_thread() &&
@@ -387,7 +391,8 @@ bool mysql_create_db(THD *thd, const char *db, HA_CREATE_INFO *create_info) {
   }
 
   bool store_in_dd = true;
-  bool if_not_exists = (create_info->options & HA_LEX_CREATE_IF_NOT_EXISTS);
+  const bool if_not_exists =
+      (create_info->options & HA_LEX_CREATE_IF_NOT_EXISTS);
   if (existing_schema != nullptr) {
     if (if_not_exists == false) {
       my_error(ER_DB_CREATE_EXISTS, MYF(0), db);
@@ -402,8 +407,8 @@ bool mysql_create_db(THD *thd, const char *db, HA_CREATE_INFO *create_info) {
   /* Check directory */
   char path[FN_REFLEN + 16];
   bool was_truncated;
-  size_t path_len = build_table_filename(path, sizeof(path) - 1, db, "", "", 0,
-                                         &was_truncated);
+  const size_t path_len = build_table_filename(path, sizeof(path) - 1, db, "",
+                                               "", 0, &was_truncated);
   if (was_truncated) {
     my_error(ER_IDENT_CAUSES_TOO_LONG_PATH, MYF(0), sizeof(path) - 1, path);
     return true;
@@ -416,10 +421,9 @@ bool mysql_create_db(THD *thd, const char *db, HA_CREATE_INFO *create_info) {
   // the physical representation of the schema is not re-created since it
   // already exists.
   MY_STAT stat_info;
-  bool schema_dir_exists =
+  const bool schema_dir_exists =
       (mysql_file_stat(key_file_misc, path, &stat_info, MYF(0)) != nullptr);
-  if (thd->is_dd_system_thread() &&
-      (!opt_initialize || dd::upgrade_57::in_progress()) &&
+  if (thd->is_dd_system_thread() && !opt_initialize &&
       dd::get_dictionary()->is_dd_schema_name(db)) {
     /*
       CREATE SCHEMA statement is being executed from bootstrap thread.
@@ -534,7 +538,7 @@ bool mysql_alter_db(THD *thd, const char *db, HA_CREATE_INFO *create_info) {
 
   if (lock_schema_name(thd, db)) return true;
 
-  dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
+  const dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
   dd::Schema *schema = nullptr;
   if (thd->dd_client()->acquire_for_modification(db, &schema)) return true;
 
@@ -718,7 +722,7 @@ bool mysql_rm_db(THD *thd, const LEX_CSTRING &db, bool if_exists) {
 
   DBUG_TRACE;
 
-  dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
+  const dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
 
   // Reject dropping the system schema except for system threads.
   if (!thd->is_dd_system_thread() &&
@@ -840,7 +844,7 @@ bool mysql_rm_db(THD *thd, const LEX_CSTRING &db, bool if_exists) {
 
       ha_drop_database(path);
       thd->clear_error(); /* @todo Do not ignore errors */
-      Disable_binlog_guard binlog_guard(thd);
+      const Disable_binlog_guard binlog_guard(thd);
       error = Events::drop_schema_events(thd, *schema);
       error = (error || sp_drop_db_routines(thd, *schema));
     }
@@ -911,7 +915,7 @@ bool mysql_rm_db(THD *thd, const LEX_CSTRING &db, bool if_exists) {
         if (thd->variables.gtid_next.type == ASSIGNED_GTID &&
             dropped_non_atomic) {
           char gtid_buf[Gtid::MAX_TEXT_LENGTH + 1];
-          thd->variables.gtid_next.gtid.to_string(global_sid_map, gtid_buf,
+          thd->variables.gtid_next.gtid.to_string(global_tsid_map, gtid_buf,
                                                   true);
           my_error(ER_CANNOT_LOG_PARTIAL_DROP_DATABASE_WITH_GTID, MYF(0), path,
                    gtid_buf, db.str);
@@ -1363,7 +1367,7 @@ bool mysql_change_db(THD *thd, const LEX_CSTRING &new_db_name,
 
   // We must make sure the schema is released and unlocked in the right order.
   dd::Schema_MDL_locker mdl_handler(thd);
-  dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
+  const dd::cache::Dictionary_client::Auto_releaser releaser(thd->dd_client());
   const dd::Schema *schema = nullptr;
 
   DBUG_TRACE;
@@ -1434,14 +1438,14 @@ bool mysql_change_db(THD *thd, const LEX_CSTRING &new_db_name,
   DBUG_PRINT("info", ("Use database: %s", new_db_file_name.str));
 
   if (sctx->get_active_roles()->size() == 0) {
-    db_access =
-        sctx->check_access(DB_OP_ACLS, new_db_file_name.str)
-            ? DB_OP_ACLS
-            : acl_get(thd, sctx->host().str, sctx->ip().str,
-                      sctx->priv_user().str, new_db_file_name.str, false) |
-                  sctx->master_access(new_db_file_name.str);
+    db_access = sctx->check_access(DB_OP_ACLS, new_db_file_name.str)
+                    ? DB_OP_ACLS
+                    : sctx->check_db_level_access(thd, new_db_file_name.str,
+                                                  new_db_file_name.length) |
+                          sctx->master_access(new_db_file_name.str);
   } else {
-    db_access = sctx->db_acl(new_db_file_name_cstr) |
+    db_access = sctx->check_db_level_access(thd, new_db_file_name.str,
+                                            new_db_file_name.length) |
                 sctx->master_access(new_db_file_name.str);
   }
 
