@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -251,7 +251,7 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
   join_type type = JT_UNKNOWN;
 
   auto cleanup = create_scope_guard([&range_scan, table] {
-    destroy(range_scan);
+    if (range_scan != nullptr) ::destroy_at(range_scan);
     table->set_keyread(false);
     table->file->ha_index_or_rnd_end();
     free_io_cache(table);
@@ -262,8 +262,10 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
   const bool using_limit = limit != HA_POS_ERROR;
 
   if (limit == 0 && thd->lex->is_explain()) {
-    Modification_plan plan(thd, MT_DELETE, table, "LIMIT is zero", true, 0);
-    bool err = explain_single_table_modification(thd, thd, &plan, query_block);
+    const Modification_plan plan(thd, MT_DELETE, table, "LIMIT is zero", true,
+                                 0);
+    const bool err =
+        explain_single_table_modification(thd, thd, &plan, query_block);
     return err;
   }
 
@@ -327,10 +329,10 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
     table->file->info(HA_STATUS_VARIABLE | HA_STATUS_NO_LOCK);
     ha_rows const maybe_deleted = table->file->stats.records;
 
-    Modification_plan plan(thd, MT_DELETE, table, "Deleting all rows", false,
-                           maybe_deleted);
+    const Modification_plan plan(thd, MT_DELETE, table, "Deleting all rows",
+                                 false, maybe_deleted);
     if (lex->is_explain()) {
-      bool err =
+      const bool err =
           explain_single_table_modification(thd, thd, &plan, query_block);
       return err;
     }
@@ -373,9 +375,9 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
       no_rows = true;
 
       if (lex->is_explain()) {
-        Modification_plan plan(thd, MT_DELETE, table, "Impossible WHERE", true,
-                               0);
-        bool err =
+        const Modification_plan plan(thd, MT_DELETE, table, "Impossible WHERE",
+                                     true, 0);
+        const bool err =
             explain_single_table_modification(thd, thd, &plan, query_block);
         return err;
       }
@@ -394,10 +396,10 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
     if (table->all_partitions_pruned_away) {
       no_rows = true;
       if (lex->is_explain()) {
-        Modification_plan plan(thd, MT_DELETE, table,
-                               "No matching rows after partition pruning", true,
-                               0);
-        bool err =
+        const Modification_plan plan(thd, MT_DELETE, table,
+                                     "No matching rows after partition pruning",
+                                     true, 0);
+        const bool err =
             explain_single_table_modification(thd, thd, &plan, query_block);
         return err;
       }
@@ -424,7 +426,8 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
     wrapper.add_utf8_table(delete_table_ref);
 
     if (!no_rows && conds != nullptr) {
-      Key_map keys_to_use(Key_map::ALL_BITS), needed_reg_dummy;
+      const Key_map keys_to_use(Key_map::ALL_BITS);
+      Key_map needed_reg_dummy;
       MEM_ROOT temp_mem_root(key_memory_test_quick_select_exec,
                              thd->variables.range_alloc_block_size);
       no_rows = test_quick_select(
@@ -438,9 +441,9 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
 
     if (no_rows) {
       if (lex->is_explain()) {
-        Modification_plan plan(thd, MT_DELETE, table, "Impossible WHERE", true,
-                               0);
-        bool err =
+        const Modification_plan plan(thd, MT_DELETE, table, "Impossible WHERE",
+                                     true, 0);
+        const bool err =
             explain_single_table_modification(thd, thd, &plan, query_block);
         return err;
       }
@@ -494,12 +497,13 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd) {
       delete_table_ref->fetch_number_of_rows();
       rows = table->file->stats.records;
     }
-    Modification_plan plan(thd, MT_DELETE, table, type, range_scan, conds,
-                           usable_index, limit, false, need_sort, false, rows);
+    const Modification_plan plan(thd, MT_DELETE, table, type, range_scan, conds,
+                                 usable_index, limit, false, need_sort, false,
+                                 rows);
     DEBUG_SYNC(thd, "planned_single_delete");
 
     if (lex->is_explain()) {
-      bool err =
+      const bool err =
           explain_single_table_modification(thd, thd, &plan, query_block);
       return err;
     }
@@ -669,7 +673,7 @@ cleanup:
         we replicate statement-based; otherwise, 'ha_delete_row()' was used to
         delete specific rows which we might log row-based.
       */
-      int log_result =
+      const int log_result =
           thd->binlog_query(query_type, thd->query().str, thd->query().length,
                             transactional_table, false, false, errcode);
 
@@ -708,7 +712,7 @@ bool Sql_cmd_delete::prepare_inner(THD *thd) {
   Opt_trace_object trace_wrapper(trace);
   Opt_trace_object trace_prepare(trace, "delete_preparation");
   trace_prepare.add_select_number(select->select_number);
-  Opt_trace_array trace_steps(trace, "steps");
+  const Opt_trace_array trace_steps(trace, "steps");
 
   apply_semijoin = multitable;
 
@@ -717,7 +721,7 @@ bool Sql_cmd_delete::prepare_inner(THD *thd) {
 
   const Access_bitmask want_privilege_saved = thd->want_privilege;
   thd->want_privilege = SELECT_ACL;
-  enum enum_mark_columns mark_used_columns_saved = thd->mark_used_columns;
+  const enum enum_mark_columns mark_used_columns_saved = thd->mark_used_columns;
   thd->mark_used_columns = MARK_COLUMNS_READ;
 
   if (select->derived_table_count || select->table_func_count) {
@@ -794,7 +798,7 @@ bool Sql_cmd_delete::prepare_inner(THD *thd) {
     if (!select->m_table_nest.empty())
       propagate_nullability(&select->m_table_nest, false);
 
-    Prepared_stmt_arena_holder ps_holder(thd);
+    const Prepared_stmt_arena_holder ps_holder(thd);
     result = new (thd->mem_root) Query_result_delete;
     if (result == nullptr) return true; /* purecov: inspected */
 
@@ -900,9 +904,9 @@ bool Sql_cmd_delete::prepare_inner(THD *thd) {
 bool Sql_cmd_delete::execute_inner(THD *thd) {
   if (is_empty_query()) {
     if (lex->is_explain()) {
-      Modification_plan plan(thd, MT_DELETE, /*table_arg=*/nullptr,
-                             "No matching rows after partition pruning", true,
-                             0);
+      const Modification_plan plan(thd, MT_DELETE, /*table_arg=*/nullptr,
+                                   "No matching rows after partition pruning",
+                                   true, 0);
       return explain_single_table_modification(thd, thd, &plan,
                                                lex->query_block);
     }
@@ -1332,7 +1336,7 @@ table_map GetImmediateDeleteTables(const JOIN *join, table_map delete_tables) {
   // in which case join order optimization is short-circuited. See
   // JOIN::create_access_paths_for_zero_rows().
   if (join->thd->lex->using_hypergraph_optimizer()) {
-    assert(join->root_access_path()->type == AccessPath::ZERO_ROWS);
+    assert(join->zero_result_cause != nullptr);
     return 0;
   }
 
