@@ -1,4 +1,4 @@
-/* Copyright (c) 2022, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2022, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -480,7 +480,7 @@ static bool is_generated_invisible_primary_key(const KEY *key) {
   @retval   KEY*     KEY instance of a generated primary key.
   @retval   nullptr  If table does not have a generated invisible primary key.
 */
-static const KEY *find_generated_invisible_primary_key(TABLE *table) {
+static const KEY *find_generated_invisible_primary_key(const TABLE *table) {
   assert(table != nullptr && table->s != nullptr);
   // GIPK is not supported for the partitioned table for now.
   if (table->part_info != nullptr) return nullptr;
@@ -493,6 +493,6 @@ static const KEY *find_generated_invisible_primary_key(TABLE *table) {
   return nullptr;
 }
 
-bool table_has_generated_invisible_primary_key(TABLE *table) {
+bool table_has_generated_invisible_primary_key(const TABLE *table) {
   return (find_generated_invisible_primary_key(table) != nullptr);
 }
