@@ -1,4 +1,4 @@
-/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -142,6 +142,7 @@ void update_func_str(THD *, SYS_VAR *, void *tgt, const void *save);
 void update_func_double(THD *, SYS_VAR *, void *tgt, const void *save);
 
 SHOW_TYPE pluginvar_show_type(SYS_VAR *plugin_var);
+const void *pluginvar_default_value(SYS_VAR *plugin_var);
 
 int item_value_type(st_mysql_value *value);
 const char *item_val_str(st_mysql_value *value, char *buffer, int *length);
@@ -249,7 +250,8 @@ class sys_var_pluginvar : public sys_var {
                         : (plugin_var_arg->flags & PLUGIN_VAR_RQCMDARG
                                ? REQUIRED_ARG
                                : REQUIRED_ARG))),
-            pluginvar_show_type(plugin_var_arg), 0, nullptr,
+            pluginvar_show_type(plugin_var_arg),
+            (intptr)pluginvar_default_value(plugin_var_arg), nullptr,
             VARIABLE_NOT_IN_BINLOG,
             (plugin_var_arg->flags & PLUGIN_VAR_NODEFAULT) ? on_check_pluginvar
                                                            : nullptr,
@@ -290,7 +292,7 @@ class sys_var_pluginvar : public sys_var {
     @param[in]     var   Plugin variable.
     @param[in,out] dest  Destination string pointer.
   */
-  void persist_only_to_string(THD *, set_var *var, String *dest) override;
+  void persist_only_to_string(THD *thd, set_var *var, String *dest) override;
   void global_save_default(THD *, set_var *) override {}
   bool session_update(THD *thd, set_var *var) override;
   bool global_update(THD *thd, set_var *var) override;
