@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -57,6 +57,7 @@
 #include "sql/sql_servers.h"  // servers_reload
 #include "sql/system_variables.h"
 #include "sql/table.h"
+#include "string_with_len.h"
 
 #ifdef WITH_WSREP
 #include "sql/sql_lex.h"
@@ -78,11 +79,6 @@
 bool is_reload_request_denied(THD *thd, unsigned long op_type) {
   DBUG_TRACE;
   Security_context *sctx = thd->security_context();
-
-  if ((op_type & REFRESH_HOSTS)) {
-    push_deprecated_warn(thd, "FLUSH HOSTS",
-                         "TRUNCATE TABLE performance_schema.host_cache");
-  }
   /*
     First check RELOAD and stop if it's granted.
     No need to pass DB since it's a global priv
@@ -90,6 +86,12 @@ bool is_reload_request_denied(THD *thd, unsigned long op_type) {
   if (sctx->check_access(RELOAD_ACL, thd->db().str ? thd->db().str : "", true))
     return false;
 
+  if ((op_type & REFRESH_GRANT) &&
+      !sctx->has_global_grant(STRING_WITH_LEN("FLUSH_PRIVILEGES")).first) {
+    my_error(ER_SPECIFIC_ACCESS_DENIED_ERROR, MYF(0),
+             "RELOAD or FLUSH_PRIVILEGES");
+    return true;
+  }
   if ((op_type & REFRESH_OPTIMIZER_COSTS) &&
       !sctx->has_global_grant(STRING_WITH_LEN("FLUSH_OPTIMIZER_COSTS")).first) {
     my_error(ER_SPECIFIC_ACCESS_DENIED_ERROR, MYF(0),
@@ -113,9 +115,9 @@ bool is_reload_request_denied(THD *thd, unsigned long op_type) {
     return true;
   }
 
-  op_type &=
-      ~(REFRESH_OPTIMIZER_COSTS | REFRESH_STATUS | REFRESH_USER_RESOURCES |
-        REFRESH_TABLES | REFRESH_FOR_EXPORT | REFRESH_READ_LOCK);
+  op_type &= ~(REFRESH_GRANT | REFRESH_OPTIMIZER_COSTS | REFRESH_STATUS |
+               REFRESH_USER_RESOURCES | REFRESH_TABLES | REFRESH_FOR_EXPORT |
+               REFRESH_READ_LOCK);
 
   /* if it was just the above we're good */
   if (!op_type) return false;
@@ -366,6 +368,30 @@ bool handle_reload_request(THD *thd, unsigned long options, Table_ref *tables,
       }
 
 #ifdef WITH_WSREP
+      if (WSREP(thd) && !thd->lex->no_write_to_binlog &&
+          (options & REFRESH_TABLES) &&
+          !(options & (REFRESH_FOR_EXPORT | REFRESH_READ_LOCK))) {
+        /*
+          This is done here because LOCK TABLES is not replicated in galera,
+          the upgrade of which is checked above.  Hence, done after/if we
+          are able to upgrade locks.
+
+          Also, note that, in error log with debug you may see
+          'thread holds MDL locks at TI' but since this is a flush
+          tables and is required for LOCK TABLE WRITE
+          it can be ignored there.
+        */
+        if (tables) {
+          if (wsrep_to_isolation_begin(thd, NULL, NULL, tables)) {
+            result = 1;
+            goto cleanup;
+          }
+        } else if (wsrep_to_isolation_begin(thd, WSREP_MYSQL_DB, NULL, NULL)) {
+          result = 1;
+          goto cleanup;
+        }
+      }
+
       if (thd && (thd->wsrep_applier || thd->slave_thread)) {
         /*
           In case of wsrep-applier/mysql-slave thread, do not wait for table
@@ -403,23 +429,25 @@ bool handle_reload_request(THD *thd, unsigned long options, Table_ref *tables,
       }
     }
   }
-  if (options & REFRESH_HOSTS) hostname_cache_refresh();
+#ifdef WITH_WSREP
+cleanup:
+#endif /* WITH_WSREP */
   if (thd && (options & REFRESH_STATUS)) refresh_status();
-  if (options & REFRESH_THREADS)
-    Per_thread_connection_handler::kill_blocked_pthreads();
-  if (options & REFRESH_MASTER) {
+  if (options & REFRESH_SOURCE) {
     assert(thd);
     tmp_write_to_binlog = 0;
     /*
-      RESET MASTER acquired global read lock (if the thread is not acquired
-      already) to make sure no transaction commits are getting executed
-      while the operation is in process. If (and only if) it is
-      acquired by RESET MASTER internal process (options will contain
-      REFRESH_READ_LOCK flag in this case), unlock the global read lock
-      in reset_master().
+      RESET BINARY LOGS AND GTIDS acquired global read lock (if the thread is
+      not acquired already) to make sure no transaction commits are getting
+      executed while the operation is in process. If (and only if) it is
+      acquired by RESET BINARY LOGS AND GTIDS internal process (options will
+      contain REFRESH_READ_LOCK flag in this case), unlock the global read lock
+      in reset_binary_logs_and_gtids().
     */
-    if (reset_master(thd, options & REFRESH_READ_LOCK)) {
-      /* NOTE: my_error() has been already called by reset_master(). */
+    if (reset_binary_logs_and_gtids(thd, options & REFRESH_READ_LOCK)) {
+      /* NOTE: my_error() has been already called by
+       * reset_binary_logs_and_gtids().
+       */
       result = true;
     }
   }
