@@ -1,4 +1,4 @@
-/* Copyright (c) 2005, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2005, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -480,11 +480,12 @@ class Sql_cmd_show_keys : public Sql_cmd_show_table_base {
   Sql_cmd_show_keys() : Sql_cmd_show_table_base(SQLCOM_SHOW_KEYS) {}
 };
 
-/// Represents SHOW MASTER STATUS statement.
+/// Represents SHOW BINARY LOG STATUS statement.
 
-class Sql_cmd_show_master_status : public Sql_cmd_show_noplan {
+class Sql_cmd_show_binary_log_status : public Sql_cmd_show_noplan {
  public:
-  Sql_cmd_show_master_status() : Sql_cmd_show_noplan(SQLCOM_SHOW_MASTER_STAT) {}
+  Sql_cmd_show_binary_log_status()
+      : Sql_cmd_show_noplan(SQLCOM_SHOW_BINLOG_STATUS) {}
   bool check_privileges(THD *thd) override;
   bool execute_inner(THD *thd) override;
 };
@@ -531,6 +532,13 @@ class Sql_cmd_show_processlist : public Sql_cmd_show {
   bool m_use_pfs{false};
 };
 
+/// Represents SHOW PARSE_TREE statement.
+
+class Sql_cmd_show_parse_tree : public Sql_cmd_show {
+ public:
+  Sql_cmd_show_parse_tree() : Sql_cmd_show(SQLCOM_SHOW_PARSE_TREE) {}
+};
+
 /// Represents SHOW PROFILE statement.
 
 class Sql_cmd_show_profile : public Sql_cmd_show {
@@ -565,7 +573,7 @@ class Sql_cmd_show_relaylog_events : public Sql_cmd_show_noplan {
 
 class Sql_cmd_show_replicas : public Sql_cmd_show_noplan {
  public:
-  Sql_cmd_show_replicas() : Sql_cmd_show_noplan(SQLCOM_SHOW_SLAVE_HOSTS) {}
+  Sql_cmd_show_replicas() : Sql_cmd_show_noplan(SQLCOM_SHOW_REPLICAS) {}
   bool check_privileges(THD *thd) override;
   bool execute_inner(THD *thd) override;
 };
@@ -574,7 +582,8 @@ class Sql_cmd_show_replicas : public Sql_cmd_show_noplan {
 
 class Sql_cmd_show_replica_status : public Sql_cmd_show_noplan {
  public:
-  Sql_cmd_show_replica_status() : Sql_cmd_show_noplan(SQLCOM_SHOW_SLAVE_STAT) {}
+  Sql_cmd_show_replica_status()
+      : Sql_cmd_show_noplan(SQLCOM_SHOW_REPLICA_STATUS) {}
   bool check_privileges(THD *thd) override;
   bool execute_inner(THD *thd) override;
 };
