@@ -1,4 +1,4 @@
-/* Copyright (c) 2020, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2020, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -40,6 +40,8 @@ std::string mysql_admin_channel("mysql_admin");
 
 /** SSL context options */
 
+bool opt_tls_certificates_enforced_validation{false};
+
 /* Related to client server connection port */
 static const char *opt_ssl_ca = nullptr;
 static const char *opt_ssl_key = nullptr;
@@ -65,6 +67,7 @@ static const char *opt_admin_tls_ciphersuites = nullptr;
 static const char *opt_admin_ssl_crl = nullptr;
 static const char *opt_admin_ssl_crlpath = nullptr;
 static const char *opt_admin_tls_version = nullptr;
+bool opt_admin_ssl_configured = false;
 
 static PolyLock_mutex lock_admin_ssl_ctx(&LOCK_admin_tls_ctx_options);
 
@@ -89,9 +92,16 @@ static bool check_admin_tls_version(sys_var *, THD *, set_var *var) {
   return check_tls_version(nullptr, nullptr, var);
 }
 
-void validate_ciphers(const char *option, const char *val,
+bool admin_tls_configured(sys_var *, THD *, enum_var_type) {
+  opt_admin_ssl_configured = true;
+  return false;
+}
+
+bool validate_ciphers(const char *option, const char *val,
                       TLS_version version) {
-  if (!val || !*val) return;
+  bool retval = false;
+  /* If nothing is specified viosslfactories.cc will use default ciphers */
+  if (!val || !*val) return retval;
   std::string ciphers{val};
   std::string haystack{};
   switch (version) {
@@ -108,29 +118,31 @@ void validate_ciphers(const char *option, const char *val,
 
   while (index != std::string::npos) {
     auto needle = ciphers.substr(0, index);
+    needle.erase(std::remove(needle.begin(), needle.end(), ' '), needle.end());
     if ((needle[0] != '!') && (haystack.find(needle) == std::string::npos)) {
-      LogErr(WARNING_LEVEL, ER_WARN_DEPRECATED_OR_BLOCKED_CIPHER, option,
-             needle.c_str());
+      LogErr(ERROR_LEVEL, ER_BLOCKED_CIPHER, option, needle.c_str());
+      retval = true;
     }
     ciphers.erase(0, index + 1);
     index = ciphers.find(':');
   }
+  ciphers.erase(std::remove(ciphers.begin(), ciphers.end(), ' '),
+                ciphers.end());
   if ((ciphers[0] != '!') && (haystack.find(ciphers) == std::string::npos)) {
-    LogErr(WARNING_LEVEL, ER_WARN_DEPRECATED_OR_BLOCKED_CIPHER, option,
-           ciphers.c_str());
+    LogErr(ERROR_LEVEL, ER_BLOCKED_CIPHER, option, ciphers.c_str());
+    retval = true;
   }
+  return retval;
 }
 
 static bool check_tls12_ciphers(sys_var *var, THD *, set_var *value) {
-  validate_ciphers(var->name.str, value->save_result.string_value.str,
-                   TLS_version::TLSv12);
-  return false;
+  return validate_ciphers(var->name.str, value->save_result.string_value.str,
+                          TLS_version::TLSv12);
 }
 
 static bool check_tls13_ciphers(sys_var *var, THD *, set_var *value) {
-  validate_ciphers(var->name.str, value->save_result.string_value.str,
-                   TLS_version::TLSv13);
-  return false;
+  return validate_ciphers(var->name.str, value->save_result.string_value.str,
+                          TLS_version::TLSv13);
 }
 
 /*
@@ -140,17 +152,16 @@ static bool check_tls13_ciphers(sys_var *var, THD *, set_var *value) {
 */
 
 /* Related to client server connection port */
-static Sys_var_charptr Sys_ssl_ca(
-    "ssl_ca", "CA file in PEM format (check OpenSSL docs, implies --ssl)",
-    PERSIST_AS_READONLY GLOBAL_VAR(opt_ssl_ca),
-    CMD_LINE(REQUIRED_ARG, OPT_SSL_CA), IN_FS_CHARSET, DEFAULT(nullptr),
-    &lock_ssl_ctx);
+static Sys_var_charptr Sys_ssl_ca("ssl_ca",
+                                  "CA file in PEM format (check OpenSSL docs)",
+                                  PERSIST_AS_READONLY GLOBAL_VAR(opt_ssl_ca),
+                                  CMD_LINE(REQUIRED_ARG), IN_FS_CHARSET,
+                                  DEFAULT(nullptr), &lock_ssl_ctx);
 
 static Sys_var_charptr Sys_ssl_capath(
-    "ssl_capath", "CA directory (check OpenSSL docs, implies --ssl)",
-    PERSIST_AS_READONLY GLOBAL_VAR(opt_ssl_capath),
-    CMD_LINE(REQUIRED_ARG, OPT_SSL_CAPATH), IN_FS_CHARSET, DEFAULT(nullptr),
-    &lock_ssl_ctx);
+    "ssl_capath", "CA directory (check OpenSSL docs)",
+    PERSIST_AS_READONLY GLOBAL_VAR(opt_ssl_capath), CMD_LINE(REQUIRED_ARG),
+    IN_FS_CHARSET, DEFAULT(nullptr), &lock_ssl_ctx);
 
 static Sys_var_charptr Sys_tls_version(
     "tls_version",
@@ -169,46 +180,41 @@ static Sys_var_charptr Sys_tls_version(
     &lock_ssl_ctx, NOT_IN_BINLOG, ON_CHECK(check_tls_version));
 
 static Sys_var_charptr Sys_ssl_cert(
-    "ssl_cert", "X509 cert in PEM format (implies --ssl)",
-    PERSIST_AS_READONLY GLOBAL_VAR(opt_ssl_cert),
-    CMD_LINE(REQUIRED_ARG, OPT_SSL_CERT), IN_FS_CHARSET, DEFAULT(nullptr),
-    &lock_ssl_ctx);
+    "ssl_cert", "X509 cert in PEM format",
+    PERSIST_AS_READONLY GLOBAL_VAR(opt_ssl_cert), CMD_LINE(REQUIRED_ARG),
+    IN_FS_CHARSET, DEFAULT(nullptr), &lock_ssl_ctx);
 
 static Sys_var_charptr Sys_ssl_cipher(
-    "ssl_cipher", "SSL cipher to use (implies --ssl)",
+    "ssl_cipher", "SSL cipher to use",
     PERSIST_AS_READONLY GLOBAL_VAR(opt_ssl_cipher),
     CMD_LINE(REQUIRED_ARG, OPT_SSL_CIPHER), IN_FS_CHARSET, DEFAULT(nullptr),
     &lock_ssl_ctx, NOT_IN_BINLOG, ON_CHECK(check_tls12_ciphers));
 
 static Sys_var_charptr Sys_tls_ciphersuites(
-    "tls_ciphersuites", "TLS v1.3 ciphersuite to use (implies --ssl)",
+    "tls_ciphersuites", "TLS v1.3 ciphersuite to use",
     PERSIST_AS_READONLY GLOBAL_VAR(opt_tls_ciphersuites),
     CMD_LINE(REQUIRED_ARG, OPT_TLS_CIPHERSUITES), IN_FS_CHARSET,
     DEFAULT(nullptr), &lock_ssl_ctx, NOT_IN_BINLOG,
     ON_CHECK(check_tls13_ciphers));
 
-static Sys_var_charptr Sys_ssl_key("ssl_key",
-                                   "X509 key in PEM format (implies --ssl)",
+static Sys_var_charptr Sys_ssl_key("ssl_key", "X509 key in PEM format",
                                    PERSIST_AS_READONLY GLOBAL_VAR(opt_ssl_key),
-                                   CMD_LINE(REQUIRED_ARG, OPT_SSL_KEY),
-                                   IN_FS_CHARSET, DEFAULT(nullptr),
-                                   &lock_ssl_ctx);
+                                   CMD_LINE(REQUIRED_ARG), IN_FS_CHARSET,
+                                   DEFAULT(nullptr), &lock_ssl_ctx);
 
 static Sys_var_charptr Sys_ssl_crl(
-    "ssl_crl", "CRL file in PEM format (check OpenSSL docs, implies --ssl)",
-    PERSIST_AS_READONLY GLOBAL_VAR(opt_ssl_crl),
-    CMD_LINE(REQUIRED_ARG, OPT_SSL_CRL), IN_FS_CHARSET, DEFAULT(nullptr),
-    &lock_ssl_ctx);
+    "ssl_crl", "CRL file in PEM format (check OpenSSL docs)",
+    PERSIST_AS_READONLY GLOBAL_VAR(opt_ssl_crl), CMD_LINE(REQUIRED_ARG),
+    IN_FS_CHARSET, DEFAULT(nullptr), &lock_ssl_ctx);
 
 static Sys_var_charptr Sys_ssl_crlpath(
-    "ssl_crlpath", "CRL directory (check OpenSSL docs, implies --ssl)",
-    PERSIST_AS_READONLY GLOBAL_VAR(opt_ssl_crlpath),
-    CMD_LINE(REQUIRED_ARG, OPT_SSL_CRLPATH), IN_FS_CHARSET, DEFAULT(nullptr),
-    &lock_ssl_ctx);
+    "ssl_crlpath", "CRL directory (check OpenSSL docs)",
+    PERSIST_AS_READONLY GLOBAL_VAR(opt_ssl_crlpath), CMD_LINE(REQUIRED_ARG),
+    IN_FS_CHARSET, DEFAULT(nullptr), &lock_ssl_ctx);
 
 #define PFS_TRAILING_PROPERTIES                                         \
-  NO_MUTEX_GUARD, NOT_IN_BINLOG, ON_CHECK(NULL), ON_UPDATE(NULL), NULL, \
-      sys_var::PARSE_EARLY
+  NO_MUTEX_GUARD, NOT_IN_BINLOG, ON_CHECK(nullptr), ON_UPDATE(nullptr), \
+      nullptr, sys_var::PARSE_EARLY
 
 static Sys_var_bool Sys_var_opt_ssl_session_cache_mode(
     "ssl_session_cache_mode", "Is TLS session cache enabled or not",
@@ -220,25 +226,25 @@ static Sys_var_long Sys_var_opt_ssl_session_cache_timeout(
     "ssl_session_cache_timeout",
     "The timeout to expire sessions in the TLS session cache",
     PERSIST_AS_READONLY GLOBAL_VAR(opt_ssl_session_cache_timeout),
-    CMD_LINE(REQUIRED_ARG, OPT_SSL_SESSION_CACHE_TIMEOUT),
-    VALID_RANGE(0, 84600), DEFAULT(300), BLOCK_SIZE(1),
+    CMD_LINE(REQUIRED_ARG), VALID_RANGE(0, 84600), DEFAULT(300), BLOCK_SIZE(1),
     PFS_TRAILING_PROPERTIES);
 
 /* Related to admin connection port */
 static Sys_var_charptr Sys_admin_ssl_ca(
     "admin_ssl_ca",
-    "CA file in PEM format (check OpenSSL docs, implies --ssl) for "
+    "CA file in PEM format (check OpenSSL docs) for "
     "--admin-port",
     PERSIST_AS_READONLY GLOBAL_VAR(opt_admin_ssl_ca),
-    CMD_LINE(REQUIRED_ARG, OPT_SSL_CA), IN_FS_CHARSET, DEFAULT(nullptr),
-    &lock_admin_ssl_ctx);
+    CMD_LINE(REQUIRED_ARG, OPT_ADMIN_SSL_CA), IN_FS_CHARSET, DEFAULT(nullptr),
+    &lock_admin_ssl_ctx, NOT_IN_BINLOG, ON_CHECK(nullptr),
+    ON_UPDATE(admin_tls_configured));
 
 static Sys_var_charptr Sys_admin_ssl_capath(
-    "admin_ssl_capath",
-    "CA directory (check OpenSSL docs, implies --ssl) for --admin-port",
+    "admin_ssl_capath", "CA directory (check OpenSSL docs) for --admin-port",
     PERSIST_AS_READONLY GLOBAL_VAR(opt_admin_ssl_capath),
-    CMD_LINE(REQUIRED_ARG, OPT_SSL_CAPATH), IN_FS_CHARSET, DEFAULT(nullptr),
-    &lock_admin_ssl_ctx);
+    CMD_LINE(REQUIRED_ARG, OPT_ADMIN_SSL_CAPATH), IN_FS_CHARSET,
+    DEFAULT(nullptr), &lock_admin_ssl_ctx, NOT_IN_BINLOG, ON_CHECK(nullptr),
+    ON_UPDATE(admin_tls_configured));
 
 static Sys_var_charptr Sys_admin_tls_version(
     "admin_tls_version",
@@ -248,55 +254,58 @@ static Sys_var_charptr Sys_admin_tls_version(
     "TLS version for --admin-port, permitted values are TLSv1.2",
 #endif
     PERSIST_AS_READONLY GLOBAL_VAR(opt_admin_tls_version),
-    CMD_LINE(REQUIRED_ARG, OPT_TLS_VERSION), IN_FS_CHARSET,
+    CMD_LINE(REQUIRED_ARG, OPT_ADMIN_TLS_VERSION), IN_FS_CHARSET,
 #ifdef HAVE_TLSv13
     "TLSv1.2,TLSv1.3",
 #else
     "TLSv1.2",
 #endif /* HAVE_TLSv13 */
-    &lock_admin_ssl_ctx, NOT_IN_BINLOG, ON_CHECK(check_admin_tls_version));
+    &lock_admin_ssl_ctx, NOT_IN_BINLOG, ON_CHECK(check_admin_tls_version),
+    ON_UPDATE(admin_tls_configured));
 
 static Sys_var_charptr Sys_admin_ssl_cert(
-    "admin_ssl_cert",
-    "X509 cert in PEM format (implies --ssl) for --admin-port",
+    "admin_ssl_cert", "X509 cert in PEM format for --admin-port",
     PERSIST_AS_READONLY GLOBAL_VAR(opt_admin_ssl_cert),
-    CMD_LINE(REQUIRED_ARG, OPT_SSL_CERT), IN_FS_CHARSET, DEFAULT(nullptr),
-    &lock_admin_ssl_ctx);
+    CMD_LINE(REQUIRED_ARG, OPT_ADMIN_SSL_CERT), IN_FS_CHARSET, DEFAULT(nullptr),
+    &lock_admin_ssl_ctx, NOT_IN_BINLOG, ON_CHECK(nullptr),
+    ON_UPDATE(admin_tls_configured));
 
 static Sys_var_charptr Sys_admin_ssl_cipher(
-    "admin_ssl_cipher", "SSL cipher to use (implies --ssl) for --admin-port",
+    "admin_ssl_cipher", "SSL cipher to use for --admin-port",
     PERSIST_AS_READONLY GLOBAL_VAR(opt_admin_ssl_cipher),
-    CMD_LINE(REQUIRED_ARG, OPT_SSL_CIPHER), IN_FS_CHARSET, DEFAULT(nullptr),
-    &lock_admin_ssl_ctx, NOT_IN_BINLOG, ON_CHECK(check_tls12_ciphers));
+    CMD_LINE(REQUIRED_ARG, OPT_ADMIN_SSL_CIPHER), IN_FS_CHARSET,
+    DEFAULT(nullptr), &lock_admin_ssl_ctx, NOT_IN_BINLOG,
+    ON_CHECK(check_tls12_ciphers), ON_UPDATE(admin_tls_configured));
 
 static Sys_var_charptr Sys_admin_tls_ciphersuites(
-    "admin_tls_ciphersuites",
-    "TLS v1.3 ciphersuite to use (implies --ssl) for --admin-port",
+    "admin_tls_ciphersuites", "TLS v1.3 ciphersuite to use for --admin-port",
     PERSIST_AS_READONLY GLOBAL_VAR(opt_admin_tls_ciphersuites),
-    CMD_LINE(REQUIRED_ARG, OPT_TLS_CIPHERSUITES), IN_FS_CHARSET,
+    CMD_LINE(REQUIRED_ARG, OPT_ADMIN_TLS_CIPHERSUITES), IN_FS_CHARSET,
     DEFAULT(nullptr), &lock_admin_ssl_ctx, NOT_IN_BINLOG,
-    ON_CHECK(check_tls13_ciphers));
+    ON_CHECK(check_tls13_ciphers), ON_UPDATE(admin_tls_configured));
 
 static Sys_var_charptr Sys_admin_ssl_key(
-    "admin_ssl_key", "X509 key in PEM format (implies --ssl) for --admin-port",
+    "admin_ssl_key", "X509 key in PEM format for --admin-port",
     PERSIST_AS_READONLY GLOBAL_VAR(opt_admin_ssl_key),
-    CMD_LINE(REQUIRED_ARG, OPT_SSL_KEY), IN_FS_CHARSET, DEFAULT(nullptr),
-    &lock_admin_ssl_ctx);
+    CMD_LINE(REQUIRED_ARG, OPT_ADMIN_SSL_KEY), IN_FS_CHARSET, DEFAULT(nullptr),
+    &lock_admin_ssl_ctx, NOT_IN_BINLOG, ON_CHECK(nullptr),
+    ON_UPDATE(admin_tls_configured));
 
 static Sys_var_charptr Sys_admin_ssl_crl(
     "admin_ssl_crl",
-    "CRL file in PEM format (check OpenSSL docs, implies --ssl) for "
+    "CRL file in PEM format (check OpenSSL docs) for "
     "--admin-port",
     PERSIST_AS_READONLY GLOBAL_VAR(opt_admin_ssl_crl),
-    CMD_LINE(REQUIRED_ARG, OPT_SSL_CRL), IN_FS_CHARSET, DEFAULT(nullptr),
-    &lock_admin_ssl_ctx);
+    CMD_LINE(REQUIRED_ARG, OPT_ADMIN_SSL_CRL), IN_FS_CHARSET, DEFAULT(nullptr),
+    &lock_admin_ssl_ctx, NOT_IN_BINLOG, ON_CHECK(nullptr),
+    ON_UPDATE(admin_tls_configured));
 
 static Sys_var_charptr Sys_admin_ssl_crlpath(
-    "admin_ssl_crlpath",
-    "CRL directory (check OpenSSL docs, implies --ssl) for --admin-port",
+    "admin_ssl_crlpath", "CRL directory (check OpenSSL docs) for --admin-port",
     PERSIST_AS_READONLY GLOBAL_VAR(opt_admin_ssl_crlpath),
-    CMD_LINE(REQUIRED_ARG, OPT_SSL_CRLPATH), IN_FS_CHARSET, DEFAULT(nullptr),
-    &lock_admin_ssl_ctx);
+    CMD_LINE(REQUIRED_ARG, OPT_ADMIN_SSL_CRLPATH), IN_FS_CHARSET,
+    DEFAULT(nullptr), &lock_admin_ssl_ctx, NOT_IN_BINLOG, ON_CHECK(nullptr),
+    ON_UPDATE(admin_tls_configured));
 
 /* Helper functions */
 static bool warn_self_signed_ca_certs(const char *ssl_ca,
@@ -393,7 +402,7 @@ void Ssl_init_callback_server_main::read_parameters(
     OptionalString *cert, OptionalString *cipher, OptionalString *ciphersuites,
     OptionalString *key, OptionalString *crl, OptionalString *crl_path,
     bool *session_cache_mode, long *session_cache_timeout) {
-  AutoRLock lock(&lock_ssl_ctx);
+  const AutoRLock lock(&lock_ssl_ctx);
   if (ca) ca->assign(opt_ssl_ca);
   if (capath) capath->assign(opt_ssl_capath);
   if (version) version->assign(opt_tls_version);
@@ -449,7 +458,7 @@ ssl_artifacts_status Ssl_init_callback_server_main::auto_detect_ssl() {
 
 bool Ssl_init_callback_server_main::provision_certs() {
   ssl_artifacts_status auto_detection_status;
-  AutoRLock lock(&lock_ssl_ctx);
+  const AutoRLock lock(&lock_ssl_ctx);
   auto_detection_status = auto_detect_ssl();
   if (auto_detection_status == SSL_ARTIFACTS_AUTO_DETECTED)
     LogErr(INFORMATION_LEVEL, ER_SSL_TRYING_DATADIR_DEFAULTS,
@@ -460,7 +469,7 @@ bool Ssl_init_callback_server_main::provision_certs() {
 }
 
 bool Ssl_init_callback_server_main::warn_self_signed_ca() {
-  AutoRLock lock(&lock_ssl_ctx);
+  const AutoRLock lock(&lock_ssl_ctx);
   return warn_self_signed_ca_certs(opt_ssl_ca, opt_ssl_capath);
 }
 
@@ -471,7 +480,7 @@ void Ssl_init_callback_server_admin::read_parameters(
     OptionalString *cert, OptionalString *cipher, OptionalString *ciphersuites,
     OptionalString *key, OptionalString *crl, OptionalString *crl_path,
     bool *session_cache_mode, long *session_cache_timeout) {
-  AutoRLock lock(&lock_admin_ssl_ctx);
+  const AutoRLock lock(&lock_admin_ssl_ctx);
   if (ca) ca->assign(opt_admin_ssl_ca);
   if (capath) capath->assign(opt_admin_ssl_capath);
   if (version) version->assign(opt_admin_tls_version);
@@ -485,14 +494,11 @@ void Ssl_init_callback_server_admin::read_parameters(
   if (session_cache_timeout)
     *session_cache_timeout = opt_ssl_session_cache_timeout;
 
-  if (opt_admin_ssl_ca || opt_admin_ssl_capath || opt_admin_ssl_cert ||
-      opt_admin_ssl_cipher || opt_admin_tls_ciphersuites || opt_admin_ssl_key ||
-      opt_admin_ssl_crl || opt_admin_ssl_crlpath)
-    g_admin_ssl_configured = true;
+  g_admin_ssl_configured = opt_admin_ssl_configured;
 }
 
 bool Ssl_init_callback_server_admin::warn_self_signed_ca() {
-  AutoRLock lock(&lock_ssl_ctx);
+  const AutoRLock lock(&lock_ssl_ctx);
   return warn_self_signed_ca_certs(opt_admin_ssl_ca, opt_admin_ssl_capath);
 }
 
