@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -25,22 +25,25 @@
 #define SYSTEM_VARIABLES_INCLUDED
 
 #include <stddef.h>
+#include <stdint.h>
 #include <sys/types.h>
 
-#include "m_ctype.h"
 #include "my_base.h"  // ha_rows
 #include "my_inttypes.h"
 #include "my_sqlcommand.h"
-#include "my_thread_local.h"     // my_thread_id
+#include "my_thread_local.h"  // my_thread_id
+#include "mysql/strings/m_ctype.h"
 #include "mysqld_error.h"
+#include "sql/mysqld_cs.h"
 #include "sql/rpl_gtid.h"        // Gitd_specification
 #include "sql/sql_plugin_ref.h"  // plugin_ref
 #include "sql_string.h"
+#include "str2int.h"
 
 class MY_LOCALE;
 class Time_zone;
 
-typedef ulonglong sql_mode_t;
+using sql_mode_t = uint64_t;
 struct LIST;
 
 // Values for binlog_format sysvar
@@ -82,13 +85,6 @@ enum enum_binlog_row_metadata {
   BINLOG_ROW_METADATA_FULL = 1
 };
 
-// Values for transaction_write_set_extraction sysvar
-enum enum_transaction_write_set_hashing_algorithm {
-  HASH_ALGORITHM_OFF = 0,
-  HASH_ALGORITHM_MURMUR32 = 1,
-  HASH_ALGORITHM_XXHASH64 = 2
-};
-
 // Values for session_track_gtids sysvar
 enum enum_session_track_gtids {
   SESSION_TRACK_GTIDS_OFF = 0,
@@ -134,19 +130,22 @@ enum enum_default_table_encryption {
 };
 
 /* Bits for different SQL modes modes (including ANSI mode) */
-#define MODE_REAL_AS_FLOAT 1
-#define MODE_PIPES_AS_CONCAT 2
-#define MODE_ANSI_QUOTES 4
-#define MODE_IGNORE_SPACE 8
-#define MODE_NOT_USED 16
-#define MODE_ONLY_FULL_GROUP_BY 32
-#define MODE_NO_UNSIGNED_SUBTRACTION 64
-#define MODE_NO_DIR_IN_CREATE 128
-#define MODE_ANSI 262144L
-#define MODE_NO_AUTO_VALUE_ON_ZERO (MODE_ANSI * 2)
-#define MODE_NO_BACKSLASH_ESCAPES (MODE_NO_AUTO_VALUE_ON_ZERO * 2)
-#define MODE_STRICT_TRANS_TABLES (MODE_NO_BACKSLASH_ESCAPES * 2)
-#define MODE_STRICT_ALL_TABLES (MODE_STRICT_TRANS_TABLES * 2)
+inline constexpr sql_mode_t MODE_REAL_AS_FLOAT = 1;
+inline constexpr sql_mode_t MODE_PIPES_AS_CONCAT = 2;
+inline constexpr sql_mode_t MODE_ANSI_QUOTES = 4;
+inline constexpr sql_mode_t MODE_IGNORE_SPACE = 8;
+inline constexpr sql_mode_t MODE_NOT_USED = 16;
+inline constexpr sql_mode_t MODE_ONLY_FULL_GROUP_BY = 32;
+inline constexpr sql_mode_t MODE_NO_UNSIGNED_SUBTRACTION = 64;
+inline constexpr sql_mode_t MODE_NO_DIR_IN_CREATE = 128;
+inline constexpr sql_mode_t MODE_ANSI = 0x40000;
+inline constexpr sql_mode_t MODE_NO_AUTO_VALUE_ON_ZERO = MODE_ANSI * 2;
+inline constexpr sql_mode_t MODE_NO_BACKSLASH_ESCAPES =
+    MODE_NO_AUTO_VALUE_ON_ZERO * 2;
+inline constexpr sql_mode_t MODE_STRICT_TRANS_TABLES =
+    MODE_NO_BACKSLASH_ESCAPES * 2;
+inline constexpr sql_mode_t MODE_STRICT_ALL_TABLES =
+    MODE_STRICT_TRANS_TABLES * 2;
 /*
  * NO_ZERO_DATE, NO_ZERO_IN_DATE and ERROR_FOR_DIVISION_BY_ZERO modes are
  * removed in 5.7 and their functionality is merged with STRICT MODE.
@@ -154,31 +153,34 @@ enum enum_default_table_encryption {
  * but they are not used. Setting these modes in 5.7 will give warning and
  * have no effect.
  */
-#define MODE_NO_ZERO_IN_DATE (MODE_STRICT_ALL_TABLES * 2)
-#define MODE_NO_ZERO_DATE (MODE_NO_ZERO_IN_DATE * 2)
-#define MODE_INVALID_DATES (MODE_NO_ZERO_DATE * 2)
-#define MODE_ERROR_FOR_DIVISION_BY_ZERO (MODE_INVALID_DATES * 2)
-#define MODE_TRADITIONAL (MODE_ERROR_FOR_DIVISION_BY_ZERO * 2)
-#define MODE_HIGH_NOT_PRECEDENCE (1ULL << 29)
-#define MODE_NO_ENGINE_SUBSTITUTION (MODE_HIGH_NOT_PRECEDENCE * 2)
-#define MODE_PAD_CHAR_TO_FULL_LENGTH (1ULL << 31)
+inline constexpr sql_mode_t MODE_NO_ZERO_IN_DATE = MODE_STRICT_ALL_TABLES * 2;
+inline constexpr sql_mode_t MODE_NO_ZERO_DATE = MODE_NO_ZERO_IN_DATE * 2;
+inline constexpr sql_mode_t MODE_INVALID_DATES = MODE_NO_ZERO_DATE * 2;
+inline constexpr sql_mode_t MODE_ERROR_FOR_DIVISION_BY_ZERO =
+    MODE_INVALID_DATES * 2;
+inline constexpr sql_mode_t MODE_TRADITIONAL =
+    MODE_ERROR_FOR_DIVISION_BY_ZERO * 2;
+inline constexpr sql_mode_t MODE_HIGH_NOT_PRECEDENCE = 1ULL << 29;
+inline constexpr sql_mode_t MODE_NO_ENGINE_SUBSTITUTION =
+    MODE_HIGH_NOT_PRECEDENCE * 2;
+inline constexpr sql_mode_t MODE_PAD_CHAR_TO_FULL_LENGTH = 1ULL << 31;
 /*
   If this mode is set the fractional seconds which cannot fit in given fsp will
   be truncated.
 */
-#define MODE_TIME_TRUNCATE_FRACTIONAL (1ULL << 32)
+inline constexpr sql_mode_t MODE_TIME_TRUNCATE_FRACTIONAL = 1ULL << 32;
 
-#define MODE_LAST (1ULL << 33)
+inline constexpr sql_mode_t MODE_LAST = 1ULL << 33;
 
-#define MODE_ALLOWED_MASK                                                      \
-  (MODE_REAL_AS_FLOAT | MODE_PIPES_AS_CONCAT | MODE_ANSI_QUOTES |              \
-   MODE_IGNORE_SPACE | MODE_NOT_USED | MODE_ONLY_FULL_GROUP_BY |               \
-   MODE_NO_UNSIGNED_SUBTRACTION | MODE_NO_DIR_IN_CREATE | MODE_ANSI |          \
-   MODE_NO_AUTO_VALUE_ON_ZERO | MODE_NO_BACKSLASH_ESCAPES |                    \
-   MODE_STRICT_TRANS_TABLES | MODE_STRICT_ALL_TABLES | MODE_NO_ZERO_IN_DATE |  \
-   MODE_NO_ZERO_DATE | MODE_INVALID_DATES | MODE_ERROR_FOR_DIVISION_BY_ZERO |  \
-   MODE_TRADITIONAL | MODE_HIGH_NOT_PRECEDENCE | MODE_NO_ENGINE_SUBSTITUTION | \
-   MODE_PAD_CHAR_TO_FULL_LENGTH | MODE_TIME_TRUNCATE_FRACTIONAL)
+inline constexpr sql_mode_t MODE_ALLOWED_MASK =
+    (MODE_REAL_AS_FLOAT | MODE_PIPES_AS_CONCAT | MODE_ANSI_QUOTES |
+     MODE_IGNORE_SPACE | MODE_NOT_USED | MODE_ONLY_FULL_GROUP_BY |
+     MODE_NO_UNSIGNED_SUBTRACTION | MODE_NO_DIR_IN_CREATE | MODE_ANSI |
+     MODE_NO_AUTO_VALUE_ON_ZERO | MODE_NO_BACKSLASH_ESCAPES |
+     MODE_STRICT_TRANS_TABLES | MODE_STRICT_ALL_TABLES | MODE_NO_ZERO_IN_DATE |
+     MODE_NO_ZERO_DATE | MODE_INVALID_DATES | MODE_ERROR_FOR_DIVISION_BY_ZERO |
+     MODE_TRADITIONAL | MODE_HIGH_NOT_PRECEDENCE | MODE_NO_ENGINE_SUBSTITUTION |
+     MODE_PAD_CHAR_TO_FULL_LENGTH | MODE_TIME_TRUNCATE_FRACTIONAL);
 
 /*
   We can safely ignore and reset these obsolete mode bits while replicating:
@@ -459,8 +461,6 @@ struct System_variables {
   uint binlog_trx_compression_level_zstd;
   ulonglong binlog_row_value_options;
   bool sql_log_bin;
-  // see enum_transaction_write_set_hashing_algorithm
-  ulong transaction_write_set_extraction;
   ulong completion_type;
   ulong transaction_isolation;
   ulong updatable_views_with_limit;
@@ -483,7 +483,6 @@ struct System_variables {
   */
   bool transaction_read_only;
   bool low_priority_updates;
-  bool new_mode;
   bool keep_files_on_create;
 
   bool old_alter_table;
@@ -570,13 +569,6 @@ struct System_variables {
   */
   bool show_create_table_verbosity;
 
-  /**
-    Compatibility option to mark the pre MySQL-5.6.4 temporals columns using
-    the old format using comments for SHOW CREATE TABLE and in I_S.COLUMNS
-    'COLUMN_TYPE' field.
-  */
-  bool show_old_temporals;
-
   bool ft_query_extra_word_chars;
 
   // Used for replication delay and lag monitoring
@@ -744,8 +736,31 @@ struct System_variables {
     @sa Sys_explain_format
    */
   Explain_format_type explain_format;
+
+  /**
+    Used to specify the JSON format version used by EXPLAIN FORMAT=JSON with
+    the old (non-hypergraph) join optimizer.
+    @sa Sys_explain_json_format_version
+   */
+  uint explain_json_format_version;
+
+  /// Max size of set operations hash buffer size.
+  ulonglong set_operations_buffer_size;
+#ifndef NDEBUG
+  char *debug_set_operations_secondary_overflow_at;
+#endif
+
+  /**
+    Restrict foreign keys on non-unique or partial keys.
+    @sa Sys_restrict_fk_on_non_standard_key
+  */
+  bool restrict_fk_on_non_standard_key;
 };
 
+static_assert(std::is_trivially_copyable<System_variables>::value);
+static_assert(std::is_trivial<System_variables>::value);
+static_assert(std::is_standard_layout<System_variables>::value);
+
 /**
   Per thread status variables.
   Must be long/ulong up to last_system_status_var so that
