@@ -1,4 +1,4 @@
-/* Copyright (c) 2012, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2012, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -41,6 +41,7 @@
 #include "mysql/components/services/bits/mysql_mutex_bits.h"
 #include "mysql/components/services/bits/psi_mutex_bits.h"
 #include "mysql/psi/mysql_mutex.h"
+#include "sql/aggregated_stats.h"
 #include "sql/handler.h"
 #include "sql/sql_base.h"
 #include "sql/sql_class.h"
@@ -53,6 +54,7 @@ class Table_cache_element;
 
 extern ulong table_cache_size_per_instance, table_cache_instances,
     table_cache_triggers, table_cache_triggers_per_instance;
+extern struct aggregated_stats global_aggregated_stats;
 
 /**
   Cache for open TABLE objects.
@@ -385,6 +387,8 @@ void Table_cache::free_unused_tables_if_necessary(THD *thd) {
       remove_table(table_to_free);
       intern_close_table(table_to_free);
       thd->status_var.table_open_cache_overflows++;
+      global_aggregated_stats.get_shard(thd->thread_id())
+          .table_open_cache_overflows++;
     }
     while (m_table_triggers_count > table_cache_triggers_per_instance &&
            !m_unused_triggers_lru.is_empty()) {
@@ -431,8 +435,8 @@ bool Table_cache::add_used_table(THD *thd, TABLE *table) {
       Allocate new Table_cache_element object and add it to the cache
       and array in TABLE_SHARE.
     */
-    std::string key(table->s->table_cache_key.str,
-                    table->s->table_cache_key.length);
+    const std::string key(table->s->table_cache_key.str,
+                          table->s->table_cache_key.length);
     assert(m_cache.count(key) == 0);
 
     el = new Table_cache_element(table->s);
@@ -483,8 +487,8 @@ void Table_cache::remove_table(TABLE *table) {
 
   if (el->used_tables.is_empty() && el->free_tables_full_triggers.is_empty() &&
       el->free_tables_slim.is_empty()) {
-    std::string key(table->s->table_cache_key.str,
-                    table->s->table_cache_key.length);
+    const std::string key(table->s->table_cache_key.str,
+                          table->s->table_cache_key.length);
     m_cache.erase(key);
     /*
       Remove reference to deleted cache element from array
@@ -527,7 +531,7 @@ TABLE *Table_cache::get_table(THD *thd, const char *key, size_t key_length,
 
   *share = nullptr;
 
-  std::string key_str(key, key_length);
+  const std::string key_str(key, key_length);
   const auto el_it = m_cache.find(key_str);
   if (el_it == m_cache.end()) return nullptr;
   Table_cache_element *el = el_it->second.get();
