@@ -1,4 +1,4 @@
-/* Copyright (c) 2017, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -132,7 +132,7 @@ class Table_function {
   /**
     Return table_map of tables used by the function
   */
-  virtual table_map used_tables() { return 0; }
+  virtual table_map used_tables() const { return 0; }
   /**
     Print table function
 
@@ -158,17 +158,6 @@ class Table_function {
 
   virtual bool walk(Item_processor processor, enum_walk walk, uchar *arg) = 0;
 
-  /**
-    Fix after tables have been moved from one query_block level to the parent
-    level, e.g by semijoin conversion.
-
-    @param parent_query_block  query_block that tables are moved to.
-    @param removed_query_block query_block that tables are moved away from,
-                          child of parent_query_block.
-  */
-  void fix_after_pullout(Query_block *parent_query_block,
-                         Query_block *removed_query_block);
-
  private:
   /**
     Get the list of fields to create the result table
@@ -184,8 +173,6 @@ class Table_function {
   virtual bool do_init_args() = 0;
   friend bool Table_ref::setup_table_function(THD *thd);
   virtual void do_cleanup() {}
-  virtual void do_fix_after_pullout(Query_block *parent_query_block,
-                                    Query_block *removed_query_block) = 0;
 };
 
 /****************************************************************************
@@ -381,7 +368,7 @@ class Table_function_json final : public Table_function {
   /**
     Return table_map of tables used by function's data source
   */
-  table_map used_tables() override;
+  table_map used_tables() const override;
 
   /**
     JSON_TABLE printout
@@ -444,8 +431,6 @@ class Table_function_json final : public Table_function {
   List<Create_field> *get_field_list() override;
   bool do_init_args() override;
   void do_cleanup() override;
-  void do_fix_after_pullout(Query_block *parent_query_block,
-                            Query_block *removed_query_block) override;
 };
 
 class Table_function_sequence final : public Table_function {
@@ -479,7 +464,7 @@ class Table_function_sequence final : public Table_function {
   /**
     Return table_map of tables used by function's data source
   */
-  virtual table_map used_tables() override;
+  virtual table_map used_tables() const override;
 
   /**
     PERCONA_SEQUENCE_TABLE printout
@@ -516,8 +501,6 @@ class Table_function_sequence final : public Table_function {
   virtual List<Create_field> *get_field_list() override;
   virtual bool do_init_args() override;
   virtual void do_cleanup() override;
-  void do_fix_after_pullout(Query_block *parent_query_block,
-                            Query_block *removed_query_block) override;
 
   ulonglong calculate_upper_bound() const;
 };
