@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2013, 2025, Oracle and/or its affiliates.
+   Copyright (c) 2013, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -25,6 +25,7 @@
 #ifndef TRIGGER_H_INCLUDED
 #define TRIGGER_H_INCLUDED
 
+#include <stdint.h>
 #include <string.h>
 
 #include "my_config.h"
@@ -45,7 +46,7 @@ class THD;
 class sp_head;
 struct MEM_ROOT;
 
-typedef ulonglong sql_mode_t;
+using sql_mode_t = uint64_t;
 
 /**
   This class represents a trigger object.
