@@ -1,4 +1,4 @@
-/* Copyright (c) 2013, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2013, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -29,11 +29,9 @@
 #include <unordered_map>
 #include <utility>
 
-#include "m_ctype.h"
-#include "m_string.h"
+#include "dig_vec.h"
 #include "map_helpers.h"
 #include "my_dbug.h"
-#include "my_loglevel.h"
 #include "my_macros.h"
 #include "my_psi_config.h"
 #include "my_sys.h"
@@ -41,10 +39,12 @@
 #include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/components/services/bits/psi_mutex_bits.h"
 #include "mysql/components/services/log_builtins.h"
+#include "mysql/my_loglevel.h"
 #include "mysql/plugin.h"  // MYSQL_XIDDATASIZE
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql/psi/mysql_transaction.h"
 #include "mysql/service_mysql_alloc.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"
 #include "scope_guard.h"  // Scope_guard
@@ -80,6 +80,7 @@
 #include "sql/xa/sql_xa_commit.h"      // Sql_cmd_xa_commit
 #include "sql/xa/transaction_cache.h"  // xa::Transaction_cache
 #include "sql_string.h"
+#include "string_with_len.h"
 #include "template_utils.h"
 #include "thr_mutex.h"
 
@@ -123,7 +124,8 @@ void xid_t::set(my_xid xid) {
   bqual_length = 0;
 }
 
-xid_t &xid_t::operator=(binary_log::XA_prepare_event::MY_XID const &rhs) {
+xid_t &xid_t::operator=(
+    mysql::binlog::event::XA_prepare_event::MY_XID const &rhs) {
   this->set_format_id(rhs.formatID);
   this->set_gtrid_length(rhs.gtrid_length);
   this->set_bqual_length(rhs.bqual_length);
@@ -499,7 +501,7 @@ void XID_STATE::store_xid_info(Protocol *protocol,
     xid_buf[0] = '0';
     xid_buf[1] = 'x';
 
-    size_t xid_str_len =
+    const size_t xid_str_len =
         bin_to_hex_str(xid_buf + 2, sizeof(xid_buf) - 2, m_xid.data,
                        m_xid.gtrid_length + m_xid.bqual_length) +
         2;
@@ -519,7 +521,7 @@ char *XID::xid_to_str(char *buf) const {
     /* is_next_dig is set if next character is a number */
     bool is_next_dig = false;
     if (i < XIDDATASIZE) {
-      char ch = data[i + 1];
+      const char ch = data[i + 1];
       is_next_dig = (ch >= '0' && ch <= '9');
     }
     if (i == gtrid_length) {
@@ -529,7 +531,7 @@ char *XID::xid_to_str(char *buf) const {
         *s++ = '\'';
       }
     }
-    uchar c = static_cast<uchar>(data[i]);
+    const uchar c = static_cast<uchar>(data[i]);
     if (c < 32 || c > 126) {
       *s++ = '\\';
       /*
@@ -537,9 +539,9 @@ char *XID::xid_to_str(char *buf) const {
         3 octal numbers to ensure that the next number is not seen
         as part of the octal number
       */
-      if (c > 077 || is_next_dig) *s++ = _dig_vec_lower[c >> 6];
-      if (c > 007 || is_next_dig) *s++ = _dig_vec_lower[(c >> 3) & 7];
-      *s++ = _dig_vec_lower[c & 7];
+      if (c > 077 || is_next_dig) *s++ = dig_vec_lower[c >> 6];
+      if (c > 007 || is_next_dig) *s++ = dig_vec_lower[(c >> 3) & 7];
+      *s++ = dig_vec_lower[c & 7];
     } else {
       if (c == '\'' || c == '\\') *s++ = '\\';
       *s++ = c;
