@@ -77,11 +77,9 @@ static handler *heap_create_handler(handlerton *hton, TABLE_SHARE *table, bool,
 *****************************************************************************/
 
 ha_heap::ha_heap(handlerton *hton, TABLE_SHARE *table_arg)
-    : handler(hton, table_arg),
-      file(nullptr),
-      records_changed(0),
-      key_stat_version(0),
-      single_instance(false) {}
+    : handler(hton, table_arg) {
+  ref_length = sizeof(HP_HEAP_POSITION);
+}
 
 /*
   Hash index statistics is updated (copied from HP_KEYDEF::hash_buckets to
@@ -132,7 +130,6 @@ int ha_heap::open(const char *name, int mode, uint test_if_locked,
     }
   }
 
-  ref_length = sizeof(HP_HEAP_POSITION);
   /*
     We cannot run update_key_stats() here because we do not have a
     lock on the table. The 'records' count might just be changed
