@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2025, Oracle and/or its affiliates.
+Copyright (c) 1995, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -124,8 +124,8 @@ ulint buf_read_page_low(dberr_t *err, bool sync, ulint type, ulint mode,
 
   IORequest request(type | IORequest::READ);
 
-  *err = _fil_io(request, sync, page_id, page_size, 0, page_size.physical(),
-                 dst, bpage, trx, should_buffer);
+  *err = fil_io(request, sync, page_id, page_size, 0, page_size.physical(), dst,
+                bpage, trx, should_buffer);
 
   if (sync) {
     thd_wait_end(nullptr);
@@ -576,8 +576,9 @@ ulint buf_read_ahead_linear(const page_id_t &page_id,
   os_aio_simulated_wake_handler_threads();
 
   if (count) {
-    DBUG_PRINT("ib_buf", ("linear read-ahead %lu pages, " UINT32PF ":" UINT32PF,
-                          count, page_id.space(), page_id.page_no()));
+    DBUG_PRINT("ib_buf",
+               ("linear read-ahead " ULINTPF " pages, " UINT32PF ":" UINT32PF,
+                count, page_id.space(), page_id.page_no()));
   }
 
   /* Read ahead is considered one I/O operation for the purpose of
@@ -657,8 +658,8 @@ void buf_read_ibuf_merge_pages(bool sync, const space_id_t *space_ids,
   }
 }
 
-void buf_read_recv_pages(bool sync, space_id_t space_id,
-                         const page_no_t *page_nos, ulint n_stored) {
+void buf_read_recv_pages(space_id_t space_id, const page_no_t *page_nos,
+                         ulint n_stored) {
   ulint count;
   fil_space_t *space = fil_space_get(space_id);
 
@@ -705,7 +706,8 @@ void buf_read_recv_pages(bool sync, space_id_t space_id,
     buf_pool = buf_pool_get(cur_page_id);
     os_rmb;
 
-    while (buf_pool->n_pend_reads >= recv_n_pool_free_frames / 2) {
+    while (buf_pool->n_pend_reads >=
+           recv_n_frames_for_pages_per_pool_instance / 2) {
       os_aio_simulated_wake_handler_threads();
       std::this_thread::sleep_for(std::chrono::milliseconds(10));
 
@@ -720,13 +722,8 @@ void buf_read_recv_pages(bool sync, space_id_t space_id,
 
     dberr_t err;
 
-    if ((i + 1 == n_stored) && sync) {
-      buf_read_page_low(&err, true, 0, BUF_READ_ANY_PAGE, cur_page_id,
-                        page_size, true, nullptr, false);
-    } else {
-      buf_read_page_low(&err, false, IORequest::DO_NOT_WAKE, BUF_READ_ANY_PAGE,
-                        cur_page_id, page_size, true, nullptr, false);
-    }
+    buf_read_page_low(&err, false, IORequest::DO_NOT_WAKE, BUF_READ_ANY_PAGE,
+                      cur_page_id, page_size, true, nullptr, false);
   }
 
   os_aio_simulated_wake_handler_threads();
