@@ -41,9 +41,9 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "sql/mysqld.h"
 #include "sql/sql_backup_lock.h"
 #include "sql/sql_class.h"
-#include "sql/sql_prepare.h"
 #include "sql/sql_table.h"
 #include "sql/sql_thd_internal_api.h"
+#include "sql/statement/ed_connection.h"
 #include "sql/strfunc.h"
 
 #include "dict0dd.h"
@@ -1430,7 +1430,7 @@ void clone_update_gtid_status(std::string &gtids) {
   std::string recovery_file(CLONE_INNODB_RECOVERY_FILE);
   if (!file_exists(recovery_file)) {
     ut_d(ut_error);
-    ut_o(return );
+    ut_o(return);
   }
   /* Open status file to append GTID. */
   std::ofstream status_file;
@@ -2336,9 +2336,9 @@ static int clone_drop_binary_logs(THD *thd) {
   skip_grants(thd, sctx);
   thd->set_security_context(&sctx);
 
-  /* 1. Attempt to stop slaves if any. */
+  /* 1. Attempt to stop replicas if any. */
   char sql_stmt[FN_LEN + FN_LEN + 64];
-  snprintf(sql_stmt, sizeof(sql_stmt), "STOP SLAVE");
+  snprintf(sql_stmt, sizeof(sql_stmt), "STOP REPLICA");
 
   channel_map.rdlock();
   auto is_slave = is_slave_configured();
@@ -2346,7 +2346,7 @@ static int clone_drop_binary_logs(THD *thd) {
 
   if (is_slave && clone_execute_query(thd, &sql_stmt[0], 1, false)) {
     err = ER_INTERNAL_ERROR;
-    my_error(err, MYF(0), "Clone failed to stop slave");
+    my_error(err, MYF(0), "Clone failed to stop replica");
   }
 
   if (err == 0) {
@@ -2354,7 +2354,7 @@ static int clone_drop_binary_logs(THD *thd) {
     thd->clear_error();
 
     /* 2. Clear all binary logs and GTID. */
-    snprintf(sql_stmt, sizeof(sql_stmt), "RESET MASTER");
+    snprintf(sql_stmt, sizeof(sql_stmt), "RESET BINARY LOGS AND GTIDS");
 
     if (clone_execute_query(thd, &sql_stmt[0], 1, false)) {
       err = ER_INTERNAL_ERROR;
@@ -2443,7 +2443,7 @@ static void clone_init_compression(THD *thd) {
 
   if (dc->fetch_global_component_ids<dd::Table>(&dd_table_ids)) {
     ut_d(ut_error);
-    ut_o(return );
+    ut_o(return);
   }
 
   for (auto dd_table_id : dd_table_ids) {
