@@ -179,14 +179,14 @@ struct File_cursor : public Load_cursor {
   @param[in] builder            The index build driver.
   @param[in] file               File to read from.
   @param[in] buffer_size        IO buffer size to use for reads.
-  @param[in] size               Size of the file in bytes.
+  @param[in] range              Offsets of the chunk to read from the file
   @param[in,out] stage          PFS observability. */
   File_cursor(Builder *builder, const Unique_os_file_descriptor &file,
-              size_t buffer_size, os_offset_t size, Alter_stage *stage,
+              size_t buffer_size, const Range &range, Alter_stage *stage,
               const Write_offsets &write_offsets) noexcept;
 
   /** Destructor. */
-  ~File_cursor() override = default;
+  ~File_cursor() override;
 
   /** Open the cursor.
   @return DB_SUCCESS or error code. */
@@ -225,6 +225,9 @@ struct File_cursor : public Load_cursor {
   /** PFS monitoring. */
   Alter_stage *m_stage{};
 
+  /** Number of rows that were fetched but not yet reported to the PFS. */
+  uint64_t m_processed_rows_to_report{};
+
   friend struct Merge_cursor;
 };
 
@@ -234,9 +237,11 @@ bool Load_cursor::duplicates_detected() const noexcept {
 
 dberr_t File_reader::get_tuple(Builder *builder, mem_heap_t *heap,
                                dtuple_t *&dtuple) noexcept {
-  dtuple = row_rec_to_index_entry_low(m_mrec, m_index, &m_offsets[0], heap);
+  dtuple =
+      row_rec_to_index_entry_low(m_mrec, m_index, &m_field_offsets[0], heap);
   if (!builder->is_fts_index()) {
-    return builder->dtuple_copy_blobs(dtuple, &m_offsets[0], m_mrec, heap);
+    return builder->dtuple_copy_blobs(dtuple, &m_field_offsets[0], m_mrec,
+                                      heap);
   } else {
     return DB_SUCCESS;
   }
@@ -244,16 +249,22 @@ dberr_t File_reader::get_tuple(Builder *builder, mem_heap_t *heap,
 
 File_cursor::File_cursor(Builder *builder,
                          const Unique_os_file_descriptor &file,
-                         size_t buffer_size, os_offset_t size,
+                         size_t buffer_size, const Range &range,
                          Alter_stage *stage,
                          const Write_offsets &write_offsets) noexcept
     : Load_cursor(builder, nullptr),
-      m_reader(file, builder->index(), buffer_size, size,
+      m_reader(file, builder->index(), buffer_size, range,
                builder->get_space_id(), write_offsets),
       m_stage(stage) {
   ut_a(m_reader.m_file.is_open());
 }
 
+File_cursor::~File_cursor() {
+  if (m_processed_rows_to_report > 0) {
+    m_stage->inc_progress_if_needed(m_processed_rows_to_report, true);
+  }
+}
+
 dberr_t File_cursor::open() noexcept {
   m_tuple_heap.create(2048, UT_LOCATION_HERE);
 
@@ -264,7 +275,8 @@ dberr_t File_cursor::fetch() noexcept {
   m_tuple_heap.clear();
 
   if (m_stage != nullptr) {
-    m_stage->inc(1);
+    m_processed_rows_to_report++;
+    m_stage->inc_progress_if_needed(m_processed_rows_to_report);
   }
 
   return m_builder->get_error();
@@ -292,7 +304,7 @@ dberr_t File_cursor::fetch(const mrec_t *&mrec, ulint *&offsets) noexcept {
   }
 
   mrec = m_reader.m_mrec;
-  offsets = &m_reader.m_offsets[0];
+  offsets = &m_reader.m_field_offsets[0];
 
   return DB_SUCCESS;
 }
@@ -300,7 +312,7 @@ dberr_t File_cursor::fetch(const mrec_t *&mrec, ulint *&offsets) noexcept {
 dberr_t File_cursor::next() noexcept {
   auto err = m_reader.next();
 
-  if (unlikely(err != DB_END_OF_INDEX)) {
+  if (likely(err != DB_END_OF_INDEX)) {
     m_err = err;
   }
 
@@ -338,25 +350,28 @@ bool Merge_cursor::Compare::operator()(const File_cursor *lhs,
 
   ut_a(l.m_index == r.m_index);
 
-  auto cmp = cmp_rec_rec_simple(r.m_mrec, l.m_mrec, &r.m_offsets[0],
-                                &l.m_offsets[0], r.m_index,
+  auto cmp = cmp_rec_rec_simple(r.m_mrec, l.m_mrec, &r.m_field_offsets[0],
+                                &l.m_field_offsets[0], r.m_index,
                                 m_dup != nullptr ? m_dup->m_table : nullptr);
 
   /* Check for duplicates. */
   if (unlikely(cmp == 0 && m_dup != nullptr)) {
-    m_dup->report(l.m_mrec, &l.m_offsets[0]);
+    m_dup->report(l.m_mrec, &l.m_field_offsets[0]);
   }
 
   return cmp < 0;
 }
 
-dberr_t Merge_cursor::add_file(const ddl::file_t &file,
-                               size_t buffer_size) noexcept {
+dberr_t Merge_cursor::add_file(const ddl::file_t &file, size_t buffer_size,
+                               const Range &range) noexcept {
   ut_a(file.m_file.is_open());
-
+  /* Keep the buffer size as much required to avoid the overlapping reads from
+  the subsequent ranges. In this iteration, buffer size would remain same for
+  subsequent reads */
+  buffer_size = std::min(size_t(range.second - range.first), buffer_size);
   auto cursor = ut::new_withkey<File_cursor>(
       ut::make_psi_memory_key(mem_key_ddl), m_builder, file.m_file, buffer_size,
-      file.m_size, m_stage, file.m_write_offsets);
+      range, m_stage, file.m_write_offsets);
 
   if (cursor == nullptr) {
     m_err = DB_OUT_OF_MEMORY;
@@ -368,16 +383,9 @@ dberr_t Merge_cursor::add_file(const ddl::file_t &file,
   return DB_SUCCESS;
 }
 
-dberr_t Merge_cursor::add_file(const ddl::file_t &file, size_t buffer_size,
-                               os_offset_t offset) noexcept {
-  auto err = add_file(file, buffer_size);
-
-  if (err != DB_SUCCESS) {
-    return err;
-  } else {
-    m_cursors.back()->m_reader.set_offset(offset);
-    return DB_SUCCESS;
-  }
+dberr_t Merge_cursor::add_file(const ddl::file_t &file,
+                               size_t buffer_size) noexcept {
+  return add_file(file, buffer_size, Range{0, file.m_size});
 }
 
 void Merge_cursor::clear_eof() noexcept {
@@ -389,7 +397,7 @@ void Merge_cursor::clear_eof() noexcept {
 
   for (auto cursor : m_cursors) {
     ut_a(cursor->m_err == DB_END_OF_INDEX);
-    if (!cursor->m_reader.eof()) {
+    if (!cursor->m_reader.end_of_range()) {
       cursor->m_err = DB_SUCCESS;
       m_pq.push(cursor);
     }
@@ -884,8 +892,6 @@ dberr_t Builder::copy_fts_column(Copy_ctx &ctx, dfield_t *field) noexcept {
     doc_id = fts.m_doc_id->current();
   }
 
-  ut_a(doc_id <= 4294967295u);
-
   if (unlikely(!dfield_is_null(field))) {
     auto ptr = ut::malloc_withkey(UT_NEW_THIS_FILE_PSI_KEY,
                                   sizeof(FTS::Doc_item) + field->len);
