@@ -543,7 +543,7 @@ dberr_t Row::build(ddl::Context &ctx, dict_index_t *index, mem_heap_t *heap,
   }
 
   if (ctx.m_add_autoinc != ULINT_UNDEFINED) {
-    auto err = ctx.handle_autoinc(m_ptr);
+    auto err = ctx.handle_autoinc(m_ptr, heap);
 
     if (err != DB_SUCCESS) {
       return err;
