@@ -35,6 +35,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
 
 #include "ha_innodb.h"
 #include "my_compiler.h"
+#include "my_inttypes.h"
 #include "partitioning/partition_handler.h"
 #include "row0mysql.h"
 #include "ut0bitset.h"
@@ -55,7 +56,7 @@ const handler::Table_flags HA_INNOPART_DISABLED_TABLE_FLAGS =
     (HA_CAN_FULLTEXT | HA_CAN_FULLTEXT_EXT | HA_CAN_GEOMETRY |
      HA_DUPLICATE_POS | HA_READ_BEFORE_WRITE_REMOVAL);
 
-typedef Bitset Sql_stat_start_parts;
+typedef Bitset<> Sql_stat_start_parts;
 
 /** InnoDB partition specific Handler_share. */
 class Ha_innopart_share : public Partition_share {
@@ -593,10 +594,15 @@ class ha_innopart : public ha_innobase,
                                       if we exhaust the max cap of number of
                                       parallel read threads that can be
                                       spawned at a time
+  @param[in]    max_desired_threads   Maximum number of desired read threads;
+                                      passing 0 has no effect, it is ignored;
+                                      upper-limited by the current value of
+                                      innodb_parallel_read_threads.
   @return error code
   @return 0 on success */
   int parallel_scan_init(void *&scan_ctx, size_t *num_threads,
-                         bool use_reserved_threads) override;
+                         bool use_reserved_threads,
+                         size_t max_desired_threads) override;
 
   using Reader = Parallel_reader_adapter;
 
