@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1994, 2025, Oracle and/or its affiliates.
+Copyright (c) 1994, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -157,7 +157,7 @@ If len>0, tests the first len bytes of the content for equality.
 @return true if both fields are NULL or if they are equal */
 [[nodiscard]] inline bool dfield_datas_are_binary_equal(const dfield_t *field1,
                                                         const dfield_t *field2,
-                                                        ulint len = 0);
+                                                        ulint len);
 /** Tests if dfield data length and content is equal to the given.
 @param[in] field Field
 @param[in] len   Data length or UNIV_SQL_NULL
@@ -518,7 +518,7 @@ struct multi_value_data {
            sizeof(*conv_buf) * multi_value->num_v);
     if (multi_value->bitset != nullptr) {
       ut_ad(bitset != nullptr);
-      ut_ad(multi_value->bitset->size_bytes() <= bitmap_bytes);
+      ut_ad_le(multi_value->bitset->size_bytes(), bitmap_bytes);
       /* adjust the size to match that of multi_value, but keep pointing to the
       old buffer */
       *bitset = Bitset<>(bitset->data(), multi_value->bitset->size_bytes());
@@ -784,6 +784,12 @@ struct dtuple_t {
     return n_ext;
   }
 
+  /** Set the flag REC_INFO_MIN_REC_FLAG in the info bits. */
+  void set_min_rec_flag();
+
+  /** Unset the flag REC_INFO_MIN_REC_FLAG in the info bits. */
+  void unset_min_rec_flag();
+
   /** Does tuple has externally stored fields.
   @retval true if there is externally stored fields. */
   inline bool has_ext() const {
@@ -794,6 +800,8 @@ struct dtuple_t {
     }
     return false;
   }
+
+  dtuple_t *deep_copy(mem_heap_t *heap) const;
 };
 
 /** A slot for a field in a big rec vector */
