@@ -46,7 +46,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "trx0types.h"
 #include "ut0new.h"
 
-#include "m_ctype.h"
+#include "mysql/strings/m_ctype.h"
 #include "sql/dd/object_id.h"
 
 #include <atomic>
@@ -141,13 +141,16 @@ enum class Fil_state {
   /** Space ID matches but the paths don't match. */
   MOVED,
 
-  /** Space ID and paths match but dd_table data dir flag doesn't
-  exist or tablespace is created outside default data dir */
-  MOVED_PREV_OR_HAS_DATADIR,
+  /** Space ID and paths match but dd_table data dir flag is false despite the
+  file being outside default data dir */
+  MOVED_PREV,
 
   /** Tablespace and/or filename was renamed. The DDL log will handle
   this case. */
-  RENAMED
+  RENAMED,
+
+  /** In case of error during comparison. */
+  COMPARE_ERROR
 };
 
 struct fil_space_t;
@@ -186,6 +189,8 @@ struct fil_node_t {
   /** whether the file actually is a raw device or disk partition */
   bool is_raw_disk;
 
+  bool is_offset_valid(os_offset_t byte_offset) const;
+
   /** size of the file in database pages (0 if not known yet);
   the possible last incomplete megabyte may be ignored
   if space->id == 0 */
@@ -424,7 +429,33 @@ struct fil_space_t {
   /** true if the tablespace is marked for deletion. */
   std::atomic_bool m_deleted{};
 
+  /** true if bulk operation is in progress. */
+  std::atomic_bool m_is_bulk{false};
+
+  /** true if bulk operation is in progress. */
+  std::atomic<uint64_t> m_bulk_extend_size;
+
  public:
+  /** Begin bulk operation on the space.
+  @param[in] extend_size space extension size for bulk operation */
+  void begin_bulk_operation(uint64_t extend_size) {
+    m_is_bulk.store(true);
+    m_bulk_extend_size.store(extend_size);
+  }
+
+  /** End bulk operation on the space. */
+  void end_bulk_operation() { m_is_bulk.store(false); }
+
+  /** @return true, if bulk operation in progress. */
+  bool is_bulk_operation_in_progress() const { return m_is_bulk.load(); }
+
+  /** @return automatic extension size for space. */
+  uint64_t get_auto_extend_size();
+
+  /** @return true if added space should be initialized while extending space.
+   */
+  bool initialize_while_extending();
+
   /** true if we want to rename the .ibd file of tablespace and
   want to temporarily prevent other threads from opening the file that is being
   renamed.  */
@@ -1307,12 +1338,20 @@ inline bool fil_page_type_is_index(page_type_t page_type) {
          page_type == FIL_PAGE_RTREE;
 }
 
-page_type_t fil_page_get_type(const byte *page);
+/** Get the file page type.
+@param[in]    page    File page
+@return page type */
+inline page_type_t fil_page_get_type(const byte *page) {
+  return (static_cast<page_type_t>(mach_read_from_2(page + FIL_PAGE_TYPE)));
+}
 
 /** Check whether the page is index page (either regular Btree index or Rtree
-index */
+index.
+@param[in]  page  page frame whose page type is to be checked. */
 inline bool fil_page_index_page_check(const byte *page) {
-  return fil_page_type_is_index(fil_page_get_type(page));
+  const page_type_t type = fil_page_get_type(page);
+  const bool is_idx = fil_page_type_is_index(type);
+  return is_idx;
 }
 
 /** @} */
@@ -1747,15 +1786,11 @@ number should be zero.
 @return error code
 @retval DB_SUCCESS on success
 @retval DB_TABLESPACE_DELETED if the tablespace does not exist */
-[[nodiscard]] dberr_t _fil_io(const IORequest &type, bool sync,
-                              const page_id_t &page_id,
-                              const page_size_t &page_size, ulint byte_offset,
-                              ulint len, void *buf, void *message, trx_t *trx,
-                              bool should_buffer);
-
-#define fil_io(type, sync, page_id, page_size, byte_offset, len, buf, message) \
-  _fil_io(type, sync, page_id, page_size, byte_offset, len, buf, message,      \
-          NULL, false)
+[[nodiscard]] dberr_t fil_io(const IORequest &type, bool sync,
+                             const page_id_t &page_id,
+                             const page_size_t &page_size, ulint byte_offset,
+                             ulint len, void *buf, void *message, trx_t *trx,
+                             bool should_buffer);
 
 /** Waits for an AIO operation to complete. This function is used to write the
 handler for completed requests. The aio array of pending requests is divided
@@ -1811,13 +1846,35 @@ Any other pages were written with uninitialized bytes in FIL_PAGE_TYPE.
 void fil_page_reset_type(const page_id_t &page_id, byte *page, ulint type,
                          mtr_t *mtr);
 
-/** Get the file page type.
-@param[in]      page            File page
-@return page type */
-inline page_type_t fil_page_get_type(const byte *page) {
-  return (static_cast<page_type_t>(mach_read_from_2(page + FIL_PAGE_TYPE)));
+/** Check (and if needed, reset) the page type.
+Data files created before MySQL 5.1 may contain
+garbage in the FIL_PAGE_TYPE field.
+In MySQL 3.23.53, only undo log pages and index pages were tagged.
+Any other pages were written with uninitialized bytes in FIL_PAGE_TYPE.
+@param[in]      page_id         Page number
+@param[in,out]  page            Page with possibly invalid FIL_PAGE_TYPE
+@param[in]      type            Expected page type
+@param[in,out]  mtr             Mini-transaction */
+inline void fil_page_check_type(const page_id_t &page_id, byte *page,
+                                ulint type, mtr_t *mtr) {
+  ulint page_type = fil_page_get_type(page);
+
+  if (page_type != type) {
+    fil_page_reset_type(page_id, page, type, mtr);
+  }
 }
 
+/** Check (and if needed, reset) the page type.
+Data files created before MySQL 5.1 may contain
+garbage in the FIL_PAGE_TYPE field.
+In MySQL 3.23.53, only undo log pages and index pages were tagged.
+Any other pages were written with uninitialized bytes in FIL_PAGE_TYPE.
+@param[in,out]  block           Block with possibly invalid FIL_PAGE_TYPE
+@param[in]      type            Expected page type
+@param[in,out]  mtr             Mini-transaction */
+#define fil_block_check_type(block, type, mtr) \
+  fil_page_check_type(block->page.id, block->frame, type, mtr)
+
 #ifdef UNIV_DEBUG
 /** Increase redo skipped count for a tablespace.
 @param[in]      space_id        Tablespace ID */
@@ -2016,16 +2073,16 @@ inline void fil_space_open_if_needed(fil_space_t *space) {
 /** Enable encryption of temporary tablespace
 @param[in,out]	space	tablespace object
 @return DB_SUCCESS on success, DB_ERROR on failure */
-MY_NODISCARD
+[[nodiscard]]
 dberr_t fil_temp_update_encryption(fil_space_t *space);
 
-#if !defined(NO_FALLOCATE) && defined(UNIV_LINUX)
+#ifdef UNIV_LINUX
 /**
 Try and enable FusionIO atomic writes.
 @param[in] file         OS file handle
 @return true if successful */
 [[nodiscard]] bool fil_fusionio_enable_atomic_write(pfs_os_file_t file);
-#endif /* !NO_FALLOCATE && UNIV_LINUX */
+#endif /* UNIV_LINUX */
 
 /** Note that the file system where the file resides doesn't support PUNCH HOLE.
 Called from AIO handlers when IO returns DB_IO_NO_PUNCH_HOLE
@@ -2049,10 +2106,11 @@ inline fil_space_t *fil_space_get_sys_space() {
 @param[in]      parse_only      Don't apply, parse only
 @return pointer to next redo log record
 @retval nullptr if this log record was truncated */
-[[nodiscard]] byte *fil_tablespace_redo_create(byte *ptr, const byte *end,
-                                               const page_id_t &page_id,
-                                               ulint parsed_bytes,
-                                               bool parse_only);
+[[nodiscard]] const byte *fil_tablespace_redo_create(const byte *ptr,
+                                                     const byte *end,
+                                                     const page_id_t &page_id,
+                                                     ulint parsed_bytes,
+                                                     bool parse_only);
 
 /** Redo a tablespace delete.
 @param[in]      ptr             redo log record
@@ -2062,10 +2120,11 @@ inline fil_space_t *fil_space_get_sys_space() {
 @param[in]      parse_only      Don't apply, parse only
 @return pointer to next redo log record
 @retval nullptr if this log record was truncated */
-[[nodiscard]] byte *fil_tablespace_redo_delete(byte *ptr, const byte *end,
-                                               const page_id_t &page_id,
-                                               ulint parsed_bytes,
-                                               bool parse_only);
+[[nodiscard]] const byte *fil_tablespace_redo_delete(const byte *ptr,
+                                                     const byte *end,
+                                                     const page_id_t &page_id,
+                                                     ulint parsed_bytes,
+                                                     bool parse_only);
 
 /** Redo a tablespace rename.
 This function doesn't do anything, simply parses the redo log record.
@@ -2076,10 +2135,11 @@ This function doesn't do anything, simply parses the redo log record.
 @param[in]      parse_only      Don't apply, parse only
 @return pointer to next redo log record
 @retval nullptr if this log record was truncated */
-[[nodiscard]] byte *fil_tablespace_redo_rename(byte *ptr, const byte *end,
-                                               const page_id_t &page_id,
-                                               ulint parsed_bytes,
-                                               bool parse_only);
+[[nodiscard]] const byte *fil_tablespace_redo_rename(const byte *ptr,
+                                                     const byte *end,
+                                                     const page_id_t &page_id,
+                                                     ulint parsed_bytes,
+                                                     bool parse_only);
 
 /** Redo a tablespace extend
 @param[in]      ptr             redo log record
@@ -2089,10 +2149,11 @@ This function doesn't do anything, simply parses the redo log record.
 @param[in]      parse_only      Don't apply the log if true
 @return pointer to next redo log record
 @retval nullptr if this log record was truncated */
-[[nodiscard]] byte *fil_tablespace_redo_extend(byte *ptr, const byte *end,
-                                               const page_id_t &page_id,
-                                               ulint parsed_bytes,
-                                               bool parse_only);
+[[nodiscard]] const byte *fil_tablespace_redo_extend(const byte *ptr,
+                                                     const byte *end,
+                                                     const page_id_t &page_id,
+                                                     ulint parsed_bytes,
+                                                     bool parse_only);
 
 /** Parse and process an encryption redo record.
 @param[in]      ptr             redo log record
@@ -2100,9 +2161,10 @@ This function doesn't do anything, simply parses the redo log record.
 @param[in]      space_id        the tablespace ID
 @param[in]      lsn             lsn for REDO record
 @return log record end, nullptr if not a complete record */
-[[nodiscard]] byte *fil_tablespace_redo_encryption(byte *ptr, const byte *end,
-                                                   space_id_t space_id,
-                                                   lsn_t lsn);
+[[nodiscard]] const byte *fil_tablespace_redo_encryption(const byte *ptr,
+                                                         const byte *end,
+                                                         space_id_t space_id,
+                                                         lsn_t lsn);
 
 /** Read the tablespace id to path mapping from the file
 @param[in]      recovery        true if called from crash recovery */
@@ -2126,18 +2188,20 @@ bool fil_update_partition_name(space_id_t space_id, uint32_t fsp_flags,
                                std::string &dd_path);
 
 /** Add tablespace to the set of tablespaces to be updated in DD.
-@param[in]    dd_object_id                   Server DD tablespace ID
-@param[in]    space_id                       InnoDB tablespace ID
-@param[in]    space_name                     Tablespace name
-@param[in]    old_path                       Old Path in the data dictionary
-@param[in]    new_path                       New path to be update in dictionary
-@param[in]    moved_prev_or_has_datadir      The move has happened before
-                                             8.0.38/8.4.1/9.0.0 or table is
-                                             created with data dir clause.*/
+@param[in]      dd_object_id    Server DD tablespace ID
+@param[in]      space_id        Innodb tablespace ID
+@param[in]      space_name      New tablespace name
+@param[in]      old_path        Old Path in the data dictionary
+@param[in]      new_path        New path to be update in dictionary
+@param[in]      dd_flag_missing This tablespace is outside default data
+                                directory, yet it is missing
+                                DD_TABLE_DATA_DIRECTORY flag. This could
+                                happen in versions earlier than
+                                8.0.38/8.4.1/9.0.0 */
 void fil_add_moved_space(dd::Object_id dd_object_id, space_id_t space_id,
                          const char *space_name, const std::string &old_path,
-                         const std::string &new_path,
-                         bool moved_prev_or_has_datadir);
+                         const std::string &new_path, bool dd_flag_missing);
+
 /** Lookup the tablespace ID and return the path to the file. The filename
 is ignored when testing for equality. Only the path up to the file name is
 considered for matching: e.g. ./test/a.ibd == ./test/b.ibd.
@@ -2280,4 +2344,16 @@ size_t fil_count_undo_deleted(space_id_t undo_num);
 @param[in]  type  the page type to be checked for validity.
 @return true if it is valid page type, false otherwise. */
 [[nodiscard]] bool fil_is_page_type_valid(page_type_t type) noexcept;
+
+dberr_t fil_prepare_file_for_io(space_id_t space_id, page_no_t &page_no,
+                                fil_node_t **node_out);
+void fil_complete_write(space_id_t space_id, fil_node_t *node);
+
+inline bool fil_node_t::is_offset_valid(os_offset_t byte_offset) const {
+  const page_size_t page_size(space->flags);
+  const os_offset_t max_offset = size * page_size.physical();
+  ut_ad(byte_offset < max_offset);
+  return byte_offset < max_offset;
+}
+
 #endif /* fil0fil_h */
