@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2006, 2025, Oracle and/or its affiliates.
+Copyright (c) 2006, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -51,6 +51,11 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "ut0vec.h"
 #include "ut0wqueue.h"
 
+struct CHARSET_INFO;
+
+/** "NULL" value of a document id. */
+#define FTS_NULL_DOC_ID 0
+
 /** FTS hidden column that is used to map to and from the row */
 #define FTS_DOC_ID_COL_NAME "FTS_DOC_ID"
 
@@ -99,9 +104,6 @@ those defined in mysql file ft_global.h */
 #define FTS_INDEX_TABLE_IND_NAME "FTS_INDEX_TABLE_IND"
 #define FTS_COMMON_TABLE_IND_NAME "FTS_COMMON_TABLE_IND"
 
-/** "NULL" value of a document id. */
-constexpr doc_id_t FTS_NULL_DOC_ID = 0;
-
 /** The number of FTS index partitions for a fulltext index. */
 constexpr size_t FTS_NUM_AUX_INDEX = 6;
 
