@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1997, 2025, Oracle and/or its affiliates.
+Copyright (c) 1997, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -297,8 +297,8 @@ ulint ibuf_merge_space(space_id_t space); /*!< in: space id */
 @param[in] block Block or nullptr.
 @param[in] mtr MTR or nullptr.
 @return end of log record or NULL */
-byte *ibuf_parse_bitmap_init(byte *ptr, byte *end_ptr, buf_block_t *block,
-                             mtr_t *mtr);
+const byte *ibuf_parse_bitmap_init(const byte *ptr, const byte *end_ptr,
+                                   buf_block_t *block, mtr_t *mtr);
 #ifndef UNIV_HOTBACKUP
 #ifdef UNIV_IBUF_COUNT_DEBUG
 
