@@ -1,7 +1,7 @@
 /***********************************************************************
 
-Copyright (c) 1995, 2025, Oracle and/or its affiliates.
-Copyright (c) 2009, Percona Inc.
+Copyright (c) 1995, 2024, Oracle and/or its affiliates.
+Copyright (c) 2009, 2017, Percona Inc.
 
 Portions of this file contain modifications contributed and copyrighted
 by Percona Inc.. Those modifications are
@@ -75,6 +75,9 @@ struct trx_t;
 
 extern bool os_has_said_disk_full;
 
+/** Number of retries for partial I/O's */
+constexpr size_t NUM_RETRIES_ON_PARTIAL_IO = 10;
+
 /** Number of pending read operations */
 extern std::atomic<ulint> os_n_pending_reads;
 /** Number of pending write operations */
@@ -259,7 +262,6 @@ static const ulint OS_BUFFERED_FILE = 103;
 static const ulint OS_CLONE_DATA_FILE = 104;
 static const ulint OS_CLONE_LOG_FILE = 105;
 static const ulint OS_DBLWR_FILE = 106;
-static const ulint OS_DATA_FILE_FOR_SPACE_ID_READ = 107;
 /** @} */
 
 /** Error codes from os_file_get_last_error @{ */
@@ -568,50 +570,15 @@ class IORequest {
 #endif /* HAVE_FALLOC_PUNCH_HOLE_AND_KEEP_SIZE || _WIN32 */
   }
 
+  static std::string type_str(const ulint type);
+
   /** @return string representation. */
   std::string to_string() const {
     std::ostringstream os;
-
     os << "bs: " << m_block_size << " flags:";
-
-    if (m_type & READ) {
-      os << " READ";
-    } else if (m_type & WRITE) {
-      os << " WRITE";
-    } else if (m_type & DBLWR) {
-      os << " DBLWR";
-    }
-
-    /** Enumerations below can be ORed to READ/WRITE above*/
-
-    /** Data file */
-    if (m_type & DATA_FILE) {
-      os << " | DATA_FILE";
-    }
-
-    if (m_type & LOG) {
-      os << " | LOG";
-    }
-
-    if (m_type & DISABLE_PARTIAL_IO_WARNINGS) {
-      os << " | DISABLE_PARTIAL_IO_WARNINGS";
-    }
-
-    if (m_type & DO_NOT_WAKE) {
-      os << " | IGNORE_MISSING";
-    }
-
-    if (m_type & PUNCH_HOLE) {
-      os << " | PUNCH_HOLE";
-    }
-
-    if (m_type & NO_COMPRESSION) {
-      os << " | NO_COMPRESSION";
-    }
-
+    os << type_str(m_type);
     os << ", comp: " << m_compression.to_string();
     os << ", enc: " << m_encryption.to_string(m_encryption.get_type());
-
     return (os.str());
   }
 
@@ -787,6 +754,12 @@ for each entry.
 bool os_file_scan_directory(const char *path, os_dir_cbk_t scan_cbk,
                             bool is_drop);
 
+/** Clang on Windows warns about umask not found. */
+MY_COMPILER_DIAGNOSTIC_PUSH()
+#ifdef _WIN32
+MY_COMPILER_CLANG_DIAGNOSTIC_IGNORE("-Wdocumentation")
+#endif
+
 /** NOTE! Use the corresponding macro
 os_file_create_simple_no_error_handling(), not directly this function!
 A simple function to open or create a file.
@@ -809,17 +782,27 @@ null-terminated string
     mode_t umask,
 #endif
     bool *success);
+MY_COMPILER_DIAGNOSTIC_POP()
 
 /** Tries to disable OS caching on an opened file descriptor.
 @param[in]      fd              file descriptor to alter
 @param[in]      file_name       file name, used in the diagnostic message
 @param[in]      operation_name  "open" or "create"; used in the diagnostic
                                 message
-@param[in]      on_error_silent if true then don't print any message to the log
-*/
-void os_file_set_nocache(int fd, const char *file_name,
-                         const char *operation_name,
-                         bool on_error_silent = false);
+@return true if operation is success and false */
+bool os_file_set_nocache(int fd, const char *file_name,
+                         const char *operation_name);
+
+/** Tries to disable OS caching on an opened file file.
+@param[in]	file		file to alter
+@param[in]	file_name	file name, used in the diagnostic message
+@param[in]	operation_name	"open" or "create"; used in the diagnostic
+message
+@return true if operation is success and false */
+static inline bool os_file_set_nocache(pfs_os_file_t file,
+                                       const char *file_name,
+                                       const char *operation_name);
+
 /** NOTE! Use the corresponding macro os_file_create(), not directly
 this function!
 Opens an existing file or creates a new.
@@ -1064,6 +1047,12 @@ The wrapper functions have the prefix of "innodb_". */
 #define os_file_set_eof_at_pfs(file, new_len) \
   pfs_os_file_set_eof_at_func(file, new_len, UT_LOCATION_HERE)
 
+/** Clang on Windows warns about umask not found. */
+MY_COMPILER_DIAGNOSTIC_PUSH()
+#ifdef _WIN32
+MY_COMPILER_CLANG_DIAGNOSTIC_IGNORE("-Wdocumentation")
+#endif
+
 /** NOTE! Please use the corresponding macro
 os_file_create_simple_no_error_handling(), not directly this function!
 A performance schema instrumented wrapper function for
@@ -1092,6 +1081,7 @@ pfs_os_file_create_simple_no_error_handling_func(
     mode_t umask,
 #endif
     bool *success, ut::Location src_location);
+MY_COMPILER_DIAGNOSTIC_POP()
 
 /** NOTE! Please use the corresponding macro os_file_create(), not directly
 this function!
@@ -1357,7 +1347,6 @@ static inline bool pfs_os_file_set_eof_at_func(pfs_os_file_t file,
 
 /* If UNIV_PFS_IO is not defined, these I/O APIs point
 to original un-instrumented file I/O APIs */
-
 #define os_file_create(key, name, create, purpose, read_only, success) \
   os_file_create_func(name, create, purpose, read_only, success)
 
@@ -1514,7 +1503,7 @@ to original un-instrumented file I/O APIs */
 
 /** Announces an intention to access file data in a specific pattern in the
 future.
-@param[in, own]	file	handle to a file
+@param[in,out]	file	handle to a file
 @param[in]	offset	file region offset
 @param[in]	len	file region length
 @param[in]	advice	advice for access pattern
@@ -2050,6 +2039,70 @@ dberr_t os_file_write_retry(IORequest &type, const char *name,
                             pfs_os_file_t file, const void *buf,
                             os_offset_t offset, ulint n);
 
+/** Helper class for doing synchronous file IO. Currently, the objective
+is to hide the OS specific code, so that the higher level functions aren't
+peppered with "#ifdef". Makes the code flow difficult to follow.  */
+class SyncFileIO {
+ public:
+  /** Constructor
+  @param[in]    fh      File handle
+  @param[in,out]        buf     Buffer to read/write
+  @param[in]    n       Number of bytes to read/write
+  @param[in]    offset  Offset where to read or write */
+  SyncFileIO(os_file_t fh, void *buf, ulint n, os_offset_t offset)
+      : m_fh(fh),
+        m_buf(buf),
+        m_n(static_cast<ssize_t>(n)),
+        m_offset(offset),
+        m_orig_bytes(n) {
+    ut_ad(m_n > 0);
+  }
+
+  /** Destructor */
+  ~SyncFileIO() = default;
+
+  /** Do the read/write
+  @param[in]    request The IO context and type
+  @return the number of bytes read/written or negative value on error */
+  ssize_t execute(const IORequest &request);
+
+  /** Do the read/write with retry.
+  @param[in] request The IO context and type
+  @param[in] max_retries  the maximum number of retries on partial i/o.
+  @return DB_SUCCESS on success, an error code on failure. */
+  dberr_t execute_with_retry(
+      const IORequest &request,
+      const size_t max_retries = NUM_RETRIES_ON_PARTIAL_IO);
+
+  /** Move the read/write offset up to where the partial IO succeeded.
+  @param[in]    n_bytes The number of bytes to advance */
+  void advance(ssize_t n_bytes) {
+    m_offset += n_bytes;
+
+    ut_ad(m_n >= n_bytes);
+
+    m_n -= n_bytes;
+
+    m_buf = reinterpret_cast<uchar *>(m_buf) + n_bytes;
+  }
+
+ private:
+  /** Open file handle */
+  os_file_t m_fh;
+
+  /** Buffer to read/write */
+  void *m_buf;
+
+  /** Number of bytes to read/write */
+  ssize_t m_n;
+
+  /** Offset from where to read/write */
+  os_offset_t m_offset;
+
+  /** The total number of bytes to be read/written. */
+  const size_t m_orig_bytes;
+};
+
 /** Submit buffered AIO requests on the given segment to the kernel. */
 void os_aio_dispatch_read_array_submit();
 
@@ -2057,4 +2110,3 @@ void os_aio_dispatch_read_array_submit();
 #endif /* UNIV_NONINL */
 
 #endif /* os0file_h */
-
