@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1997, 2025, Oracle and/or its affiliates.
+Copyright (c) 1997, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -67,12 +67,6 @@ class MVCC {
   @param own_mutex      true if caller owns trx_sys_t::mutex */
   void view_close(ReadView *&view, bool own_mutex);
 
-  /**
-  Release a view that is inactive but not closed. Caller must own
-  the trx_sys_t::mutex.
-  @param view           View to release */
-  void view_release(ReadView *&view);
-
   /** Clones the oldest view and stores it in view. No need to
   call view_close(). The caller owns the view that is passed in.
   It will also move the closed views from the m_views list to the
@@ -124,7 +118,6 @@ class MVCC {
   marked read views from the views list to the freed list.
   @return oldest view if found or NULL */
   ReadView *get_oldest_view() const;
-  ReadView *get_view_created_by_trx_id(trx_id_t trx_id) const;
 
  private:
   // Prevent copying
