@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2000, 2025, Oracle and/or its affiliates.
+Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -111,7 +111,8 @@ void row_mysql_prebuilt_free_compress_heap(row_prebuilt_t *prebuilt) noexcept;
 @param[in,out]	len	in: data length, out: length of decomprssed data
 @param[in]	dict_data	optional dictionary data used for decompression
 @param[in]	dict_data_len	optional dictionary data length
-@param[in]	compress_heap
+@param[in]	compress_heap   memory heap used to compress/decompress
+                                blob column
 @return pointer to the uncompressed data */
 [[nodiscard]]
 const byte *row_decompress_column(const byte *data, ulint *len,
@@ -124,7 +125,8 @@ const byte *row_decompress_column(const byte *data, ulint *len,
 @param[in]	lenlen	bytes used to store the length of data
 @param[in]	dict_data	optional dictionary data used for compression
 @param[in]	dict_data_len	optional dictionary data length
-@param[in]	compress_heap
+@param[in]	compress_heap   memory heap used to compress/decompress
+                                blob column
 @return pointer to the compressed data */
 [[nodiscard]]
 byte *row_compress_column(const byte *data, ulint *len, ulint lenlen,
@@ -160,7 +162,9 @@ remember also to set the null bit in the mysql record header!
 @param[in] need_decompression If the data need to be compressed
 @param[in] dict_data Optional compression dictionary
 @param[in] dict_data_len Optional compression dictionary data
-@param[in] compress_heap */
+@param[in] compress_heap Memory heap used to compress/decompress
+                         blob column
+*/
 void row_mysql_store_blob_ref(byte *dest, ulint col_len, const void *data,
                               ulint len, bool need_decompression,
                               const byte *dict_data, ulint dict_data_len,
@@ -173,7 +177,8 @@ void row_mysql_store_blob_ref(byte *dest, ulint col_len, const void *data,
 @param[in] need_compression     if the data need to be compressed
 @param[in] dict_data            optional compression dictionary data
 @param[in] dict_data_len        optional compression dictionary data length
-@param[in] compress_heap
+@param[in] compress_heap        memory heap used to compress/decompress
+                                blob column
 @return pointer to BLOB data */
 const byte *row_mysql_read_blob_ref(ulint *len, const byte *ref, ulint col_len,
                                     bool need_compression,
@@ -1001,28 +1006,35 @@ struct SysIndexCallback {
   virtual void operator()(mtr_t *mtr, btr_pcur_t *pcur) noexcept = 0;
 };
 
+/** Get the updated parent field value from the update vector for the
+given col_no.
+@param[in]      foreign         foreign key information
+@param[in]      update          updated parent vector.
+@param[in]      col_no          base column position of the child table to check
+@return updated field from the parent update vector, else NULL */
+dfield_t *innobase_get_field_from_update_vector(dict_foreign_t *foreign,
+                                                upd_t *update, uint32_t col_no);
+
 /** Get the computed value by supplying the base column values.
 @param[in,out]  row             the data row
 @param[in]      col             virtual column
-@param[in]      table           the table on which the virtual column is
-                                defined
+@param[in]      index           index on the virtual column
 @param[in,out]  local_heap      heap memory for processing large data etc.
 @param[in,out]  heap            memory heap that copies the actual index row
+@param[in]      ifield          index field
 @param[in]      thd             MySQL thread handle
 @param[in,out]  mysql_table     mysql table object
-@param[in]      ifield          metadata for secondary index field into which
-                                the computed value is to be materialized,
-                                nullptr if not applicable
 @param[in]      old_table       during ALTER TABLE, this is the old table
-                                or nullptr.
-@param[in]      row_update      update vector for the row, supersedes row
-                                values
-@return the field filled with computed value, or nullptr on failure */
+                                or NULL.
+@param[in]      parent_update   update vector for the parent row
+@param[in]      foreign         foreign key information
+@return the field filled with computed value, or NULL if just want
+to store the value in passed in "my_rec" */
 dfield_t *innobase_get_computed_value(
-    const dtuple_t *row, const dict_v_col_t *col, const dict_table_t *table,
-    mem_heap_t **local_heap, mem_heap_t *heap, THD *thd, TABLE *mysql_table,
-    mem_heap_t **compress_heap, const dict_field_t *ifield = nullptr,
-    const dict_table_t *old_table = nullptr, upd_t *row_update = nullptr);
+    const dtuple_t *row, const dict_v_col_t *col, const dict_index_t *index,
+    mem_heap_t **local_heap, mem_heap_t *heap, const dict_field_t *ifield,
+    THD *thd, TABLE *mysql_table, const dict_table_t *old_table,
+    upd_t *parent_update, dict_foreign_t *foreign, mem_heap_t **compress_heap);
 
 /** Parse out multi-values from a MySQL record
 @param[in]      mysql_table     MySQL table structure
