@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2025, Oracle and/or its affiliates.
+Copyright (c) 1995, 2024, Oracle and/or its affiliates.
 Copyright (c) 2008, 2009, Google Inc.
 Copyright (c) 2009, Percona Inc.
 
@@ -254,12 +254,6 @@ struct Srv_threads {
   same shared state as m_page_cleaner_coordinator. */
   IB_thread *m_page_cleaner_workers;
 
-  /** Number of LRU manager threads and size of array below. */
-  size_t m_lru_managers_n;
-
-  /** LRU manager threads. */
-  IB_thread *m_lru_managers;
-
   /** Archiver's log archiver (used by Clone). */
   IB_thread m_log_archiver;
 
@@ -657,14 +651,6 @@ extern ulint srv_show_locks_held;
 
 extern ulint srv_lock_table_size;
 
-extern ulint srv_cleaner_max_lru_time; /*!< the maximum time limit for a
-                                      single LRU tail flush iteration by the
-                                      page cleaner thread */
-
-extern ulint srv_cleaner_max_flush_time; /*!< the maximum time limit for a
-                                      single flush list flush iteration by
-                                      the page cleaner thread */
-
 extern ulong srv_cleaner_lsn_age_factor;
 /*!< page cleaner LSN age factor
 formula option */
@@ -911,7 +897,6 @@ extern srv_stats_t srv_stats;
 extern mysql_pfs_key_t log_archiver_thread_key;
 extern mysql_pfs_key_t page_archiver_thread_key;
 extern mysql_pfs_key_t buf_dump_thread_key;
-extern mysql_pfs_key_t buf_lru_manager_thread_key;
 extern mysql_pfs_key_t buf_resize_thread_key;
 extern mysql_pfs_key_t clone_ddl_thread_key;
 extern mysql_pfs_key_t clone_gtid_thread_key;
@@ -931,6 +916,7 @@ extern mysql_pfs_key_t log_write_notifier_thread_key;
 extern mysql_pfs_key_t log_flush_notifier_thread_key;
 extern mysql_pfs_key_t page_flush_coordinator_thread_key;
 extern mysql_pfs_key_t page_flush_thread_key;
+extern mysql_pfs_key_t recv_writer_thread_key;
 extern mysql_pfs_key_t srv_error_monitor_thread_key;
 extern mysql_pfs_key_t srv_lock_timeout_thread_key;
 extern mysql_pfs_key_t srv_master_thread_key;
@@ -941,6 +927,8 @@ extern mysql_pfs_key_t trx_recovery_rollback_thread_key;
 extern mysql_pfs_key_t srv_ts_alter_encrypt_thread_key;
 extern mysql_pfs_key_t parallel_read_thread_key;
 extern mysql_pfs_key_t parallel_rseg_init_thread_key;
+extern mysql_pfs_key_t bulk_flusher_thread_key;
+extern mysql_pfs_key_t bulk_alloc_thread_key;
 #endif /* UNIV_PFS_THREAD */
 #endif /* !UNIV_HOTBACKUP */
 
@@ -994,25 +982,23 @@ extern PSI_stage_info srv_stage_clone_page_copy;
 /** Alternatives for the file flush option in Unix.
 @see innodb_flush_method_names */
 enum srv_unix_flush_t {
-  SRV_UNIX_FSYNC = 0,  /*!< fsync, the default */
-  SRV_UNIX_O_DSYNC,    /*!< open log files in O_SYNC mode */
-  SRV_UNIX_LITTLESYNC, /*!< do not call os_file_flush()
-                       when writing data files, but do flush
-                       after writing to log files */
-  SRV_UNIX_NOSYNC,     /*!< do not flush after writing */
-  SRV_UNIX_O_DIRECT,   /*!< invoke os_file_set_nocache() on
-                       data files. This implies using
-                       non-buffered IO but still using fsync,
-                       the reason for which is that some FS
-                       do not flush meta-data when
-                       unbuffered IO happens */
+  /** fsync, the default if O_DIRECT is not supported */
+  SRV_UNIX_FSYNC = 0,
+  /** open log files in O_SYNC mode */
+  SRV_UNIX_O_DSYNC,
+  /** do not call os_file_flush() when writing data files, but do flush after
+     writing to log files */
+  SRV_UNIX_LITTLESYNC,
+  /** do not flush after writing */
+  SRV_UNIX_NOSYNC,
+  /** invoke os_file_set_nocache() on data files. This implies using
+     non-buffered IO but still using fsync, the reason for which is that some FS
+     do not flush meta-data when unbuffered IO happens. Default if supported */
+  SRV_UNIX_O_DIRECT,
+  /** do not use fsync() when using direct IO i.e.: it can be set to avoid the
+     fsync() call that we make when using SRV_UNIX_O_DIRECT. However, in this
+     case user/DBA should be sure about the integrity of the meta-data */
   SRV_UNIX_O_DIRECT_NO_FSYNC
-  /*!< do not use fsync() when using
-  direct IO i.e.: it can be set to avoid
-  the fsync() call that we make when
-  using SRV_UNIX_O_DIRECT. However, in
-  this case user/DBA should be sure about
-  the integrity of the meta-data */
 };
 extern enum srv_unix_flush_t srv_unix_file_flush_method;
 
