@@ -215,6 +215,8 @@ enum latch_level_t {
 
   SYNC_ANY_LATCH,
 
+  SYNC_ALTER_STAGE,
+
   SYNC_FIL_SHARD,
 
   SYNC_PAGE_ARCH_OPER,
@@ -251,6 +253,7 @@ enum latch_level_t {
   SYNC_LOG_CHECKPOINTER,
   SYNC_LOG_SN,
   SYNC_LOG_SN_MUTEX,
+  SYNC_LOG_GOVERNOR_MUTEX,
   SYNC_PAGE_ARCH,
   SYNC_PAGE_ARCH_CLIENT,
   SYNC_LOG_ARCH,
@@ -323,6 +326,8 @@ enum latch_level_t {
 
   SYNC_TRX_I_S_RWLOCK,
 
+  SYNC_RECV_WRITER,
+
   /** Level is varying. Only used with buffer pool page locks, which
   do not have a fixed level, but instead have their level set after
   the page is locked; see e.g.  ibuf_bitmap_get_map_page(). */
@@ -342,6 +347,7 @@ alphabetical ordering seems useful */
 enum latch_id_t {
   LATCH_ID_NONE = 0,
   LATCH_ID_AHI_ENABLED,
+  LATCH_ID_ALTER_STAGE,
   LATCH_ID_AUTOINC,
   LATCH_ID_BUF_BLOCK_MUTEX,
   LATCH_ID_BUF_POOL_CHUNKS,
@@ -385,6 +391,7 @@ enum latch_id_t {
   LATCH_ID_LOG_LIMITS,
   LATCH_ID_LOG_ONLINE,
   LATCH_ID_LOG_FILES,
+  LATCH_ID_LOG_GOVERNOR_MUTEX,
   LATCH_ID_PARSER,
   LATCH_ID_LOG_ARCH,
   LATCH_ID_PAGE_ARCH,
@@ -1127,7 +1134,8 @@ struct dict_sync_check : public sync_check_functor_t {
     if (!m_dict_mutex_allowed ||
         (level != SYNC_DICT && level != SYNC_UNDO_SPACES &&
          level != SYNC_FTS_CACHE && level != SYNC_DICT_OPERATION &&
-         level != SYNC_NO_ORDER_CHECK)) {
+         /* This only happens in recv_apply_hashed_log_recs. */
+         level != SYNC_RECV_WRITER && level != SYNC_NO_ORDER_CHECK)) {
       m_result = true;
 #ifdef UNIV_NO_ERR_MSGS
       ib::error()
