@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2025, Oracle and/or its affiliates.
+Copyright (c) 1996, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -1875,6 +1875,10 @@ bool trx_is_prepared_in_tc(trx_t const *trx);
 @return DB_SUCCESS or error number */
 dberr_t trx_set_prepared_in_tc_for_mysql(trx_t *trx);
 
+/** Human readable transaction state, for diagnostic purposes.
+@return pointer to static string, nullptr if state is not valid */
+inline const char *trx_state_string(trx_state_t state);
+
 #include "trx0trx.ic"
 #endif /* !UNIV_HOTBACKUP */
 
