@@ -35,12 +35,14 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 #include <string>
 #include <vector>
 
+#include "m_string.h"
 #include "mysql/components/services/dynamic_privilege.h"
 #include "mysql/components/services/udf_registration.h"
 #include "mysql/plugin.h"
 #include "mysql/service_plugin_registry.h"
 #include "sql/mysqld.h"
 #include "sql/sql_class.h"
+#include "string_with_len.h"
 
 #include "db0err.h"
 #include "dict0dd.h"
@@ -576,16 +578,16 @@ long long innodb_redo_log_sharp_checkpoint(
     [[maybe_unused]] unsigned char *error);
 bool innodb_redo_log_consumer_register_init([[maybe_unused]] UDF_INIT *initid,
                                             UDF_ARGS *args, char *message);
-void innodb_redo_log_consumer_register_deinit([
-    [maybe_unused]] UDF_INIT *initid);
+void innodb_redo_log_consumer_register_deinit(
+    [[maybe_unused]] UDF_INIT *initid);
 long long innodb_redo_log_consumer_register(
     [[maybe_unused]] UDF_INIT *initid, [[maybe_unused]] UDF_ARGS *args,
     [[maybe_unused]] unsigned char *null_value,
     [[maybe_unused]] unsigned char *error);
 bool innodb_redo_log_consumer_unregister_init([[maybe_unused]] UDF_INIT *initid,
                                               UDF_ARGS *args, char *message);
-void innodb_redo_log_consumer_unregister_deinit([
-    [maybe_unused]] UDF_INIT *initid);
+void innodb_redo_log_consumer_unregister_deinit(
+    [[maybe_unused]] UDF_INIT *initid);
 long long innodb_redo_log_consumer_unregister(
     [[maybe_unused]] UDF_INIT *initid, [[maybe_unused]] UDF_ARGS *args,
     [[maybe_unused]] unsigned char *null_value,
@@ -2251,8 +2253,8 @@ bool innodb_redo_log_sharp_checkpoint_init([[maybe_unused]] UDF_INIT *initid,
 
   See include/mysql/udf_registration_types.h
 */
-void innodb_redo_log_sharp_checkpoint_deinit([
-    [maybe_unused]] UDF_INIT *initid) {
+void innodb_redo_log_sharp_checkpoint_deinit(
+    [[maybe_unused]] UDF_INIT *initid) {
   return;
 }
 
@@ -2306,8 +2308,8 @@ bool innodb_redo_log_consumer_register_init([[maybe_unused]] UDF_INIT *initid,
 
   See include/mysql/udf_registration_types.h
 */
-void innodb_redo_log_consumer_register_deinit([
-    [maybe_unused]] UDF_INIT *initid) {}
+void innodb_redo_log_consumer_register_deinit(
+    [[maybe_unused]] UDF_INIT *initid) {}
 
 /**
   UDF innodb_redo_log_consumer_register
@@ -2356,8 +2358,8 @@ bool innodb_redo_log_consumer_unregister_init([[maybe_unused]] UDF_INIT *initid,
 
   See include/mysql/udf_registration_types.h
 */
-void innodb_redo_log_consumer_unregister_deinit([
-    [maybe_unused]] UDF_INIT *initid) {}
+void innodb_redo_log_consumer_unregister_deinit(
+    [[maybe_unused]] UDF_INIT *initid) {}
 
 /**
   UDF innodb_redo_log_consumer_unregister
@@ -2403,8 +2405,8 @@ bool innodb_redo_log_consumer_advance_init([[maybe_unused]] UDF_INIT *initid,
 
   See include/mysql/udf_registration_types.h
 */
-void innodb_redo_log_consumer_advance_deinit([
-    [maybe_unused]] UDF_INIT *initid) {}
+void innodb_redo_log_consumer_advance_deinit(
+    [[maybe_unused]] UDF_INIT *initid) {}
 
 /**
   UDF innodb_redo_log_consumer_advance
@@ -2424,8 +2426,7 @@ long long innodb_redo_log_consumer_advance(
     [[maybe_unused]] UDF_INIT *initid, UDF_ARGS *args,
     [[maybe_unused]] unsigned char *null_value,
     [[maybe_unused]] unsigned char *error) {
-  if (current_thd == nullptr ||
-      verify_privilege(current_thd, backup_admin_privilege)) {
+  if (current_thd == nullptr) {
     return 1;
   }
   return static_cast<long long>(meb::redo_log_consumer_advance(
