@@ -144,7 +144,7 @@ no-op on other systems.
 @param[in]      relative_priority       system-specific priority value
 
 @return An actual thread priority after the update */
-MY_NODISCARD
+[[nodiscard]]
 ulint os_thread_set_priority(os_tid_t thread_id,
                              ulint relative_priority) noexcept {
 #ifdef UNIV_LINUX
