@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2005, 2025, Oracle and/or its affiliates.
+Copyright (c) 2005, 2024, Oracle and/or its affiliates.
 Copyright (c) 2012, Facebook Inc.
 
 This program is free software; you can redistribute it and/or modify it under
@@ -1829,9 +1829,9 @@ void page_zip_write_rec(
 
 /** Parses a log record of writing a BLOB pointer of a record.
  @return end of log record or NULL */
-byte *page_zip_parse_write_blob_ptr(
-    byte *ptr,                /*!< in: redo log buffer */
-    byte *end_ptr,            /*!< in: redo log buffer end */
+const byte *page_zip_parse_write_blob_ptr(
+    const byte *ptr,          /*!< in: redo log buffer */
+    const byte *end_ptr,      /*!< in: redo log buffer end */
     page_t *page,             /*!< in/out: uncompressed page */
     page_zip_des_t *page_zip) /*!< in/out: compressed page */
 {
@@ -1960,9 +1960,9 @@ void page_zip_write_blob_ptr(
 
 /** Parses a log record of writing the node pointer of a record.
  @return end of log record or NULL */
-byte *page_zip_parse_write_node_ptr(
-    byte *ptr,                /*!< in: redo log buffer */
-    byte *end_ptr,            /*!< in: redo log buffer end */
+const byte *page_zip_parse_write_node_ptr(
+    const byte *ptr,          /*!< in: redo log buffer */
+    const byte *end_ptr,      /*!< in: redo log buffer end */
     page_t *page,             /*!< in/out: uncompressed page */
     page_zip_des_t *page_zip) /*!< in/out: compressed page */
 {
@@ -2455,9 +2455,9 @@ void page_zip_dir_add_slot(page_zip_des_t *page_zip, bool is_clustered) {
 
 /** Parses a log record of writing to the header of a page.
  @return end of log record or NULL */
-byte *page_zip_parse_write_header(
-    byte *ptr,                /*!< in: redo log buffer */
-    byte *end_ptr,            /*!< in: redo log buffer end */
+const byte *page_zip_parse_write_header(
+    const byte *ptr,          /*!< in: redo log buffer */
+    const byte *end_ptr,      /*!< in: redo log buffer end */
     page_t *page,             /*!< in/out: uncompressed page */
     page_zip_des_t *page_zip) /*!< in/out: compressed page */
 {
@@ -2716,9 +2716,9 @@ void page_zip_copy_recs(
 
 /** Parses a log record of compressing an index page.
  @return end of log record or NULL */
-byte *page_zip_parse_compress(
-    byte *ptr,                /*!< in: buffer */
-    byte *end_ptr,            /*!< in: buffer end */
+const byte *page_zip_parse_compress(
+    const byte *ptr,          /*!< in: buffer */
+    const byte *end_ptr,      /*!< in: buffer end */
     page_t *page,             /*!< out: uncompressed page */
     page_zip_des_t *page_zip) /*!< out: compressed page */
 {
