@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1994, 2025, Oracle and/or its affiliates.
+Copyright (c) 1994, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -829,7 +829,9 @@ static inline Rec_instant_state rec_convert_dtuple_to_rec_comp(
         }
         break;
       case REC_STATUS_NODE_PTR:
-        ut_ad(n_fields == dict_index_get_n_unique_in_tree_nonleaf(index) + 1U);
+        ut_ad(n_fields ==
+              static_cast<ulint>(
+                  dict_index_get_n_unique_in_tree_nonleaf(index) + 1));
         n_node_ptr_field = n_fields - 1;
         n_null = index->get_nullable_before_instant_add_drop();
         break;
@@ -912,6 +914,7 @@ static inline Rec_instant_state rec_convert_dtuple_to_rec_comp(
         ut_ad(!dfield_is_ext(field));
 #endif /* UNIV_DEBUG */
       } else if (dfield_is_ext(field)) {
+        ut_ad(index->is_clustered());
         ut_ad(DATA_BIG_COL(col));
         ut_ad(len <=
               REC_ANTELOPE_MAX_INDEX_COL_LEN + BTR_EXTERN_FIELD_REF_SIZE);
@@ -2044,4 +2047,3 @@ err_out:
   return DB_ERROR;
 }
 #endif /* WITH_WSREP */
-
