@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -31,7 +31,6 @@
 #include <string.h>
 #include <algorithm>
 
-#include "m_ctype.h"
 #include "m_string.h"
 #include "my_bitmap.h"
 #include "my_byteorder.h"
@@ -42,6 +41,7 @@
 #include "myisampack.h"
 #include "mysql/components/services/bits/psi_bits.h"
 #include "mysql/components/services/bits/psi_mutex_bits.h"
+#include "mysql/strings/m_ctype.h"
 #include "sql/auth/auth_acls.h"
 #include "sql/current_thd.h"
 #include "sql/field.h"
@@ -135,6 +135,8 @@
 #include "storage/perfschema/table_setup_actors.h"
 #include "storage/perfschema/table_setup_consumers.h"
 #include "storage/perfschema/table_setup_instruments.h"
+#include "storage/perfschema/table_setup_meters.h"
+#include "storage/perfschema/table_setup_metrics.h"
 #include "storage/perfschema/table_setup_objects.h"
 #include "storage/perfschema/table_setup_threads.h"
 #include "storage/perfschema/table_socket_instances.h"
@@ -509,6 +511,8 @@ static PFS_engine_table_share *all_shares[] = {
     &table_setup_actors::m_share,
     &table_setup_consumers::m_share,
     &table_setup_instruments::m_share,
+    &table_setup_meters::m_share,
+    &table_setup_metrics::m_share,
     &table_setup_objects::m_share,
     &table_setup_threads::m_share,
     &table_tiws_by_index_usage::m_share,
@@ -854,6 +858,7 @@ int PFS_engine_table::index_read(KEY *key_infos, uint index, const uchar *key,
   assert(find_flag != HA_READ_MBR_WITHIN);
   assert(find_flag != HA_READ_MBR_DISJOINT);
   assert(find_flag != HA_READ_MBR_EQUAL);
+  assert(find_flag != HA_READ_NEAREST_NEIGHBOR);
 
   KEY *key_info = key_infos + index;
   m_index->set_key_info(key_info);
