@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -30,10 +30,10 @@
 
 #include <assert.h>
 #include "lex_string.h"
-#include "m_string.h"
 #include "my_compiler.h"
 
 #include "my_thread.h"
+#include "mysql/strings/int2str.h"
 #include "sql/field.h"
 #include "sql/plugin_table.h"
 #include "sql/table.h"
@@ -43,6 +43,7 @@
 #include "storage/perfschema/pfs_instr.h"
 #include "storage/perfschema/pfs_instr_class.h"
 #include "storage/perfschema/pfs_timer.h"
+#include "string_with_len.h"
 
 bool PFS_index_events_waits::match(PFS_thread *pfs) {
   if (m_fields >= 1) {
