@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -36,6 +36,7 @@
 
 #include "my_compiler.h"
 
+#include "m_string.h"
 #include "my_macros.h"
 #include "my_thread.h"
 #include "sql-common/json_dom.h"
@@ -54,6 +55,8 @@
 #include "storage/perfschema/pfs_user.h"
 #include "storage/perfschema/pfs_variable.h"
 
+struct CHARSET_INFO;
+
 /* TINYINT TYPE */
 void set_field_tiny(Field *f, long value) {
   assert(f->real_type() == MYSQL_TYPE_TINY);
@@ -2078,6 +2081,22 @@ bool PFS_key_event_name::match_view(uint view) {
   }
 }
 
+bool PFS_key_meter_name::match(PFS_meter_class *pfs) {
+  PFS_meter_class *safe_class = sanitize_meter_class(pfs);
+  if (unlikely(safe_class == nullptr)) {
+    return false;
+  }
+  return do_match(false, safe_class->m_meter, safe_class->m_meter_length);
+}
+
+bool PFS_key_metric_name::match(PFS_metric_class *pfs) {
+  PFS_metric_class *safe_class = sanitize_metric_class(pfs);
+  if (unlikely(safe_class == nullptr)) {
+    return false;
+  }
+  return do_match(false, safe_class->m_metric, safe_class->m_metric_length);
+}
+
 bool PFS_key_user::match(const PFS_thread *pfs) {
   const bool record_null = (pfs->m_user_name.length() == 0);
   return do_match(record_null, pfs->m_user_name.ptr(),
