@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -48,6 +48,7 @@
 #include "storage/perfschema/pfs_stat.h"
 #include "storage/perfschema/pfs_timer.h"
 
+struct CHARSET_INFO;
 struct PFS_host;
 struct PFS_user;
 struct PFS_account;
@@ -1488,6 +1489,24 @@ class PFS_key_event_name : public PFS_key_string<PFS_MAX_INFO_NAME_LENGTH> {
   bool match_view(uint view);
 };
 
+class PFS_key_meter_name : public PFS_key_string<PFS_MAX_INFO_NAME_LENGTH> {
+ public:
+  explicit PFS_key_meter_name(const char *name) : PFS_key_string(name) {}
+
+  ~PFS_key_meter_name() override = default;
+
+  bool match(PFS_meter_class *pfs);
+};
+
+class PFS_key_metric_name : public PFS_key_string<PFS_MAX_INFO_NAME_LENGTH> {
+ public:
+  explicit PFS_key_metric_name(const char *name) : PFS_key_string(name) {}
+
+  ~PFS_key_metric_name() override = default;
+
+  bool match(PFS_metric_class *pfs);
+};
+
 class PFS_key_user : public PFS_key_string<USERNAME_LENGTH> {
  public:
   explicit PFS_key_user(const char *name) : PFS_key_string(name) {}
