@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -29,7 +29,9 @@
 #include <stddef.h>
 #include <sys/types.h>
 
-#include "m_ctype.h"
+#include "mysql/strings/m_ctype.h"
+#include "sql/current_thd.h"
+#include "sql/debug_sync.h"
 #include "sql/mysqld.h"
 #include "sql/sql_class.h"
 #include "sql/sql_show.h"
@@ -84,3 +86,9 @@ bool acl_is_utility_user(const char *, const char *, const char *) {
 }
 
 void reset_status_by_thd() {}
+
+thread_local THD *current_thd;
+
+uint opt_debug_sync_timeout = 0;
+
+void debug_sync(THD *, const char *, size_t) {}
