@@ -17,8 +17,10 @@
 #include "sql_dd.h"
 #include <assert.h>
 #include <memory>
+#include "./rdb_utils.h"
+#include "my_sys.h"
+#include "mysql/psi/psi_memory.h"
 #include "mysql/service_mysql_alloc.h"
-#include "sql/field.h"
 
 namespace myrocks {
 
@@ -172,7 +174,6 @@ void dd_copy_table_columns(dd::Table &new_table, const dd::Table &old_table) {
 /** Copy the engine-private parts of a table or partition definition
 when the change does not affect RocksDB. This mainly copies the common
 private data between dd::Table
-@tparam		Table		dd::Table
 @param[in,out]	new_table	Copy of old table or partition definition
 @param[in]	old_table	Old table or partition definition */
 void dd_copy_private(dd::Table &new_table, const dd::Table &old_table) {
