@@ -1,4 +1,4 @@
-/* Copyright (c) 2016, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -190,10 +190,8 @@ class TableResourceMonitor {
  */
 template <typename Block_size_policy, typename Block_source_policy>
 struct Allocation_scheme {
-  static Source block_source(size_t block_size,
-                             TableResourceMonitor *table_resource_monitor) {
-    return Block_source_policy::block_source(block_size,
-                                             table_resource_monitor);
+  static Source block_source(size_t block_size) {
+    return Block_source_policy::block_source(block_size);
   }
   static size_t block_size(size_t number_of_blocks, size_t n_bytes_requested) {
     return Block_size_policy::block_size(number_of_blocks, n_bytes_requested);
@@ -216,8 +214,7 @@ struct Allocation_scheme {
  *     tmp_table_size SYSVAR.
  * */
 struct Prefer_RAM_over_MMAP_policy {
-  static Source block_source(uint32_t block_size,
-                             TableResourceMonitor * = nullptr) {
+  static Source block_source(uint32_t block_size) {
     if (MemoryMonitor::RAM::consumption() < MemoryMonitor::RAM::threshold()) {
       if (MemoryMonitor::RAM::increase(block_size) <=
           MemoryMonitor::RAM::threshold()) {
@@ -246,48 +243,6 @@ struct Prefer_RAM_over_MMAP_policy {
   }
 };
 
-/* Another concrete implementation of Block_source_policy, a type which controls
- * where TempTable allocator is going to be allocating next Block of memory
- * from. It acts the same as Prefer_RAM_over_MMAP_policy with the main
- * difference being that this policy obeys the per-table limit.
- *
- * What this means is that each temptable::Table is allowed to fit no more data
- * than the given threshold controlled through TableResourceMonitor abstraction.
- * TableResourceMonitor is a simple abstraction which is in its part an alias
- * for tmp_table_size, a system variable that end MySQL users will be using to
- * control this threshold.
- *
- * Updating the tmp_table_size threshold can only be done through the separate
- * SET statement which implies that the tmp_table_size threshold cannot be
- * updated during the duration of some query which is running within the same
- * session. Separate sessions can still of course change this value to their
- * liking.
- * */
-struct Prefer_RAM_over_MMAP_policy_obeying_per_table_limit {
-  static Source block_source(uint32_t block_size,
-                             TableResourceMonitor *table_resource_monitor) {
-    assert(table_resource_monitor);
-    assert(table_resource_monitor->consumption() <=
-           std::max(table_resource_monitor->threshold(), 1024ul * 1024ul));
-
-    if (table_resource_monitor->consumption() + block_size >
-        table_resource_monitor->threshold()) {
-      if (table_resource_monitor->consumption() == 0) {
-        // Always allow to allocate at least one block, otherwise the temptable
-        // engine is unusable
-        return Prefer_RAM_over_MMAP_policy::block_source(block_size);
-      }
-      throw Result::RECORD_FILE_FULL;
-    }
-
-    return Prefer_RAM_over_MMAP_policy::block_source(block_size);
-  }
-
-  static void block_freed(uint32_t block_size, Source block_source) {
-    Prefer_RAM_over_MMAP_policy::block_freed(block_size, block_source);
-  }
-};
-
 /* Concrete implementation of Block_size_policy, a type which controls how big
  * next Block of memory is going to be allocated by TempTable allocator.
  *
@@ -336,8 +291,7 @@ struct Exponential_policy {
  * over MMAP allocations.
  */
 using Exponential_growth_preferring_RAM_over_MMAP =
-    Allocation_scheme<Exponential_policy,
-                      Prefer_RAM_over_MMAP_policy_obeying_per_table_limit>;
+    Allocation_scheme<Exponential_policy, Prefer_RAM_over_MMAP_policy>;
 
 /**
   Shared state between all instances of a given allocator.
@@ -379,8 +333,7 @@ class AllocatorState {
    * small.
    * [in] Number of bytes that will be allocated from the returned block.
    */
-  Block *get_block_for_new_allocation(
-      size_t n_bytes_requested, TableResourceMonitor *table_resource_monitor) {
+  Block *get_block_for_new_allocation(size_t n_bytes_requested) {
     if (current_block.is_empty() ||
         !current_block.can_accommodate(n_bytes_requested)) {
       /* The current_block may have been left empty during some deallocate()
@@ -394,9 +347,8 @@ class AllocatorState {
 
       const size_t block_size =
           AllocationScheme::block_size(number_of_blocks, n_bytes_requested);
-      current_block = Block(
-          block_size,
-          AllocationScheme::block_source(block_size, table_resource_monitor));
+      current_block =
+          Block(block_size, AllocationScheme::block_source(block_size));
       ++number_of_blocks;
     }
     return &current_block;
@@ -676,18 +628,32 @@ inline T *Allocator<T, AllocationScheme>::allocate(size_t n_elements) {
   if (m_shared_block && m_shared_block->is_empty()) {
     const size_t block_size =
         AllocationScheme::block_size(0, n_bytes_requested);
-    *m_shared_block = Block(
-        block_size,
-        AllocationScheme::block_source(block_size, &m_table_resource_monitor));
+    *m_shared_block =
+        Block(block_size, AllocationScheme::block_source(block_size));
     block = m_shared_block;
   } else if (m_shared_block &&
              m_shared_block->can_accommodate(n_bytes_requested)) {
     block = m_shared_block;
   } else {
-    block = m_state->get_block_for_new_allocation(n_bytes_requested,
-                                                  &m_table_resource_monitor);
+    block = m_state->get_block_for_new_allocation(n_bytes_requested);
   }
 
+  /* temptable::Table is allowed to fit no more data than the given threshold
+   * controlled through TableResourceMonitor abstraction. TableResourceMonitor
+   * is a simple abstraction which is in its part an alias for tmp_table_size, a
+   * system variable that end MySQL users will be using to control this
+   * threshold.
+   *
+   * Updating the tmp_table_size threshold can only be done through the separate
+   * SET statement which implies that the tmp_table_size threshold cannot be
+   * updated during the duration of some query which is running within the same
+   * session. Separate sessions can still of course change this value to their
+   * liking.
+   */
+  if (m_table_resource_monitor.consumption() + n_bytes_requested >
+      m_table_resource_monitor.threshold()) {
+    throw Result::RECORD_FILE_FULL;
+  }
   m_table_resource_monitor.increase(n_bytes_requested);
 
   T *chunk_data =
