@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018-2025 Codership Oy <info@codership.com>
+ * Copyright (C) 2018 Codership Oy <info@codership.com>
  *
  * This file is part of wsrep-lib.
  *
@@ -31,8 +31,6 @@
 
 #include <cstring>
 
-#include <functional>
-#include <memory>
 #include <string>
 #include <vector>
 #include <ostream>
@@ -51,9 +49,7 @@ namespace wsrep
     class tls_service;
     class allowlist_service;
     class event_service;
-    class client_service;
-    class connection_monitor_service;
-    class provider_options;
+
     class stid
     {
     public:
@@ -289,6 +285,7 @@ namespace wsrep
             static const int streaming = (1 << 15);
             static const int snapshot = (1 << 16);
             static const int nbo = (1 << 17);
+
             /** decipher capability bitmask */
             static std::string str(int);
         };
@@ -337,37 +334,10 @@ namespace wsrep
         virtual int append_key(wsrep::ws_handle&, const wsrep::key&) = 0;
         virtual enum status append_data(
             wsrep::ws_handle&, const wsrep::const_buffer&) = 0;
-
-        /**
-         * Callback for application defined sequential consistency.
-         * The provider will call
-         * the callback once it can guarantee sequential consistency. */
-        typedef struct seq_cb {
-            /** Opaque caller context */
-            void *ctx;
-            /** Function to be called by the provider when sequential
-             * consistency is guaranteed. */
-            void (*fn)(void *ctx);
-        } seq_cb_t;
-
-        /**
-         * Certify the write set.
-         *
-         * @param client_id[in] Id of the client session.
-         * @param ws_handle[in,out] Write set handle associated to the current
-         *                 transaction.
-         * @param flags[in] Flags associated to the write set (see struct flag).
-         * @param ws_meta[out] Write set meta data associated to the
-         *                     replicated write set.
-         * @param seq_cb[in] Optional callback for application defined
-         *                   sequential consistency.
-         *
-         * @return Status code defined in struct status.
-         */
         virtual enum status
-        certify(wsrep::client_id client_id, wsrep::ws_handle& ws_handle,
-                int flags, wsrep::ws_meta& ws_meta, const seq_cb_t* seq_cb)
-            = 0;
+        certify(wsrep::client_id, wsrep::ws_handle&,
+                int,
+                wsrep::ws_meta&) = 0;
         /**
          * BF abort a transaction inside provider.
          *
@@ -380,7 +350,6 @@ namespace wsrep
          */
         virtual enum status bf_abort(wsrep::seqno bf_seqno,
                                      wsrep::transaction_id victim_trx,
-                                     wsrep::client_service& victim_ctx,
                                      wsrep::seqno& victim_seqno) = 0;
         virtual enum status rollback(wsrep::transaction_id) = 0;
         virtual enum status commit_order_enter(const wsrep::ws_handle&,
@@ -413,7 +382,6 @@ namespace wsrep
          * Leave total order isolation critical section
          */
         virtual enum status leave_toi(wsrep::client_id,
-                                      const wsrep::ws_meta& ws_meta,
                                       const wsrep::mutable_buffer& err) = 0;
 
         /**
@@ -474,14 +442,6 @@ namespace wsrep
          */
         virtual void* native() const = 0;
 
-        /**
-         * Get conflicting gtid for the given transaction handle, if any.
-         */
-        virtual wsrep::gtid conflict_gtid(const wsrep::ws_handle&) const
-        {
-            return wsrep::gtid::undefined();
-        }
-
         /**
          * Fetch cluster node information to populate PXC cluster view table.
          */
@@ -502,7 +462,6 @@ namespace wsrep
             wsrep::tls_service* tls_service;
             wsrep::allowlist_service* allowlist_service;
             wsrep::event_service* event_service;
-            wsrep::connection_monitor_service* connection_monitor_service;
 
             // some GCC and clang versions don't support C++11 default
             // initializers fully, so we need to use explicit constructors
@@ -514,20 +473,17 @@ namespace wsrep
                 , tls_service()
                 , allowlist_service()
                 , event_service()
-                , connection_monitor_service()
             {
             }
 
             services(wsrep::thread_service* thr,
                      wsrep::tls_service*    tls,
                      wsrep::allowlist_service* all,
-                     wsrep::event_service*  event,
-                     wsrep::connection_monitor_service* con)
+                     wsrep::event_service*  event)
                 : thread_service(thr)
                 , tls_service(tls)
                 , allowlist_service(all)
                 , event_service(event)
-                , connection_monitor_service(con)
             {
             }
         };
@@ -535,15 +491,14 @@ namespace wsrep
          * Create a new provider.
          *
          * @param provider_spec Provider specification
-         * @param provider_options_cb Callback to get initial provider options
+         * @param provider_options Initial options to provider
          * @param thread_service Optional thread service implementation.
          */
-        static std::unique_ptr<provider> make_provider(
-            wsrep::server_state&, const std::string& provider_spec,
-            const std::function<int(const provider_options&, std::string&)>&
-                provider_options_cb,
-            const wsrep::provider::services& services
-            = wsrep::provider::services());
+        static provider* make_provider(wsrep::server_state&,
+                                       const std::string& provider_spec,
+                                       const std::string& provider_options,
+                                       const wsrep::provider::services& services
+                                       = wsrep::provider::services());
 
     protected:
         wsrep::server_state& server_state_;
